/**************************************************************************

Module Name:

   CSMISASA.H

Abstract:

   This file contains the Adaptec extensions to the Common Storage
   Management Interface.

Revision History:

   001  KDS  2004/11/03  Initial release.
   002  KDS  2004/11/16  Added uControllerFlags, usFlashManufacturerID, and
                         usFlashDeviceID fields to
                         CSMI_SAS_GET_ADPT_CNTLR_CONFIG structure.

**************************************************************************/

#ifndef _CSMI_SAS_A_H_
#define _CSMI_SAS_A_H_

//#include "csmisas.h"
#include "adp94xx_csmi.h"

/*************************************************************************/
/* TARGET OS LINUX SPECIFIC CODE                                         */
/*************************************************************************/

#ifdef _linux

// Control Codes requiring CSMI_ADAPTEC_SIGNATURE

#define CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG       0xCC778001
#define CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES   0xCC778002
#define CC_CSMI_SAS_WRITE_NV_SEGMENT            0xCC778003
#define CC_CSMI_SAS_READ_NV_SEGMENT             0xCC778004

#endif

/*************************************************************************/
/* TARGET OS WINDOWS SPECIFIC CODE                                       */
/*************************************************************************/

#ifdef _WIN32

// Control Codes requiring CSMI_ADAPTEC_SIGNATURE

#define CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG       0x8001
#define CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES   0x8002
#define CC_CSMI_SAS_WRITE_NV_SEGMENT            0x8003
#define CC_CSMI_SAS_READ_NV_SEGMENT             0x8004

#endif

/*************************************************************************/
/* TARGET OS NOT DEFINED ERROR                                           */
/*************************************************************************/

#if (!_WIN32 && !_linux)
   #error "Unknown target OS."
#endif

/*************************************************************************/
/* OS INDEPENDENT CODE                                                   */
/*************************************************************************/

/* * * * * * * * * Adaptec Vendor Unique Class IOCTL Constants * * * * * * * */

// Signature value
// (IoctlHeader.Signature)

#define CSMI_ADAPTEC_SIGNATURE    "CSMIADP"

// Timeout value default of 60 seconds
// (IoctlHeader.Timeout)

#define CSMI_ADAPTEC_TIMEOUT      60

// Controller flags
// (uControllerFlags)
#define CSMI_ADPT_CNTLR_FLASH_NOT_PRESENT          0x00000001
#define CSMI_ADPT_CNTLR_FLASH_READONLY             0x00000002

// Invalid value for usFlashManufacturerID
#define CSMI_SAS_APDT_INVALID_FLASH_MANUFACTURER_ID   0xFFFF

// Invalid value for usFlashDeviceID
#define CSMI_SAS_ADPT_INVALID_FLASH_DEVICE_ID         0xFFFF

// Segment identifiers
// (uSegmentID)
#define CSMI_SAS_SEGMENT_ID_FLASH_DIRECTORY0       0x80000000
#define CSMI_SAS_SEGMENT_ID_CTRL_A_USER_SETTINGS0  0x800000E0
#define CSMI_SAS_SEGMENT_ID_MANUFACTURING_SECTOR0  0x80000120
#define CSMI_SAS_SEGMENT_ID_COMPATIBILITY_SECTOR0  0x80000140

#define CSMI_SAS_SEGMENT_ID_FLASH0                 0x90000000
#define CSMI_SAS_SEGMENT_ID_SEEPROM0               0xA0000000

// NVRAM status
// (uStatus)
#define CSMI_SAS_NV_SUCCESS                  0
#define CSMI_SAS_NV_FAILURE                  1
#define CSMI_SAS_NV_INVALID_SEGMENT_ID       3
#define CSMI_SAS_NV_INVALID_REQUEST          4
#define CSMI_SAS_NV_WRITE_NOT_SUPPORTED      5
#define CSMI_SAS_NV_NO_NVRAM                 6

// NVRAM segment attributes
// (usSegmentAttributes)
#define CSMI_SAS_SEGMENT_ATTRIBUTE_ERASEWRITE   0x0001
#define CSMI_SAS_SEGMENT_ATTRIBUTE_READONLY     0x0002

// Invalid value for uSegmentSize
#define CSMI_SAS_SEGMENT_SIZE_INVALID        0xFFFFFFFF

// Invalid value for uSegmentImageSize
#define CSMI_SAS_SEGMENT_IMAGE_SIZE_INVALID  0xFFFFFFFF

/*************************************************************************/
/* DATA STRUCTURES                                                       */
/*************************************************************************/

/* * * * * * * * * * Adaptec Vendor Unique Class Structures * * * * * * * * * */

#pragma CSMI_SAS_BEGIN_PACK(8)

// CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG

typedef struct _CSMI_SAS_GET_ADPT_CNTLR_CONFIG {
   __u16 usPCIVendorID;
   __u16 usPCIDeviceID;
   __u16 usPCISubsystemVendorID;
   __u16 usPCISubsystemID;
   __u32 uControllerFlags;
   __u16 usFlashManufacturerID;
   __u16 usFlashDeviceID;
} CSMI_SAS_GET_ADPT_CNTLR_CONFIG,
  *PCSMI_SAS_GET_ADPT_CNTLR_CONFIG;

typedef struct _CSMI_SAS_GET_ADPT_CNTLR_CONFIG_BUFFER {
   IOCTL_HEADER IoctlHeader;
   CSMI_SAS_GET_ADPT_CNTLR_CONFIG Configuration;
} CSMI_SAS_GET_ADPT_CNTLR_CONFIG_BUFFER,
  *PCSMI_SAS_GET_ADPT_CNTLR_CONFIG_BUFFER;

// CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES

typedef struct _CSMI_SAS_NV_SEGMENT_PROPERTIES {
   __u32 uStatus;
   __u32 uSegmentID;
   __u16 usSegmentAttributes;
   __u32 uSegmentSize;
   __u32 uSegmentImageSize;
} CSMI_SAS_NV_SEGMENT_PROPERTIES,
  *PCSMI_SAS_NV_SEGMENT_PROPERTIES;

typedef struct _CSMI_SAS_NV_SEGMENT_PROPERTIES_BUFFER {
   IOCTL_HEADER IoctlHeader;
   CSMI_SAS_NV_SEGMENT_PROPERTIES Information;
} CSMI_SAS_NV_SEGMENT_PROPERTIES_BUFFER,
  *PCSMI_SAS_NV_SEGMENT_PROPERTIES_BUFFER;

// CC_CSMI_SAS_WRITE_NV_SEGMENT

typedef struct _CSMI_SAS_WRITE_NV_SEGMENT {
   __u32 uStatus;
   __u32 uSegmentID;
   __u32 uDestinationOffset;
   __u32 uBufferLength;
} CSMI_SAS_WRITE_NV_SEGMENT,
  *PCSMI_SAS_WRITE_NV_SEGMENT;

typedef struct _CSMI_SAS_WRITE_NV_SEGMENT_BUFFER {
   IOCTL_HEADER IoctlHeader;
   CSMI_SAS_WRITE_NV_SEGMENT Information;
   __u8 bSourceBuffer[1];
} CSMI_SAS_WRITE_NV_SEGMENT_BUFFER,
  *PCSMI_SAS_WRITE_NV_SEGMENT_BUFFER;

// CC_CSMI_SAS_READ_NV_SEGMENT

typedef struct _CSMI_SAS_READ_NV_SEGMENT {
   __u32 uStatus;
   __u32 uSegmentID;
   __u32 uSourceOffset;
   __u32 uBytesToRead;
   __u32 uBytesRead;
} CSMI_SAS_READ_NV_SEGMENT,
  *PCSMI_SAS_READ_NV_SEGMENT;

typedef struct _CSMI_SAS_READ_NV_SEGMENT_BUFFER {
   IOCTL_HEADER IoctlHeader;
   CSMI_SAS_READ_NV_SEGMENT Information;
   __u8 bDestinationBuffer[1];
} CSMI_SAS_READ_NV_SEGMENT_BUFFER,
  *PCSMI_SAS_READ_NV_SEGMENT_BUFFER;

#pragma CSMI_SAS_END_PACK

#endif // _CSMI_SAS_A_H_
