/*
 * Adaptec ADP94xx SAS HBA driver - CSMI IOCTL interface for Linux.
 *
 * Written by : Naveen Chandrasekaran <naveen_chandrasekaran@adaptec.com>
 * 
 * Copyright (c) 2004 Adaptec Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification.
 * 2. Redistributions in binary form must reproduce at minimum a disclaimer
 *    substantially similar to the "NO WARRANTY" disclaimer below
 *    ("Disclaimer") and any redistribution must be conditioned upon
 *    including a substantially similar Disclaimer requirement for further
 *    binary redistribution.
 * 3. Neither the names of the above-listed copyright holders nor the names
 *    of any contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * NO WARRANTY
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR CONTRIBUTORS BE LIABLE FOR SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 */	

#define REMOVE_WHEN_ERROR_RECOVERY_FIXED
#define ASD_ADPT_CSMI_EXTENSION

#ifdef ASD_ADPT_CSMI_EXTENSION
#include "adp94xx_csmiadpt.h"
#else 
#include "adp94xx_csmi.h"
#endif

#include "adp94xx_osm.h"
#include "adp94xx_inline.h"
#include "adp94xx_sata.h"
#include <linux/miscdevice.h>

#ifdef __x86_64__
#include <asm/ioctl32.h>
#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 0)	
#include <linux/syscalls.h>
#endif
#endif

/*
 * TBD: Tentatively use this minor till a minor 
 * number is assigned to this driver
 */
#define ASD_MISC_DEV_MINOR	225
#define ASD_MISC_DEV_NAME	"asdctl"

#define DBGLVL 2

#if (DBGLVL == 2)
#define ASDDBG1(x) printk x
#else
#define ASDDBG1(x)
#endif

/* protos for misc driver entry points */
static int asd_ctl_open(struct inode *inode, struct file *file);
static int asd_ctl_close(struct inode *inode, struct file *file);
static int asd_ctl_ioctl(struct inode *inode, struct file *file,
			 unsigned int cmd, unsigned long arg);

/* protos for ioctl handlers */
static int asd_csmi_get_driver_info(unsigned long arg);
static int asd_csmi_get_cntlr_config(unsigned long arg);
static int asd_csmi_get_cntlr_status(unsigned long arg);
static int asd_csmi_get_phy_info(unsigned long arg);
static int asd_csmi_set_phy_info(unsigned long arg);
static int asd_csmi_get_link_errors(unsigned long arg);
static int asd_csmi_smp_passthru(unsigned long arg);
static int asd_csmi_ssp_passthru(unsigned long arg);
static int asd_csmi_stp_passthru(unsigned long arg);
static int asd_csmi_get_sata_signature(unsigned long arg);
static int asd_csmi_get_scsi_address(unsigned long arg);
static int asd_csmi_get_device_address(unsigned long arg);
static int asd_csmi_sas_task_mgmt(unsigned long arg);
static int asd_csmi_sas_get_connector_info(unsigned long arg);
static int asd_csmi_sas_phy_control(unsigned long arg);

#ifdef ASD_ADPT_CSMI_EXTENSION
/* protos for Adaptec extension ioctl handlers */
static int asd_csmi_sas_get_adpt_cntlr_conf(unsigned long arg);
static int asd_csmi_sas_get_nv_seg_prop(unsigned long arg);
static int asd_csmi_sas_write_nv_seg(unsigned long arg);
static int asd_csmi_sas_read_nv_seg(unsigned long arg);
#endif

/* protos for helper - csmi mapping routines */
static void asd_ctl_map_open_reject_to_csmi(uint8_t reason, int abandon,
					    uint8_t *csmi_conn_stat);
 
static int asd_ctl_find_smp_txr_len(uint8_t smpfunc, uint32_t *reqlen, 
			uint32_t *resp_len);
static int asd_ctl_map_conn_type_to_csmi(uint8_t asd_conn_type, 
			uint8_t lane, uint32_t *csmi_conn_type);

/* protos for helper - work horse routines for ioctl handlers */
static int asd_ctl_issue_smp_pt(struct asd_softc *asd,
	struct asd_port	*port, 
	PCSMI_SAS_SMP_PASSTHRU_BUFFER pcsmi_sas_smp_passthru_buffer);

static int asd_ctl_issue_ssp_pt(struct asd_softc *asd,
	struct asd_port	*port, 
	PCSMI_SAS_SSP_PASSTHRU_BUFFER pcsmi_sas_ssp_passthru_buffer,
	unsigned long arg);

static int asd_ctl_issue_stp_pt(struct asd_softc *asd,
	struct asd_port	*port, 
	PCSMI_SAS_STP_PASSTHRU_BUFFER pcsmi_sas_stp_passthru_buffer,
	unsigned long arg);

static int asd_ctl_handle_hard_reset(struct asd_softc *asd, struct scb *scb);

static int asd_ctl_handle_task_mgmt(struct asd_softc *asd, struct scb *scb,
				    int tmf_opcode);

static int asd_ctl_abort_query_task(struct asd_softc *asd, struct scb *scb,
				    int tag, int tmf_opcode);

static int asd_ctl_process_tmf(struct asd_softc *asd, struct scb *scb);

static void  asd_ctl_issue_tmf(struct asd_softc *asd, struct scb *scb);

static int asd_ctl_issue_phy_ctl(struct asd_softc *asd,
	struct asd_phy	*phy, 
	PIOCTL_HEADER pioctl_header,
	uint32_t phy_ctl_func); 

static int asd_ctl_pc_get_phy_settings(struct asd_softc *asd, 
	struct asd_phy *phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer,
	int *num_cntls);

static int asd_ctl_process_ssp_resp(struct asd_softc *asd, 
	struct scb *scb, struct asd_done_list *done_listp,
	PCSMI_SAS_SSP_PASSTHRU_STATUS pcsmi_sas_ssp_passthru_status);

static int asd_ctl_issue_link_adm(struct asd_softc *asd,
	struct asd_port	*port, 
	PCSMI_SAS_LINK_ERRORS_BUFFER pcsmi_sas_link_errors_buffer,
	uint8_t sub_func);

static int asd_ctl_queue_cmd(struct asd_softc *asd, 
	struct scb *scb, struct asd_target *targ, struct asd_device *dev);

static int asd_ctl_check_phy_port(struct asd_softc *asd, 
	uint8_t phy_id, uint8_t port_id, struct asd_phy **phy, 
	struct asd_port **port);

static int asd_ctl_pc_map_phy_settings(struct asd_phy *phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer,
	asd_attrib attr, int link_rate, int ctl_index);

static int asd_ctl_pc_get_user_pc_val(struct asd_softc *asd, 
	struct asd_phy *phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer);

static int asd_ctl_copy_map_user_data_buf(struct asd_softc *asd, 
	unsigned long arg, struct scb *scb, uint8_t **preq_buf, 
	dma_addr_t *paddr, size_t *psg_copy_size, uint32_t data_len, 
	uint32_t user_buf_offset, uint32_t pci_data_dir);

static struct asd_nd_phydesc_format *asd_ctl_get_node_phyd(uint8_t *pnode_buf, 
	uint32_t node_buf_size, int phy_id);

static int asd_ctl_process_phy_ctl(struct asd_softc *asd, 
	struct asd_phy	*phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer);

static int asd_ctl_enter_pattern_gen(struct asd_softc *asd, 
	struct asd_phy	*phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer,
	int pattern_ctl_index);

static int asd_ctl_primitive_pat_gen(struct asd_softc *asd, 
	struct asd_phy	*phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer);

static int asd_ctl_data_pat_gen(struct asd_softc *asd, 
	struct asd_phy	*phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer,
	int pattern_ctl_index,
	uint8_t *ppat_buf,
	uint32_t pat_len);

static int asd_ctl_prep_pat_gen_mode(struct asd_softc *asd,
					struct asd_phy *phy,
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer);

static int asd_ctl_switch_pat_gen_mode(struct asd_softc *asd,
					struct asd_phy *phy);

#ifdef ASD_ADPT_CSMI_EXTENSION
static int asd_ctl_write_to_nvram(struct asd_softc *asd,
	PCSMI_SAS_WRITE_NV_SEGMENT_BUFFER pcsmi_sas_write_nv_seg_buffer,
	unsigned long arg,
	uint8_t **psource_buffer);

static int asd_ctl_read_from_nvram(struct asd_softc *asd,
	PCSMI_SAS_READ_NV_SEGMENT_BUFFER pcsmi_sas_read_nv_seg_buffer,
	unsigned long arg,
	uint8_t **psource_buffer);

static uint32_t asd_ctl_write_to_flash (struct asd_softc *asd, 
			uint8_t **src_img_addr, uint32_t segment_id, 
			uint32_t src_img_offset, uint32_t src_img_size);
#endif

/* Protos for IOCTL completion post-routines. */ 
static asd_scb_post_t		asd_ctl_issue_smp_pt_done;
static asd_scb_post_t		asd_ctl_issue_ssp_pt_done;
static asd_scb_post_t 		asd_ctl_issue_stp_pt_done;
static asd_scb_post_t		asd_ctl_issue_phy_ctl_done;
static asd_scb_post_t 		asd_ctl_issue_link_adm_done;
static asd_scb_post_t		asd_ctl_abort_query_done;
static asd_scb_post_t		asd_ctl_issue_tmf_done;
static asd_scb_eh_post_t	asd_ctl_task_mgmt_done;

/* protos for extern routines */
#ifdef __x86_64__
extern int register_ioctl32_conversion(unsigned int cmd, 
	int (*handler)(unsigned int, unsigned int, unsigned long,
	struct file *));

extern int unregister_ioctl32_conversion(unsigned int cmd); 
#endif

extern void asd_run_device_queues(struct asd_softc *asd);

extern int asd_hwi_search_nv_cookie(struct asd_softc *asd, 
			uint32_t *addr,
			struct asd_flash_dir_layout *pflash_dir_buf);

/* fops table */
static struct file_operations asd_ctl_fops = {
	.owner		= THIS_MODULE,
	.open		= asd_ctl_open,
	.release	= asd_ctl_close,
	.ioctl		= asd_ctl_ioctl
};

static struct miscdevice asd_ctl_miscdev = {
	.minor	= ASD_MISC_DEV_MINOR,
	.name	= ASD_MISC_DEV_NAME,
	.fops	= &asd_ctl_fops
};

/* macros */
#define ASD_CSMI_FILL_SSP_STATUS(TaskIUStat, ConnStatus)		\
{									\
	(TaskIUStat)->bConnectionStatus = ConnStatus;			\
	(TaskIUStat)->bDataPresent = CSMI_SAS_SSP_NO_DATA_PRESENT;	\
	(TaskIUStat)->bStatus = GOOD;					\
	(TaskIUStat)->bResponseLength[0] = 0;				\
	(TaskIUStat)->bResponseLength[1] = 0;				\
	(TaskIUStat)->uDataBytes = 0;					\
} while (0)

#define ASD_CSMI_TIMEOUT(pioctl_header) 	\
((pioctl_header->Timeout == 0? CSMI_ALL_TIMEOUT: pioctl_header->Timeout) * HZ) 

/* represents busy bit number */
#define ASD_CSMI_INTERNAL_BUSY_BIT_NR 0

/* PHY control register manipulation */
#define PC_PHY_DRV_MAX_VAL	7
#define PC_PREEMP_MAX_VAL	3
#define PC_SLEW_MAX_VAL		3
#define PC_RXCOMC_MAX_VAL	3
#define PC_DETLVL_MAX_VAL	7

#define PC_PHY_DRV_SHIFT	5
#define PC_PREEMP_SHIFT		3
#define PC_SLEW_SHIFT		0
#define PC_RXCOMC_SHIFT		2
#define PC_SAS_DETLVL_SHIFT	3
#define PC_SATA_DETLVL_SHIFT	0

/* LSeq Modes */
#define LSEQ_MODE_RX		0
#define LSEQ_MODE_TX		1
#define LSEQ_MODE_OOB		5


/* CJTPAT */
uint8_t sascjtpat[] = {
	/* 
	 * CJTPAT SSP DATA frame from SAS spec 1.0 Table A.4 
	 * The value 0x00 is used for bytes defined as unknown
	 */

	/* 0 - SSP DATA frame header */
	0x00, 0x00, 0x00, 0x00,		/* 1 */
	0x00, 0x00, 0x00, 0x00,		/* 2 */
	0x00, 0x00, 0x00, 0x00,		/* 3 */
	0x00, 0x00, 0x00, 0x00,		/* 4 */
	0x00, 0x00, 0x00, 0x00,		/* 5 */
	0x00, 0x00, 0x00, 0x00,		/* 6 */
	
	/* 1 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 2 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 3 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 4 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 5 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 6 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x74,		/* 2 */
	0x7E, 0xAB, 0xB5, 0xB5,		/* 3 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 4 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 5 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 6 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 7 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 8 */
	
	/* 7 - SSP frame data */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 1 */
	0xB5, 0xB5, 0xB5, 0x74,		/* 2 */
	0xB5, 0xAB, 0xB5, 0xB5,		/* 3 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 4 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 5 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 6 */
	0xB5, 0xB5, 0xB5, 0xB5,		/* 7 */
	0xB5, 0x5E, 0x4A, 0x7E,		/* 8 */
	
	/* 8 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 9 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 10 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 11 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 12 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 2 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 3 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 4 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 5 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 6 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 7 */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 8 */
	
	/* 13 - SSP frame data */
	0x7E, 0x7E, 0x7E, 0x7E,		/* 1 */
	0x7E, 0x7E, 0x7E, 0x6B,		/* 2 */
	0x7E, 0x54, 0x4A, 0x4A,		/* 3 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 4 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 5 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 6 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 7 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 8 */
	
	/* 14 - SSP frame data */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 1 */
	0x4A, 0x4A, 0x4A, 0x6B,		/* 2 */
	0x4A, 0x54, 0x4A, 0x4A,		/* 3 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 4 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 5 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 6 */
	0x4A, 0x4A, 0x4A, 0x4A,		/* 7 */
	0x4A, 0xBE, 0xB5, 0x7E		/* 8 */

	/* CRC - appended by HW */
};

/* inline routines */
/*
 * Function:
 * 	asd_ctl_check_buf_len()
 *
 * Description:
 * 	This routine validates the buffer length specified in the CSMI
 * 	IOCTL header with the actual size of the buffer needed for the 
 * 	corresponding IOCTL.
 */
static inline int asd_ctl_check_buf_len(unsigned long arg, int buffer_length)
{
	IOCTL_HEADER ioctl_header;
	int err;
	
	if ((err = copy_from_user(&ioctl_header, (void *)arg, 
				 sizeof(ioctl_header)))) {
		return -EFAULT;
	}

	if (ioctl_header.Length < buffer_length) {
		ioctl_header.ReturnCode = CSMI_SAS_STATUS_INVALID_PARAMETER;
		if ((err = copy_to_user((void *)arg, &ioctl_header, 
					sizeof(ioctl_header)))) {
			return -EFAULT;
		}
		return -EINVAL;
	}
	return 0;
}

static inline int asd_ctl_copy_to_user(void *dest, void *src, uint32_t length, 
			uint32_t status)
{
	PIOCTL_HEADER pioctl_header;
	int err;

	pioctl_header = (PIOCTL_HEADER)src;
	pioctl_header->ReturnCode = status;
	if ((err = copy_to_user(dest, src, length))) {
		return -EFAULT;
	}
	return 0;
}

static inline void asd_ctl_start_csmi_scb_timer(struct timer_list *timer, 
			u_long timeout, struct scb *scb, 
			void (*time_out_func)(unsigned long))
{
	timer->expires = jiffies + timeout;
	timer->data = (u_long) scb;
	timer->function = time_out_func;
	add_timer(timer);
	return;
}

static inline void asd_ctl_csmi_timeout(unsigned long ptr)
{
	struct scb *scb;
	struct asd_softc *asd;
	struct asd_csmi_mgmt *pcsmi_mgmt;

	scb = (struct scb *)ptr;
	asd = scb->softc;
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;

	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
	wake_up(&pcsmi_mgmt->waitq);
	return;
}

static inline int asd_ctl_set_pc_reg(uint8_t *pc_reg, int8_t userval,
			uint8_t mask, int shift, int min_val, int max_val)
{
	uint8_t regval;

	regval = (*pc_reg & mask) >> shift;
	if ((regval + userval < min_val) || (regval + userval > max_val)) {
		return -1;
	}
	if ((regval + userval) == regval) {
		return 0;
	}
	regval += userval;
	*pc_reg |= (regval << shift);
	return 0;
}

static inline int asd_ctl_get_pc_tx_drvamp(uint8_t pc_reg, int8_t *drvamp)
{
	/* Caller passes the proper register values for SAS or SATA */
	/* 
	 * DC: Need to change this whether we are try obtaining 
	 *     SAS or SATA values.
	 */
	*drvamp = (pc_reg & SAS_PHY_DRV_MASK) >> PC_PHY_DRV_SHIFT; 
	return 0;
}
static inline int asd_ctl_get_pc_preemp(uint8_t pc_reg, int8_t *preemp)
{
	/* Caller passes the proper register values for SAS or SATA */
	/* 
	 * DC: Need to change this whether we are try obtaining 
	 *     SAS or SATA values.
	 */
	*preemp = (pc_reg & SAS_PREEMP_MASK) >> PC_PREEMP_SHIFT;
	return 0;
}

static inline int asd_ctl_get_pc_slewrate(uint8_t pc_reg, int8_t *slew)
{
	/* Caller passes the proper register values for SAS or SATA */
	/* 
	 * DC: Need to change this whether we are try obtaining 
	 *     SAS or SATA values.
	 */
	*slew = (pc_reg & SAS_SLEW_MASK); 
	return 0;
}

static inline int asd_ctl_get_pc_rxcomv(uint8_t pc_reg, int *comv)
{
	*comv = (pc_reg & PHY_RXCOMCENTER_MASK) >> PC_RXCOMC_SHIFT;
	return 0;
}

static inline int asd_ctl_get_pc_detlvl(uint8_t pc_reg, uint8_t mask, 
					int shift, int8_t *detlvl)
{
	*detlvl = (pc_reg & mask) >> shift;
	return 0;
}

static inline int asd_ctl_map_ssp_resp_to_csmi(uint8_t asd_data_pres, 
			uint8_t *csmi_data_pres)
{
	switch (asd_data_pres) {
	case SSP_RIU_DATAPRES_NONE:
		*csmi_data_pres = CSMI_SAS_SSP_NO_DATA_PRESENT;
		break;
	case SSP_RIU_DATAPRES_RESP:
		*csmi_data_pres = CSMI_SAS_SSP_RESPONSE_DATA_PRESENT;
		break;
	case SSP_RIU_DATAPRES_SENSE:
		*csmi_data_pres = CSMI_SAS_SSP_SENSE_DATA_PRESENT;
		break;
	default:
		return -EINVAL;
	}

	return 0;
}

static inline int asd_ctl_copy_ssp_resp_data(struct ssp_resp_iu	*riu, 
		PCSMI_SAS_SSP_PASSTHRU_STATUS pcsmi_sas_ssp_passthru_status)
{
	uint32_t resp_len;

	resp_len = 0;
	asd_ctl_map_ssp_resp_to_csmi(riu->datapres, 
			&pcsmi_sas_ssp_passthru_status->bDataPresent);
	
	if (pcsmi_sas_ssp_passthru_status->bDataPresent 
	    == CSMI_SAS_SSP_RESPONSE_DATA_PRESENT) {
		resp_len = scsi_4btoul(riu->response_len);
	} else {
		if (pcsmi_sas_ssp_passthru_status->bDataPresent 
		== CSMI_SAS_SSP_SENSE_DATA_PRESENT) {
			resp_len = scsi_4btoul(riu->sense_len);
		}
	}
		
	/* csmi buffer restriction */
	resp_len = (resp_len > 256) ? 256 : resp_len;

	/* Response length is 2 bytes in CSMI while its 4 bytes in SAS */
	pcsmi_sas_ssp_passthru_status->bResponseLength[0] = resp_len >> 8;
	pcsmi_sas_ssp_passthru_status->bResponseLength[1] = resp_len & 0xff;

	memcpy(pcsmi_sas_ssp_passthru_status->bResponse, riu->data, resp_len);

	return 0;
}

static inline int asd_ctl_map_protocol_to_csmi(u_int asd_attr, 
				uint8_t *csmi_attr)
{
	if (asd_attr & ASD_SSP_INITIATOR) {
		*csmi_attr |= CSMI_SAS_PROTOCOL_SSP;
	}
	if (asd_attr & ASD_SMP_INITIATOR) {
		*csmi_attr |= CSMI_SAS_PROTOCOL_SMP;
	}
	if (asd_attr & ASD_STP_INITIATOR) {
		*csmi_attr |= CSMI_SAS_PROTOCOL_STP;
	}
	
	/* we support direct attached SATA device ? */
	*csmi_attr |= CSMI_SAS_PROTOCOL_SATA;

	return 0;
}

static inline int asd_ctl_map_transport_to_csmi(TRANSPORT_TYPE asd_trans, 
				uint8_t *csmi_trans)
{
	switch (asd_trans) {
	case ASD_TRANSPORT_SSP:
		*csmi_trans |= CSMI_SAS_PROTOCOL_SSP;
		break;
	case ASD_TRANSPORT_SMP:
		*csmi_trans |= CSMI_SAS_PROTOCOL_SMP;
		break;
	case ASD_TRANSPORT_STP:
		*csmi_trans |= CSMI_SAS_PROTOCOL_STP;
		break;
	case ASD_TRANSPORT_ATA:
		*csmi_trans |= CSMI_SAS_PROTOCOL_SATA;
		break;
	case ASD_TRANSPORT_UNKNOWN:
	default:
		return -EINVAL;
	}
	return 0;
}

static inline int asd_ctl_map_link_rate_to_csmi(u_int link_rate, 
			uint8_t *csmi_link_rate)
{
	switch (link_rate) {
	case SAS_15GBPS_RATE:
		*csmi_link_rate = CSMI_SAS_LINK_RATE_1_5_GBPS; 
		break;
	case SAS_30GBPS_RATE:
		*csmi_link_rate = CSMI_SAS_LINK_RATE_3_0_GBPS;
		break;
	default:
		return -1;
	/*TBD: programmed link rate and move to a separate function*/
	}
	return 0;
}

static inline int asd_ctl_map_link_rate_from_csmi(uint8_t csmi_link_rate, 
				u_int *link_rate)
{
	switch (csmi_link_rate) {
	case CSMI_SAS_LINK_RATE_1_5_GBPS:
		*link_rate = SAS_15GBPS_RATE;
		break;
	case CSMI_SAS_LINK_RATE_3_0_GBPS:
		*link_rate = SAS_30GBPS_RATE;
		break;
	case CSMI_SAS_PROGRAMMED_LINK_RATE_UNCHANGED:
		break;
	default:
		return -EINVAL;
	}
	return 0;
}

static inline int asd_ctl_map_conn_rate_to_csmi(u_int conn_rate, 
		uint8_t *csmi_conn_rate)
{
	switch (conn_rate) {
	case SAS_15GBPS_RATE:
		*csmi_conn_rate = CSMI_SAS_LINK_RATE_1_5_GBPS; 
		break;
	case SAS_30GBPS_RATE:
		*csmi_conn_rate = CSMI_SAS_LINK_RATE_3_0_GBPS;
		break;
	case SAS_60GBPS_RATE:
	case SAS_INVALID_RATE:
		*csmi_conn_rate = CSMI_SAS_LINK_RATE_UNKNOWN; 
		break;
	default:
		*csmi_conn_rate = CSMI_SAS_LINK_RATE_FAILED; 
		break;
	/*TBD: phy disabled case */
	}
	return 0;
}

static inline int asd_ctl_map_targ_typ_to_csmi(struct asd_port *port, 
				uint8_t *csmi_dev_type)
{
	switch (port->management_type) {
	case ASD_DEVICE_NONE:
		*csmi_dev_type = CSMI_SAS_NO_DEVICE_ATTACHED;
		break;
	case ASD_DEVICE_END:
		*csmi_dev_type = CSMI_SAS_END_DEVICE;
		break;
	case ASD_DEVICE_EDGE_EXPANDER:
		*csmi_dev_type = CSMI_SAS_EDGE_EXPANDER_DEVICE ;
		break;
	case ASD_DEVICE_FANOUT_EXPANDER:
		*csmi_dev_type = CSMI_SAS_FANOUT_EXPANDER_DEVICE ;
		break;
	default:
		return -EINVAL;
	}
	return 0;
}

static inline int asd_ctl_map_ssp_dir_from_csmi(uint32_t csmi_dir, 
		uint32_t *asd_dir,
		uint32_t *scsi_dir)
{
	*asd_dir = DATA_DIR_UNSPECIFIED;
	*scsi_dir = SCSI_DATA_UNKNOWN;

	if (csmi_dir & CSMI_SAS_SSP_READ) {
		*asd_dir = DATA_DIR_INBOUND;
		*scsi_dir = SCSI_DATA_READ;
	}

	if (csmi_dir & CSMI_SAS_SSP_WRITE) {
		*asd_dir = DATA_DIR_OUTBOUND;
		*scsi_dir = SCSI_DATA_WRITE;
	}

	if (((csmi_dir & CSMI_SAS_SSP_READ) 
	   && (csmi_dir & CSMI_SAS_SSP_WRITE))
	  || (csmi_dir & CSMI_SAS_SSP_UNSPECIFIED)) {
		*asd_dir = DATA_DIR_UNSPECIFIED;
		*scsi_dir = SCSI_DATA_UNKNOWN;
	}
	/*
	if ((csmi_dir & CSMI_SAS_SSP_TASK_ATTRIBUTE_SIMPLE)
	   || (csmi_dir & CSMI_SAS_SSP_TASK_ATTRIBUTE_HEAD_OF_QUEUE)
	   || (csmi_dir & CSMI_SAS_SSP_TASK_ATTRIBUTE_ORDERED)
	   || (csmi_dir & CSMI_SAS_SSP_TASK_ATTRIBUTE_ACA)) {
		*asd_dir |= DATA_DIR_NO_XFER;
		*scsi_dir |= SCSI_DATA_NONE;
	}
	*/
	return 0;
}

static inline int asd_ctl_map_stp_dir_from_csmi(uint32_t csmi_dir, 
		uint32_t *asd_dir,
		uint32_t *scsi_dir)
{
	*asd_dir = DATA_DIR_UNSPECIFIED;
	*scsi_dir = SCSI_DATA_UNKNOWN;

	if (csmi_dir & CSMI_SAS_STP_READ) {
		*asd_dir = DATA_DIR_INBOUND;
		*scsi_dir = SCSI_DATA_READ;
	}

	if (csmi_dir & CSMI_SAS_STP_WRITE) {
		*asd_dir = DATA_DIR_OUTBOUND;
		*scsi_dir = SCSI_DATA_WRITE;
	}
	
	if (((csmi_dir & CSMI_SAS_STP_READ) 
	   && (csmi_dir & CSMI_SAS_STP_WRITE)) 
	  || (csmi_dir & CSMI_SAS_STP_UNSPECIFIED)) {
		*asd_dir = DATA_DIR_UNSPECIFIED;
		*scsi_dir = SCSI_DATA_UNKNOWN;
	}
	/*
	if ((csmi_dir & CSMI_SAS_STP_PIO)
	   || (csmi_dir & CSMI_SAS_STP_DMA)
	   || (csmi_dir & CSMI_SAS_STP_PACKET)
	   || (csmi_dir & CSMI_SAS_STP_DMA_QUEUED)
	   || (csmi_dir & CSMI_SAS_STP_EXECUTE_DIAG)
	   || (csmi_dir & CSMI_SAS_STP_RESET_DEVICE)) {
		*asd_dir |= DATA_DIR_NO_XFER;
		*scsi_dir |= SCSI_DATA_NONE;
	}
	*/	
	return 0;
}

static inline int asd_ctl_map_pc_func_from_csmi(uint32_t csmi_phy_ctl_func, 
			uint32_t *asd_phy_ctl_func)
{
	switch (csmi_phy_ctl_func) {
	case CSMI_SAS_PC_HARD_RESET:
		*asd_phy_ctl_func = EXECUTE_HARD_RESET;
		break;
	case CSMI_SAS_PC_PHY_DISABLE:
		*asd_phy_ctl_func = DISABLE_PHY;
		break;
	case CSMI_SAS_PC_LINK_RESET:
		*asd_phy_ctl_func = ENABLE_PHY;
		break;
	case CSMI_SAS_PC_NOP:
		*asd_phy_ctl_func = PHY_NO_OP;
		break;
	default:
		return -EINVAL;
	}
	return 0;
}

static inline int asd_ctl_map_dev_type_to_csmi(uint8_t asd_attr, 
			uint8_t *csmi_dev_type)
{
	int err;
	
	err = -1;
	if (asd_attr & ASD_SAS_MODE) {
		*csmi_dev_type = CSMI_SAS_SAS;
		err = 0;
	}
	
	if (asd_attr & ASD_SATA_MODE) {
		*csmi_dev_type = CSMI_SAS_SATA;
		err = 0;
	}

	return err;
}

static inline int asd_ctl_map_dev_type_from_csmi(uint8_t csmi_dev_type, 
			uint8_t *asd_attr)
{
	int err;
	
	err = -1;
	if (csmi_dev_type == CSMI_SAS_SAS) {
		*asd_attr |= ASD_SAS_MODE;
		err = 0;
	}
	if (csmi_dev_type == CSMI_SAS_SATA) {
		*asd_attr |= ASD_SATA_MODE;
		err = 0;
	}

	return err;
}

static inline int asd_ctl_map_conn_loc_to_csmi(uint8_t asd_location, 
			uint8_t *csmi_location)
{
	int err;
	
	err = 0;
	switch (asd_location) {
	case NV_CONNL_INTERNAL:
		*csmi_location = CSMI_SAS_CON_INTERNAL;
		break;
	case NV_CONNL_EXTERNAL:
		*csmi_location = CSMI_SAS_CON_EXTERNAL;
		break;
	case NV_CONNL_B2B:
	case NV_CONNL_UNKNOWN:
	default:
		*csmi_location = CSMI_SAS_CON_UNKNOWN;
		break;
		//CSMI_SAS_CON_SWITCHABLE;
		//CSMI_SAS_CON_AUTO;
	}

	return err;
}

static inline struct asd_phy *asd_ctl_get_phy(struct asd_softc *asd, 
		uint8_t phy_id)
{
	struct asd_phy *phy;
	int phy_num;
		
	phy = NULL;
	for (phy_num = 0; 
	     phy_num < asd->hw_profile.max_phys; 
	     phy_num++) {
		phy = asd->phy_list[phy_num];
		if ((phy) && (phy->id == phy_id)) {
			return phy;
		}
	}
	
	return NULL;
}

static inline struct asd_port *asd_ctl_get_port(struct asd_softc *asd, 
		uint8_t port_id)
{
	struct asd_port *port;
	int port_num;

	port = NULL;
	for (port_num = 0; port_num < ASD_MAX_PORTS; port_num++) {
		port = asd->port_list[port_num];
		if ((port) && (port->id == port_id)) {
			return port;
		}
	}
	return NULL;
}

static inline int asd_ctl_valloc_copy_buf(struct asd_softc *asd, 
		uint8_t **preq_buf, uint32_t buf_len, uint32_t user_buf_offset, 
		unsigned long arg)
{
	uint8_t *pdata_buf;
	int err;

	/* 
	 * The callers of this routine probably are in need of more memory 
	 * (> 128K), hence we use vmalloc() here.
	 */
	pdata_buf = NULL;
	if ((pdata_buf = (uint8_t *)vmalloc(buf_len)) == NULL) {
		return -ENOMEM;
	}
	memset(pdata_buf, 0, buf_len);
		
	if ((err = copy_from_user(pdata_buf, 
			 (void *)arg + user_buf_offset,
			 buf_len))) {
		vfree(pdata_buf);
		return -EFAULT;
	}
	*preq_buf = pdata_buf;

	return 0;
}
	

#ifdef ASD_ADPT_CSMI_EXTENSION
static inline int asd_ctl_map_nv_segid_from_csmi(uint32_t csmi_segid, 
			uint32_t *asd_segid)
{
	switch (csmi_segid) {
	case CSMI_SAS_SEGMENT_ID_CTRL_A_USER_SETTINGS0:
		*asd_segid = NVRAM_CTRL_A_SETTING;
		break;
	case CSMI_SAS_SEGMENT_ID_MANUFACTURING_SECTOR0: 
		*asd_segid = NVRAM_MANUF_TYPE;
		break;
	case CSMI_SAS_SEGMENT_ID_FLASH_DIRECTORY0:/*TBD*/
	case CSMI_SAS_SEGMENT_ID_FLASH0: /* TBD */
	case CSMI_SAS_SEGMENT_ID_SEEPROM0: /* TBD */
	case CSMI_SAS_SEGMENT_ID_COMPATIBILITY_SECTOR0:/*TBD*/
		*asd_segid = NVRAM_NO_SEGMENT_ID;
	default:
		return -1;
	}

	return 0;
}

static inline int asd_ctl_map_seg_attr_to_csmi(uint16_t asd_seg_attr, 
			uint16_t *csmi_seg_attr)
{
	switch (asd_seg_attr) {
	case 0:
		*csmi_seg_attr = CSMI_SAS_SEGMENT_ATTRIBUTE_READONLY;
		break;	
	case 1:
		*csmi_seg_attr = CSMI_SAS_SEGMENT_ATTRIBUTE_ERASEWRITE;
		break;
	default:
		return -1;
	}
	return 0;
}
#endif /* #ifdef ASD_ADPT_CSMI_EXTENSION */

/* registration routines */
int asd_register_ioctl_dev(void)
{
	int err;

	if ((err = misc_register(&asd_ctl_miscdev))) {
		return err;
	}

#ifdef __x86_64__
	/* This ioctl handler module is compatible between 64 and 32 bit
	 * environments, hence register the default handler as 32bit 
	 * ioctl conversion handler.
	 */
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_DRIVER_INFO, 
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_CNTLR_CONFIG, 
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_CNTLR_STATUS,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_FIRMWARE_DOWNLOAD,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_RAID_INFO,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_RAID_CONFIG,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_PHY_INFO,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_SET_PHY_INFO,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_LINK_ERRORS,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_SMP_PASSTHRU,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_SSP_PASSTHRU,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_STP_PASSTHRU,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_SATA_SIGNATURE,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_SCSI_ADDRESS,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_DEVICE_ADDRESS,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_TASK_MANAGEMENT,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_CONNECTOR_INFO,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_PHY_CONTROL,
							(void *)sys_ioctl);
#ifdef ASD_ADPT_CSMI_EXTENSION
	err |= register_ioctl32_conversion(CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(
					CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_WRITE_NV_SEGMENT,
							(void *)sys_ioctl);
	err |= register_ioctl32_conversion(CC_CSMI_SAS_READ_NV_SEGMENT,
							(void *)sys_ioctl);
#endif /* #ifdef ASD_ADPT_CSMI_EXTENSION */
#endif /* #ifdef __x86_64__ */
	return err;
}

int asd_unregister_ioctl_dev(void)
{
#ifdef __x86_64__
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_DRIVER_INFO);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_CNTLR_CONFIG);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_CNTLR_STATUS);
	unregister_ioctl32_conversion(CC_CSMI_SAS_FIRMWARE_DOWNLOAD);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_RAID_INFO);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_RAID_CONFIG);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_PHY_INFO);
	unregister_ioctl32_conversion(CC_CSMI_SAS_SET_PHY_INFO);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_LINK_ERRORS);
	unregister_ioctl32_conversion(CC_CSMI_SAS_SMP_PASSTHRU);
	unregister_ioctl32_conversion(CC_CSMI_SAS_SSP_PASSTHRU);
	unregister_ioctl32_conversion(CC_CSMI_SAS_STP_PASSTHRU);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_SATA_SIGNATURE);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_SCSI_ADDRESS);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_DEVICE_ADDRESS);
	unregister_ioctl32_conversion(CC_CSMI_SAS_TASK_MANAGEMENT);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_CONNECTOR_INFO);
	unregister_ioctl32_conversion(CC_CSMI_SAS_PHY_CONTROL);
#ifdef ASD_ADPT_CSMI_EXTENSION
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG);
	unregister_ioctl32_conversion(CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES);
	unregister_ioctl32_conversion(CC_CSMI_SAS_WRITE_NV_SEGMENT);
	unregister_ioctl32_conversion(CC_CSMI_SAS_READ_NV_SEGMENT);
#endif /* #ifdef ASD_ADPT_CSMI_EXTENSION */
#endif /* #ifdef __x86_64__ */
	return misc_deregister(&asd_ctl_miscdev);
}

/* init routine */
int asd_ctl_init_csmi_internal_data(struct asd_softc *asd)
{
	struct asd_csmi_mgmt *pcsmi_mgmt;
	
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	
	/* Initialize waitq */
	init_waitqueue_head(&pcsmi_mgmt->waitq);
	
	/* Initialize busy flags */
	pcsmi_mgmt->busy = 0;
	
	/* Initialize semaphore */
	init_MUTEX(&pcsmi_mgmt->sem);
	
	/* Initialize timers */
	init_timer(&pcsmi_mgmt->timer);

	/* Initialize err flag */
	pcsmi_mgmt->err = SCB_EH_FAILED;
	
	return 0;
}

/* entry points */
static int asd_ctl_open(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)	
	MOD_INC_USE_COUNT;
#endif	
	return 0;
}

static int asd_ctl_close(struct inode *inode, struct file *file)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)	
	MOD_DEC_USE_COUNT;
#endif	
	return 0;
}

static int asd_ctl_ioctl(struct inode *inode, struct file *file,
			unsigned int cmd, unsigned long arg)
{
	IOCTL_HEADER ioctl_header;
	int csmi_user_buf_len;
	int hba, max_hba;
	int err;

	if (!capable(CAP_SYS_ADMIN)) {
		return -EACCES;
	}
	
	csmi_user_buf_len = sizeof(ioctl_header);
	if ((err = copy_from_user(&ioctl_header, (void *)arg, 
				csmi_user_buf_len))) {
		return -EFAULT;
	}

	hba = ioctl_header.IOControllerNumber;

	max_hba = asd_get_number_of_hbas_present();

	if (hba < 0 || hba >= max_hba) {
		asd_ctl_copy_to_user((void *)arg, 
				     &ioctl_header, 
				     csmi_user_buf_len,
				     CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}

	switch (cmd) {
	case CC_CSMI_SAS_GET_DRIVER_INFO:
		err = asd_csmi_get_driver_info(arg);
		break;
	case CC_CSMI_SAS_GET_CNTLR_CONFIG:
		err = asd_csmi_get_cntlr_config(arg);
		break;
	case CC_CSMI_SAS_GET_CNTLR_STATUS:
		err = asd_csmi_get_cntlr_status(arg);
		break;
	case CC_CSMI_SAS_GET_PHY_INFO:
		err = asd_csmi_get_phy_info(arg);
		break;
	case CC_CSMI_SAS_SET_PHY_INFO:
		err = asd_csmi_set_phy_info(arg);
		break;
	case CC_CSMI_SAS_GET_LINK_ERRORS:
		err = asd_csmi_get_link_errors(arg);
		break;
	case CC_CSMI_SAS_SMP_PASSTHRU:
		err = asd_csmi_smp_passthru(arg);
		break;
	case CC_CSMI_SAS_SSP_PASSTHRU:
		err = asd_csmi_ssp_passthru(arg);
		break;
	case CC_CSMI_SAS_STP_PASSTHRU:
		err = asd_csmi_stp_passthru(arg);
		break;
	case CC_CSMI_SAS_GET_SATA_SIGNATURE:
		err = asd_csmi_get_sata_signature(arg);
		break;
	case CC_CSMI_SAS_GET_SCSI_ADDRESS:
		err = asd_csmi_get_scsi_address(arg);
		break;
	case CC_CSMI_SAS_GET_DEVICE_ADDRESS:
		err = asd_csmi_get_device_address(arg);
		break;
	case CC_CSMI_SAS_TASK_MANAGEMENT:
		err = asd_csmi_sas_task_mgmt(arg);
		break;
	case CC_CSMI_SAS_GET_CONNECTOR_INFO :
		err = asd_csmi_sas_get_connector_info(arg);
		break;
	case CC_CSMI_SAS_PHY_CONTROL :
		err = asd_csmi_sas_phy_control(arg);
		break;
   
#ifdef ASD_ADPT_CSMI_EXTENSION
	/* Adaptec specific IOCTL extensions to CSMI */
	case CC_CSMI_SAS_GET_ADPT_CNTLR_CONFIG:
		err = asd_csmi_sas_get_adpt_cntlr_conf(arg);
		break;
	case CC_CSMI_SAS_GET_NV_SEGMENT_PROPERTIES:
		err = asd_csmi_sas_get_nv_seg_prop(arg);
		break;
	case CC_CSMI_SAS_WRITE_NV_SEGMENT:
		err = asd_csmi_sas_write_nv_seg(arg);
		break;
	case CC_CSMI_SAS_READ_NV_SEGMENT:
		err = asd_csmi_sas_read_nv_seg(arg);
		break;
#endif
	/* Unsupported/Unknown IOCTLs */
	case CC_CSMI_SAS_FIRMWARE_DOWNLOAD:
	case CC_CSMI_SAS_GET_RAID_INFO:
	case CC_CSMI_SAS_GET_RAID_CONFIG:
	default:
		asd_ctl_copy_to_user((void *)arg, 
				     &ioctl_header, 
				     csmi_user_buf_len,
				     CSMI_SAS_STATUS_BAD_CNTL_CODE);
		return -EINVAL;
	}

	return err;
}


/* ioctl handler routines */
static int asd_csmi_get_driver_info(unsigned long arg)
{
	CSMI_SAS_DRIVER_INFO_BUFFER csmi_sas_driver_info_buffer;
	PCSMI_SAS_DRIVER_INFO pcsmi_sas_driver_info;
	PIOCTL_HEADER pioctl_header;
	int csmi_user_buf_len;
	int err;

	csmi_user_buf_len = sizeof(csmi_sas_driver_info_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len ))) {
		/* Insufficient buffer space */
		return -EINVAL;
	}

	if ((err = copy_from_user(&csmi_sas_driver_info_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}

	pioctl_header = &csmi_sas_driver_info_buffer.IoctlHeader;
	pcsmi_sas_driver_info = &csmi_sas_driver_info_buffer.Information;
	
	strcpy(pcsmi_sas_driver_info->szName, ASD_DRIVER_NAME);
	strcpy(pcsmi_sas_driver_info->szDescription, ASD_DRIVER_DESCRIPTION);
	pcsmi_sas_driver_info->usMajorRevision = ASD_MAJOR_VERSION;
	pcsmi_sas_driver_info->usMinorRevision = ASD_MINOR_VERSION;
	pcsmi_sas_driver_info->usBuildRevision = ASD_BUILD_VERSION;
	pcsmi_sas_driver_info->usReleaseRevision = ASD_RELEASE_VERSION;
	pcsmi_sas_driver_info->usCSMIMajorRevision = CSMI_MAJOR_REVISION;
	pcsmi_sas_driver_info->usCSMIMinorRevision = CSMI_MINOR_REVISION;

	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_driver_info_buffer, 
			csmi_user_buf_len,
			CSMI_SAS_STATUS_SUCCESS);
}

static int asd_csmi_get_cntlr_config(unsigned long arg)
{
	CSMI_SAS_CNTLR_CONFIG_BUFFER csmi_sas_cntlr_config_buffer;
	PCSMI_SAS_CNTLR_CONFIG pcsmi_sas_cntlr_config;
	PCSMI_SAS_IO_BUS_ADDRESS pcsmi_sas_io_bus_address;
	PCSMI_SAS_PCI_BUS_ADDRESS pcsmi_sas_pci_bus_address;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	int csmi_user_buf_len;
	int err;
	
	csmi_user_buf_len = sizeof(csmi_sas_cntlr_config_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_cntlr_config_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_cntlr_config_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_cntlr_config_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}
		
	pcsmi_sas_cntlr_config = &csmi_sas_cntlr_config_buffer.Configuration;
	pcsmi_sas_io_bus_address = &pcsmi_sas_cntlr_config->BusAddress;
	pcsmi_sas_pci_bus_address = &pcsmi_sas_io_bus_address->PciAddress;

	pcsmi_sas_cntlr_config->uBaseIoAddress = 
				asd_pcic_read_dword(asd, PCI_BASE_ADDRESS_0);
	pcsmi_sas_cntlr_config->BaseMemoryAddress.uLowPart =
				asd_pcic_read_dword(asd, PCI_BASE_ADDRESS_2);
	pcsmi_sas_cntlr_config->BaseMemoryAddress.uHighPart =
				asd_pcic_read_dword(asd, PCI_BASE_ADDRESS_3);

	pcsmi_sas_cntlr_config->uBoardID =
				(asd_pci_dev(asd)->device << 16) |
				asd_pci_dev(asd)->vendor;

	pcsmi_sas_cntlr_config->usSlotNumber = 
				PCI_SLOT(asd_pci_dev(asd)->devfn);
	pcsmi_sas_cntlr_config->bControllerClass = CSMI_SAS_CNTLR_CLASS_HBA;
	pcsmi_sas_cntlr_config->bIoBusType = CSMI_SAS_BUS_TYPE_PCI;
	pcsmi_sas_pci_bus_address->bBusNumber = asd_pci_dev(asd)->bus->number;
	pcsmi_sas_pci_bus_address->bDeviceNumber =
				PCI_SLOT(asd_pci_dev(asd)->devfn);
	pcsmi_sas_pci_bus_address->bFunctionNumber =
				PCI_FUNC(asd_pci_dev(asd)->devfn);
	
	sprintf(pcsmi_sas_cntlr_config->szSerialNumber,"%01x%06x%08x%x", 
			ASD_BSAR_GET_NAA(asd->hw_profile.wwn), 
			ASD_BSAR_GET_IEEE_ID(asd->hw_profile.wwn),
			ASD_BSAR_GET_SN(asd->hw_profile.wwn),
			ASD_BSAR_GET_LSB(asd->hw_profile.wwn));

	pcsmi_sas_cntlr_config->usMajorRevision = 0;
	pcsmi_sas_cntlr_config->usMinorRevision = 0;
	pcsmi_sas_cntlr_config->usBuildRevision = 0;
	pcsmi_sas_cntlr_config->usReleaseRevision = 0;
	pcsmi_sas_cntlr_config->usBIOSMajorRevision = 
				asd->hw_profile.bios_maj_ver; 
	pcsmi_sas_cntlr_config->usBIOSMinorRevision = 
				asd->hw_profile.bios_min_ver; 
	pcsmi_sas_cntlr_config->usBIOSBuildRevision = 
				asd->hw_profile.bios_bld_num & 0xffff; 
	pcsmi_sas_cntlr_config->usBIOSReleaseRevision = 
				asd->hw_profile.bios_bld_num >> 16; 
	
	pcsmi_sas_cntlr_config->uControllerFlags = CSMI_SAS_CNTLR_SAS_HBA;
					   //|CSMI_SAS_CNTLR_SATA_HBA;//TBD
	pcsmi_sas_cntlr_config->usRromMajorRevision = 0;
	pcsmi_sas_cntlr_config->usRromMinorRevision = 0;
	pcsmi_sas_cntlr_config->usRromBuildRevision = 0;
	pcsmi_sas_cntlr_config->usRromReleaseRevision = 0;
	pcsmi_sas_cntlr_config->usRromBIOSMajorRevision = 0;
	pcsmi_sas_cntlr_config->usRromBIOSMinorRevision = 0;
	pcsmi_sas_cntlr_config->usRromBIOSBuildRevision = 0;
	pcsmi_sas_cntlr_config->usRromBIOSReleaseRevision = 0;
	
	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_cntlr_config_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_SUCCESS);
}	

static int asd_csmi_get_cntlr_status(unsigned long arg)
{
	CSMI_SAS_CNTLR_STATUS_BUFFER csmi_sas_cntlr_status_buffer;
	PCSMI_SAS_CNTLR_STATUS pcsmi_sas_cntlr_status;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	int csmi_user_buf_len;
	int err;
	
	csmi_user_buf_len = sizeof(csmi_sas_cntlr_status_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_cntlr_status_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_cntlr_status_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_cntlr_status_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}

	pcsmi_sas_cntlr_status = &csmi_sas_cntlr_status_buffer.Status;

	/* TBD: Always return status 'good' by default */
	pcsmi_sas_cntlr_status->uStatus = 
				CSMI_SAS_CNTLR_STATUS_GOOD;

	pcsmi_sas_cntlr_status->uOfflineReason = 0;/*TBD*/

	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_cntlr_status_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_SUCCESS);
}

static int asd_csmi_get_phy_info(unsigned long arg)
{
	CSMI_SAS_PHY_INFO_BUFFER csmi_sas_phy_info_buffer;
	PCSMI_SAS_PHY_INFO pcsmi_sas_phy_info;
	PCSMI_SAS_PHY_ENTITY pcsmi_sas_phy_entity;
	PCSMI_SAS_IDENTIFY pcsmi_sas_identify;
	PCSMI_SAS_IDENTIFY pcsmi_sas_identify_attached;
	PIOCTL_HEADER pioctl_header;
	u_long flags;
	struct asd_softc *asd;
	struct asd_phy *phy;
	struct asd_port *port;
	struct asd_target *targ;
	int csmi_user_buf_len;
	int err, phy_num;

	csmi_user_buf_len = sizeof(csmi_sas_phy_info_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_phy_info_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_phy_info_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_phy_info_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}

	asd_lock(asd, &flags);
	pcsmi_sas_phy_info = &csmi_sas_phy_info_buffer.Information;
	pcsmi_sas_phy_info->bNumberOfPhys = asd->hw_profile.max_phys;
	
	for (phy_num = 0; 
	     phy_num < pcsmi_sas_phy_info->bNumberOfPhys; 
	     phy_num++) {

		pcsmi_sas_phy_entity = &pcsmi_sas_phy_info->Phy[phy_num];
		pcsmi_sas_identify = &pcsmi_sas_phy_entity->Identify;
		pcsmi_sas_identify_attached = &pcsmi_sas_phy_entity->Attached;
		phy = asd->phy_list[phy_num];
		pcsmi_sas_identify->bDeviceType = 
			(phy->state == ASD_PHY_UNUSED)?
			CSMI_SAS_PHY_UNUSED: CSMI_SAS_END_DEVICE;

		asd_ctl_map_protocol_to_csmi(phy->attr, 
				&pcsmi_sas_identify->bInitiatorPortProtocol);

		pcsmi_sas_identify->bTargetPortProtocol = 0;
		
		memcpy(pcsmi_sas_identify->bSASAddress, phy->sas_addr, 
				SAS_ADDR_LEN) ;
		
		pcsmi_sas_identify->bPhyIdentifier = phy->id;
		//pcsmi_sas_identify->bSignalClass;
		
		asd_ctl_map_conn_rate_to_csmi(phy->conn_rate, 
				&pcsmi_sas_phy_entity->bNegotiatedLinkRate);
		
		asd_ctl_map_link_rate_to_csmi(phy->min_link_rate, 
				&pcsmi_sas_phy_entity->bMinimumLinkRate);
		
		asd_ctl_map_link_rate_to_csmi(phy->max_link_rate, 
				&pcsmi_sas_phy_entity->bMaximumLinkRate);
		
		pcsmi_sas_phy_entity->bPhyChangeCount = phy->brdcst_rcvd_cnt;
		/* TBD: autodiscover
		pcsmi_sas_phy_entity->bAutoDiscover
		*/
		if (phy->src_port != NULL) {
			port = phy->src_port;

			pcsmi_sas_phy_entity->bPortIdentifier = port->id;
	
			asd_ctl_map_targ_typ_to_csmi(port,
			       	&pcsmi_sas_identify_attached->bDeviceType);
		
			/*asd_ctl_map_protocol_to_csmi(port->attr, 
			  &pcsmi_sas_identify_attached->bInitiatorPortProtocol);
			*/
			if (!list_empty(&port->targets)) {
				list_for_each_entry(targ, &port->targets, 
					    all_domain_targets) {
					/* 
					 * For now map initiator port protocol
					 * and target port protocol using
					 * transport_type only.
					 */
					asd_ctl_map_transport_to_csmi(
						targ->transport_type,
						&pcsmi_sas_identify_attached->
							bInitiatorPortProtocol);
					asd_ctl_map_transport_to_csmi(
						targ->transport_type,
						&pcsmi_sas_identify_attached->
							bTargetPortProtocol);
					if (targ->transport_type
					   == ASD_TRANSPORT_ATA) {
						memset(pcsmi_sas_identify->
							bSASAddress,
							0, SAS_ADDR_LEN) ;
					}
				}
			}
			memcpy(pcsmi_sas_identify_attached->bSASAddress, 
			       port->attached_sas_addr, 
			       SAS_ADDR_LEN );
			pcsmi_sas_identify_attached->bPhyIdentifier = port->id;
		}
	}
	asd_unlock(asd, &flags);

	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_phy_info_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_SUCCESS);
}

static int asd_csmi_set_phy_info(unsigned long arg)
{
	CSMI_SAS_SET_PHY_INFO_BUFFER csmi_sas_set_phy_info_buffer;
	PCSMI_SAS_SET_PHY_INFO pcsmi_sas_set_phy_info;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_phy *phy;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	csmi_user_buf_len = sizeof(csmi_sas_set_phy_info_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_set_phy_info_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_set_phy_info_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_set_phy_info_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}
	pcsmi_sas_set_phy_info = &csmi_sas_set_phy_info_buffer.Information;
	
	phy = asd_ctl_get_phy(asd, 
			pcsmi_sas_set_phy_info->bPhyIdentifier);
	if (phy == NULL) {
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_set_phy_info_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_PHY_DOES_NOT_EXIST);
		return -EINVAL;
	}
	
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	down_interruptible(&pcsmi_mgmt->sem);

	retcode = CSMI_SAS_STATUS_SUCCESS;
	err = 0;

	switch (pcsmi_sas_set_phy_info->bNegotiatedLinkRate) {
	case CSMI_SAS_LINK_RATE_NEGOTIATE:
	{
		if ((pcsmi_sas_set_phy_info->bProgrammedMinimumLinkRate 
		     == CSMI_SAS_PROGRAMMED_LINK_RATE_UNCHANGED)
		  && (pcsmi_sas_set_phy_info->bProgrammedMaximumLinkRate 
		     == CSMI_SAS_PROGRAMMED_LINK_RATE_UNCHANGED)) {
			
			retcode = CSMI_SAS_STATUS_SUCCESS;
			break;
		}

		if (pcsmi_sas_set_phy_info->bProgrammedMinimumLinkRate 
		  > pcsmi_sas_set_phy_info->bProgrammedMaximumLinkRate) {
			retcode = CSMI_SAS_LINK_RATE_OUT_OF_RANGE;
			err = -EINVAL;
			break;
		}
		
		if ((err = asd_ctl_map_link_rate_from_csmi(
			pcsmi_sas_set_phy_info->bProgrammedMinimumLinkRate,
			&phy->min_link_rate)) < 0) {
			retcode = CSMI_SAS_LINK_RATE_OUT_OF_RANGE;
			err = -EINVAL;
			break;
		}

		if ((err = asd_ctl_map_link_rate_from_csmi(
			pcsmi_sas_set_phy_info->bProgrammedMaximumLinkRate,
			&phy->max_link_rate)) < 0) {
			retcode = CSMI_SAS_LINK_RATE_OUT_OF_RANGE;
			err = -EINVAL;
			break;
		}

		/* TBD: Signal Class */
		asd_ctl_switch_pat_gen_mode(asd, phy);
		err = asd_ctl_issue_phy_ctl(asd, phy,
				&csmi_sas_set_phy_info_buffer.IoctlHeader,
				ENABLE_PHY);
		if (err)
			retcode = CSMI_SAS_PHY_INFO_NOT_CHANGEABLE;
		else
			retcode = CSMI_SAS_PHY_INFO_CHANGED;
		
		break;
	}
	case CSMI_SAS_LINK_RATE_PHY_DISABLED:
	default:
		/* TBD: code to disable phy */
		break;
		
	}
	
	/* return status */
	asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_set_phy_info_buffer, 
			csmi_user_buf_len, 
			retcode);
	up(&pcsmi_mgmt->sem);
	return err;
}

static int asd_csmi_get_link_errors(unsigned long arg)
{
	PCSMI_SAS_LINK_ERRORS_BUFFER pcsmi_sas_link_errors_buffer;
	PCSMI_SAS_LINK_ERRORS pcsmi_sas_link_errors;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_phy *phy;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	csmi_user_buf_len = sizeof(*pcsmi_sas_link_errors_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	/* The xxx_buffer data structure needs to be shared between
	 * the request and done routines, hence we allocate it dynamically.
	 */
	if ((pcsmi_sas_link_errors_buffer = asd_alloc_mem(
				csmi_user_buf_len,
				GFP_ATOMIC)) == NULL) {
		return -ENOMEM;
	}
	
	if ((err = copy_from_user(pcsmi_sas_link_errors_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		asd_free_mem(pcsmi_sas_link_errors_buffer);
		return -EFAULT;
	}
	
	pioctl_header = &pcsmi_sas_link_errors_buffer->IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
				pcsmi_sas_link_errors_buffer, 
				csmi_user_buf_len, 
				CSMI_SAS_STATUS_INVALID_PARAMETER);
		asd_free_mem(pcsmi_sas_link_errors_buffer);
		return -ENODEV;
	}
	pcsmi_sas_link_errors = &pcsmi_sas_link_errors_buffer->Information;

	phy = asd_ctl_get_phy(asd, 
			pcsmi_sas_link_errors->bPhyIdentifier);
	if (phy == NULL) {
		err = asd_ctl_copy_to_user((void *)arg, 
				pcsmi_sas_link_errors_buffer, 
				csmi_user_buf_len, 
				CSMI_SAS_PHY_DOES_NOT_EXIST);
		asd_free_mem(pcsmi_sas_link_errors_buffer);
		return -EINVAL;
	}

	err = asd_ctl_issue_link_adm(asd, 
			phy->src_port, 
			pcsmi_sas_link_errors_buffer, 
			GET_LINK_ERR_CNT);
	
	if (pcsmi_sas_link_errors->bResetCounts 
	   == CSMI_SAS_LINK_ERROR_RESET_COUNTS) {
		err |= asd_ctl_issue_link_adm(asd, 
				phy->src_port, 
				pcsmi_sas_link_errors_buffer,
				RESET_LINK_ERR_CNT);
	}

	if (err)
		retcode = CSMI_SAS_STATUS_FAILED;
	else
		retcode = CSMI_SAS_STATUS_SUCCESS;
	
	/* return status */	
	asd_ctl_copy_to_user((void *)arg, 
			pcsmi_sas_link_errors_buffer, 
			csmi_user_buf_len, 
			retcode);
	asd_free_mem(pcsmi_sas_link_errors_buffer);
	return err;
}

static int asd_csmi_smp_passthru(unsigned long arg)
{
	CSMI_SAS_SMP_PASSTHRU_BUFFER csmi_sas_smp_passthru_buffer;
	PCSMI_SAS_SMP_PASSTHRU pcsmi_sas_smp_passthru;
	PCSMI_SAS_SMP_RESPONSE pcsmi_sas_smp_response;
	PCSMI_SAS_SMP_REQUEST pcsmi_sas_smp_request;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_phy *phy;
	struct asd_port *port;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	phy = NULL;
	port = NULL;
	csmi_user_buf_len = sizeof(csmi_sas_smp_passthru_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_smp_passthru_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_smp_passthru_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_smp_passthru_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}
	pcsmi_sas_smp_passthru = &csmi_sas_smp_passthru_buffer.Parameters;
	pcsmi_sas_smp_response = &pcsmi_sas_smp_passthru->Response;
	pcsmi_sas_smp_request = &pcsmi_sas_smp_passthru->Request;

	if ((retcode = asd_ctl_check_phy_port(asd, 
			pcsmi_sas_smp_passthru->bPhyIdentifier,
			pcsmi_sas_smp_passthru->bPortIdentifier,
			&phy,
			&port))) {
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_smp_passthru_buffer, 
			csmi_user_buf_len, 
			retcode);
		return -EINVAL;
	}
	
	if (pcsmi_sas_smp_request->bFrameType != SMP_REQUEST_FRAME) {
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_smp_passthru_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -EINVAL;
	}

	/* build and issue scb */	
	err = asd_ctl_issue_smp_pt(
			asd, port, 
			&csmi_sas_smp_passthru_buffer);
				
	if (err)
		retcode = CSMI_SAS_STATUS_FAILED;
	else
		retcode = CSMI_SAS_STATUS_SUCCESS;
	
	/* return status */	
	asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_smp_passthru_buffer, 
			csmi_user_buf_len, 
			retcode);
	return err;
}

static int asd_csmi_ssp_passthru(unsigned long arg)
{
	PCSMI_SAS_SSP_PASSTHRU_BUFFER pcsmi_sas_ssp_passthru_buffer;
	PCSMI_SAS_SSP_PASSTHRU pcsmi_sas_ssp_passthru;
	PCSMI_SAS_SSP_PASSTHRU_STATUS pcsmi_sas_ssp_passthru_status;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_phy *phy;
	struct asd_port *port;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	phy = NULL;
	port = NULL;
	
	/* 
	 * Leave the data buffer at the end of the struct so as not to 
	 * overwrite the contents filled in by asd_ctl_issue_xxx() routine.
	 */
	csmi_user_buf_len = offsetof(CSMI_SAS_SSP_PASSTHRU_BUFFER, 
				     bDataBuffer) ;
	/* The xxx_buffer data structure needs to be shared between
	 * the request and done routines, hence we allocate it dynamically.
	 */
	if ((pcsmi_sas_ssp_passthru_buffer = asd_alloc_mem(
				csmi_user_buf_len,
				GFP_ATOMIC)) == NULL) {
		return -ENOMEM;
	}

	if ((err = copy_from_user(pcsmi_sas_ssp_passthru_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		asd_free_mem(pcsmi_sas_ssp_passthru_buffer);
		return -EFAULT;
	}
	
	pioctl_header = &pcsmi_sas_ssp_passthru_buffer->IoctlHeader;
	pcsmi_sas_ssp_passthru = &pcsmi_sas_ssp_passthru_buffer->Parameters;
	pcsmi_sas_ssp_passthru_status = &pcsmi_sas_ssp_passthru_buffer->Status;

	if ((err = asd_ctl_check_buf_len(arg, 
			  csmi_user_buf_len
			  + pcsmi_sas_ssp_passthru->uDataLength))) {
		asd_free_mem(pcsmi_sas_ssp_passthru_buffer);
		return -EINVAL;
	}
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
				pcsmi_sas_ssp_passthru_buffer, 
				csmi_user_buf_len,
				CSMI_SAS_STATUS_INVALID_PARAMETER);
		asd_free_mem(pcsmi_sas_ssp_passthru_buffer);
		return -ENODEV;
	}
	
	if ((retcode = asd_ctl_check_phy_port(asd, 
			pcsmi_sas_ssp_passthru->bPhyIdentifier,
			pcsmi_sas_ssp_passthru->bPortIdentifier,
			&phy, &port))) {
		asd_ctl_copy_to_user((void *)arg, 
			pcsmi_sas_ssp_passthru_buffer, 
			csmi_user_buf_len, 
			retcode);
		return -EINVAL;
	}
	
	/* build and issue scb */
	err = asd_ctl_issue_ssp_pt(asd, port, 
			pcsmi_sas_ssp_passthru_buffer, arg);

	if (err) 
		retcode = CSMI_SAS_STATUS_FAILED;
	else
		retcode = CSMI_SAS_STATUS_SUCCESS;
	
	/* return status */	
	asd_ctl_copy_to_user((void *)arg, 
			pcsmi_sas_ssp_passthru_buffer, 
			csmi_user_buf_len,
			retcode);
	asd_free_mem(pcsmi_sas_ssp_passthru_buffer);
	return err;
}

static int asd_csmi_stp_passthru(unsigned long arg)
{
	PCSMI_SAS_STP_PASSTHRU_BUFFER pcsmi_sas_stp_passthru_buffer;
	PCSMI_SAS_STP_PASSTHRU pcsmi_sas_stp_passthru;
	PCSMI_SAS_STP_PASSTHRU_STATUS pcsmi_sas_stp_passthru_status;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_phy *phy;
	struct asd_port *port;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	phy = NULL;
	port = NULL;
	
	/* 
	 * Leave the data buffer at the end of the struct so as not to 
	 * overwrite the contents filled in by asd_ctl_issue_xxx() routine.
	 */
	csmi_user_buf_len = offsetof(CSMI_SAS_STP_PASSTHRU_BUFFER, 
				     bDataBuffer) ;
	
	/* The xxx_buffer data structure needs to be shared between
	 * the request and done routines, hence we allocate it dynamically.
	 */
	if ((pcsmi_sas_stp_passthru_buffer = asd_alloc_mem(
				csmi_user_buf_len,
				GFP_ATOMIC)) == NULL) {
		return -ENOMEM;
	}
	if ((err = copy_from_user(pcsmi_sas_stp_passthru_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		asd_free_mem(pcsmi_sas_stp_passthru_buffer);
		return -EFAULT;
	}
	
	pioctl_header = &pcsmi_sas_stp_passthru_buffer->IoctlHeader;
	pcsmi_sas_stp_passthru = &pcsmi_sas_stp_passthru_buffer->Parameters;
	pcsmi_sas_stp_passthru_status =  &pcsmi_sas_stp_passthru_buffer->Status;
	
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len
			  + pcsmi_sas_stp_passthru->uDataLength))) {
		asd_free_mem(pcsmi_sas_stp_passthru_buffer);
		return -EINVAL;
	}
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	
	if (asd == NULL) {
		/* Requested controller not found */
		err = asd_ctl_copy_to_user((void *)arg, 
				pcsmi_sas_stp_passthru_buffer, 
				csmi_user_buf_len,
				CSMI_SAS_STATUS_INVALID_PARAMETER);
		asd_free_mem(pcsmi_sas_stp_passthru_buffer);
		return -ENODEV;
	}

	if ((retcode = asd_ctl_check_phy_port(asd, 
			pcsmi_sas_stp_passthru->bPhyIdentifier,
			pcsmi_sas_stp_passthru->bPortIdentifier,
			&phy,
			&port))) {
		asd_ctl_copy_to_user((void *)arg, 
			pcsmi_sas_stp_passthru_buffer, 
			csmi_user_buf_len, 
			retcode);
		return -EINVAL;
	}
	/* build and issue scb */	
	err = asd_ctl_issue_stp_pt(
			asd, port, 
			pcsmi_sas_stp_passthru_buffer,
			arg);

	if (err)
		retcode = CSMI_SAS_STATUS_FAILED;
	else
		retcode = CSMI_SAS_STATUS_SUCCESS;
	
	/* return status */	
	asd_ctl_copy_to_user((void *)arg, 
			pcsmi_sas_stp_passthru_buffer, 
			csmi_user_buf_len,
			retcode);
	asd_free_mem(pcsmi_sas_stp_passthru_buffer);
	return err;
	
}

static int asd_csmi_get_sata_signature(unsigned long arg)
{
	CSMI_SAS_SATA_SIGNATURE_BUFFER csmi_sas_sata_signature_buffer;
	PCSMI_SAS_SATA_SIGNATURE pcsmi_sas_sata_signature;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_phy *phy;
	int csmi_user_buf_len;
	int err;
	
	phy = NULL;
	csmi_user_buf_len = sizeof(csmi_sas_sata_signature_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_sata_signature_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_sata_signature_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_sata_signature_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}
	pcsmi_sas_sata_signature = &csmi_sas_sata_signature_buffer.Signature;
	phy = asd_ctl_get_phy(asd, 
			pcsmi_sas_sata_signature->bPhyIdentifier);
	if (phy == NULL) {
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_sata_signature_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_PHY_DOES_NOT_EXIST);
		return -EINVAL;
	}
	
	/*
	 * TODO: is this right??  Shouldn't we check the target structure
	 * instead??
	 */
	if (phy->src_port == NULL 
	   || phy->src_port->link_type != ASD_LINK_SATA) {
		return asd_ctl_copy_to_user((void *)arg, 
				&csmi_sas_sata_signature_buffer, 
				csmi_user_buf_len, 
				CSMI_SAS_NO_SATA_DEVICE);
	}

	memcpy(pcsmi_sas_sata_signature->bSignatureFIS, 
	       phy->bytes_dmaed_rcvd.initial_fis_rcvd.fis, 20);
	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_sata_signature_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_SUCCESS);
			//CSMI_SAS_NO_SATA_SIGNATURE);
	
}

static int asd_csmi_get_scsi_address(unsigned long arg)
{
	CSMI_SAS_GET_SCSI_ADDRESS_BUFFER csmi_sas_get_scsi_address_buffer;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_port *port;
	struct asd_target *targ; 
	int csmi_user_buf_len;
	int err, port_num;
	
	port = NULL;
	csmi_user_buf_len = sizeof(csmi_sas_get_scsi_address_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_get_scsi_address_buffer, 
				(void *)arg, 
				csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_get_scsi_address_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_get_scsi_address_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -EINVAL;
	}
	
	port = NULL;
	targ = NULL;
	for (port_num = 0; port_num < ASD_MAX_PORTS; port_num++) {
		targ = asd_get_sas_target_from_sasaddr(asd, 
				asd->port_list[port_num], 
				csmi_sas_get_scsi_address_buffer.bSASAddress);
		if (targ != NULL) {
			if (targ->management_type != ASD_DEVICE_END) {
				asd_ctl_copy_to_user((void *)arg, 
					&csmi_sas_get_scsi_address_buffer, 
					csmi_user_buf_len, 
					CSMI_SAS_NOT_AN_END_DEVICE);
				return -EINVAL;
			}
			port = asd->port_list[port_num]; 
			break;
		}
	}

	if (port == NULL || targ == NULL) {
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_get_scsi_address_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_NO_SCSI_ADDRESS);
		return -EINVAL;
	}
	
	if (asd_get_os_platform_map_from_sasaddr(asd, port, 
		csmi_sas_get_scsi_address_buffer.bSASAddress,
		csmi_sas_get_scsi_address_buffer.bSASLun,
		&csmi_sas_get_scsi_address_buffer.bHostIndex,
		&csmi_sas_get_scsi_address_buffer.bPathId,
		&csmi_sas_get_scsi_address_buffer.bTargetId,
		&csmi_sas_get_scsi_address_buffer.bLun) != 0) {

		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_get_scsi_address_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_NO_SCSI_ADDRESS);
		return -EINVAL;
	}
	
	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_get_scsi_address_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_SUCCESS);
	
}

static int asd_csmi_get_device_address(unsigned long arg)
{
	CSMI_SAS_GET_DEVICE_ADDRESS_BUFFER csmi_sas_get_device_address_buffer;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_port *port;
	int csmi_user_buf_len;
	int err;
	
	port = NULL;
	csmi_user_buf_len = sizeof(csmi_sas_get_device_address_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_get_device_address_buffer, 
			(void *)arg, 
		  	csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_get_device_address_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_get_device_address_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -EINVAL;
	}
	
	if ((port = asd_get_sas_addr_from_platform_map(asd,
		csmi_sas_get_device_address_buffer.bHostIndex,
		csmi_sas_get_device_address_buffer.bPathId,
		csmi_sas_get_device_address_buffer.bTargetId,
		csmi_sas_get_device_address_buffer.bLun,
		csmi_sas_get_device_address_buffer.bSASAddress,
		csmi_sas_get_device_address_buffer.bSASLun)) == NULL) {
	
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_get_device_address_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_NO_DEVICE_ADDRESS);
		return -EINVAL;
	}

	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_get_device_address_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_SUCCESS);
}

static int
asd_csmi_sas_task_mgmt(unsigned long arg)
{
	PCSMI_SAS_SSP_TASK_IU_BUFFER 	 task_iu_buf;
	PCSMI_SAS_SSP_TASK_IU 		 task_iu;
	PIOCTL_HEADER 			 pioctl_header;
	struct asd_softc 		*asd;
	struct asd_csmi_mgmt 		*pcsmi_mgmt;
	struct scb			*scb;
	int 				 err;
	int 				 retcode;
	int 				 csmi_user_buf_len;
	u_long				 flags;
	
	err = 0;
	csmi_user_buf_len = sizeof(*task_iu_buf);

	/* The xxx_buffer data structure needs to be shared between
	 * the request and done routines, hence we allocate it dynamically.
	 */
	if ((task_iu_buf = asd_alloc_mem(csmi_user_buf_len,
					 GFP_ATOMIC)) == NULL) {
		return (-ENOMEM);
	}

	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len)) != 0) {
		asd_free_mem(task_iu_buf);
		return (-EINVAL);
	}
	
	if ((err = copy_from_user(task_iu_buf, (void *) arg, 
				  csmi_user_buf_len))) {
		asd_free_mem(task_iu_buf);
		return (-EFAULT);
	}
	pioctl_header = &task_iu_buf->IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *) arg, 
				     task_iu_buf,
				     csmi_user_buf_len, 
				     CSMI_SAS_STATUS_INVALID_PARAMETER);
		asd_free_mem(task_iu_buf);
		return (-EINVAL);
	}
	task_iu = &task_iu_buf->Parameters;
	pcsmi_mgmt =  &asd->asd_ctl_csmi_internal.mgmt;

	down_interruptible(&pcsmi_mgmt->sem);

	/* We serialized SSP TMF to the target. */
	if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR,
			     &pcsmi_mgmt->busy)) { 
		retcode = CSMI_SAS_STATUS_FAILED;
		err = -EINVAL;
		goto exit;
	}
	/* 
	 * CSMI Spec. restrict only flags can be set at a time, either
	 * CSMI_SAS_TAK_IU or CSMI_SAS_HARD_RESET_SEQUENCE.
	 */
	 if (((task_iu->uFlags & CSMI_SAS_TASK_IU) != 0) && 
	    ((task_iu->uFlags & CSMI_SAS_HARD_RESET_SEQUENCE) != 0)) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retcode = CSMI_SAS_STATUS_INVALID_PARAMETER;
		err = -EINVAL;
		goto exit;
	}
	
	if (((task_iu->uFlags & CSMI_SAS_TASK_IU) != 0) && 
	     (task_iu->bTaskManagementFunction == 0)) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retcode = CSMI_SAS_STATUS_INVALID_PARAMETER;
		err = -EINVAL;
		goto exit;
	}

	asd_lock(asd, &flags);
	if ((scb = asd_hwi_get_scb(asd, 1)) == NULL) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retcode = CSMI_SAS_STATUS_FAILED;
		err = -EINVAL;
		asd_unlock(asd, &flags);
		goto exit;
	}
	asd_unlock(asd, &flags);

	/* Fill in the common SCB information. */
	scb->io_ctx = (void *) task_iu_buf;

	/* Check if CSMI requested us to perform Hard Reser or TMF. */
	if ((task_iu->uFlags & CSMI_SAS_HARD_RESET_SEQUENCE) != 0) {
		retcode = asd_ctl_handle_hard_reset(asd, scb);
	}
	else {
		retcode = asd_ctl_handle_task_mgmt(asd, scb,
					   task_iu->bTaskManagementFunction);
	}
	
	asd_lock(asd, &flags);
	asd_hwi_free_scb(asd, scb);
	asd_unlock(asd, &flags);

exit:
	/* return status */	
	asd_ctl_copy_to_user((void *) arg, task_iu_buf, 
			     csmi_user_buf_len, retcode);
	asd_free_mem(task_iu_buf);
	up(&pcsmi_mgmt->sem);

	return (err);
}

static int asd_csmi_sas_get_connector_info(unsigned long arg)
{
	CSMI_SAS_CONNECTOR_INFO_BUFFER csmi_sas_connector_info_buffer;
	PCSMI_SAS_CONNECTOR_INFO pcsmi_sas_connector_info;
	PIOCTL_HEADER pioctl_header;
	struct asd_conn_map_format *pconn_map;
	struct asd_conn_desc_format *pconn_desc;
	struct asd_nd_phydesc_format *pnd_phy_desc;
	struct asd_softc *asd;
	uint8_t *pconn_buf;
	uint8_t *pnode_buf;
	uint32_t conn_size, noded_size;
	int csmi_user_buf_len;
	int err, phy_num;
	
	csmi_user_buf_len = sizeof(csmi_sas_connector_info_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_connector_info_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_connector_info_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_connector_info_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}

	pconn_buf = NULL;
	pnode_buf = NULL;
	conn_size = 0;
	noded_size = 0;
	err = asd_hwi_get_nv_conn_info(asd, &pconn_buf, &conn_size, 
				 &pnode_buf, &noded_size);

	if (err != 0 || pconn_buf == NULL || pnode_buf == NULL 
	    || conn_size == 0 || noded_size == 0) {
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_connector_info_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_FAILED);
		return -EINVAL;
	}

	pconn_map = (struct asd_conn_map_format *)pconn_buf;

	for (phy_num = 0; 
	     phy_num < asd->hw_profile.max_phys; 
	     phy_num++) {
		pcsmi_sas_connector_info = 
			&csmi_sas_connector_info_buffer.Reference[phy_num];

		pnd_phy_desc = asd_ctl_get_node_phyd(pnode_buf, 
						noded_size, phy_num);
		if (pnd_phy_desc != NULL) {

			/* TBD: NODEATTACH & UATTACH */
			if (pnd_phy_desc->type == NV_PD_CONATTACH) {
				pconn_desc = (struct asd_conn_desc_format *) 
				     (pconn_buf + sizeof(*pconn_map) +
				     (pconn_map->conn_desc_size 
				     * pnd_phy_desc->att_type.catt.idx));

				asd_ctl_map_conn_type_to_csmi(
					pconn_desc->type,
					pnd_phy_desc->att_type.catt.lane & 0x7F,
					&pcsmi_sas_connector_info->uPinout);
			
				memcpy(pcsmi_sas_connector_info->bConnector, 
				       pconn_desc->inst_name, 
				       NV_CONND_INST_NAME_SIZE);
			
				asd_ctl_map_conn_loc_to_csmi(
					pconn_desc->location,
					&pcsmi_sas_connector_info->bLocation); 
			}
		}
	}

	asd_free_mem(pconn_buf);
	asd_free_mem(pnode_buf);

	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_connector_info_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_SUCCESS);
}

static int asd_csmi_sas_phy_control(unsigned long arg)
{
	CSMI_SAS_PHY_CONTROL_BUFFER csmi_sas_pc_buf_temp;
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer;
	PCSMI_SAS_PHY_CONTROL pcsmi_sas_phy_control;
	//PCSMI_SAS_CHARACTER pcsmi_sas_character;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	struct asd_phy *phy;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	int csmi_ubuf_len_temp;
	int max_ctls;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	phy = NULL;
	csmi_user_buf_len = 0;
	/* 
	 * find and validate the control length using temp buffer 
	 * before allocating actual buffer 
	 */
	csmi_ubuf_len_temp = offsetof(CSMI_SAS_PHY_CONTROL_BUFFER, 
				     Control);
	
	if ((err = copy_from_user(&csmi_sas_pc_buf_temp, (void *)arg, 
				  csmi_ubuf_len_temp))) {
		return -EFAULT;
	}

	pioctl_header = &csmi_sas_pc_buf_temp.IoctlHeader;
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_pc_buf_temp, 
			csmi_ubuf_len_temp, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}

	phy = asd_ctl_get_phy(asd, 
			csmi_sas_pc_buf_temp.bPhyIdentifier);
	if (phy == NULL) {
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_pc_buf_temp, 
			csmi_ubuf_len_temp, 
			CSMI_SAS_PHY_DOES_NOT_EXIST);
		return -EINVAL;
	}
	if (csmi_sas_pc_buf_temp.uFunction
	   == CSMI_SAS_PC_GET_PHY_SETTINGS) {
		max_ctls = 0;
		if (phy->attr & ASD_SSP_INITIATOR) {
			max_ctls++;
			if (phy->min_link_rate != phy->max_link_rate) {
				max_ctls++;
			}
		}
		if (phy->attr & ASD_STP_INITIATOR) {
			max_ctls++;
		}
		if ((csmi_sas_pc_buf_temp.bNumberOfControls <  max_ctls)
		   || (csmi_sas_pc_buf_temp.usLengthOfControl <  
			max_ctls * sizeof(*pcsmi_sas_phy_control))) {
			csmi_sas_pc_buf_temp.bNumberOfControls = 
							max_ctls;
			csmi_sas_pc_buf_temp.usLengthOfControl = 
				max_ctls * sizeof(*pcsmi_sas_phy_control); 
			asd_ctl_copy_to_user((void *)arg, 
				&csmi_sas_pc_buf_temp, 
				csmi_ubuf_len_temp, 
				CSMI_SAS_STATUS_INVALID_PARAMETER);
			return -EINVAL;
		}
	}

	if (csmi_sas_pc_buf_temp.usLengthOfControl !=  
	    (csmi_sas_pc_buf_temp.bNumberOfControls 
	    * sizeof(*pcsmi_sas_phy_control))) {
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_pc_buf_temp, 
			csmi_ubuf_len_temp, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -EINVAL;
	}

	/* 
	 * Now we know the actual length of buffer to allocate including 
	 * control length.
	 */
	csmi_user_buf_len = csmi_ubuf_len_temp 
			    + csmi_sas_pc_buf_temp.usLengthOfControl;
	
	if ((pcsmi_sas_phy_control_buffer = asd_alloc_mem(
				csmi_user_buf_len,
				GFP_ATOMIC)) == NULL) {
		return -ENOMEM;
	}
	
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		asd_free_mem(pcsmi_sas_phy_control_buffer);
		return -EINVAL;
	}
	
	if ((err = copy_from_user(pcsmi_sas_phy_control_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		asd_free_mem(pcsmi_sas_phy_control_buffer);
		return -EFAULT;
	}
	
	pioctl_header = &pcsmi_sas_phy_control_buffer->IoctlHeader;

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;

	down_interruptible(&pcsmi_mgmt->sem);

	err = 0;
	if (pcsmi_sas_phy_control_buffer->uFunction
	   == CSMI_SAS_PC_GET_PHY_SETTINGS) {
		err = asd_ctl_pc_get_phy_settings(asd, phy, 
				pcsmi_sas_phy_control_buffer, &max_ctls);
		pcsmi_sas_phy_control_buffer->bNumberOfControls = 
						max_ctls;
		pcsmi_sas_phy_control_buffer->usLengthOfControl = 
			max_ctls * sizeof(*pcsmi_sas_phy_control); 
	} else {
		/* build and issue scb */
		err = asd_ctl_process_phy_ctl(asd, phy,
					pcsmi_sas_phy_control_buffer);
	}

	if (err)
		retcode = CSMI_SAS_STATUS_FAILED;
	else
		retcode = CSMI_SAS_STATUS_SUCCESS;
		
	/* return status */
	asd_ctl_copy_to_user((void *)arg, 
			pcsmi_sas_phy_control_buffer, 
			csmi_user_buf_len, 
			retcode);
	asd_free_mem(pcsmi_sas_phy_control_buffer);

	up(&pcsmi_mgmt->sem);

	return err;
}

#ifdef ASD_ADPT_CSMI_EXTENSION
/* Adaptec extension IOCTL handler routines */
static int asd_csmi_sas_get_adpt_cntlr_conf(unsigned long arg)
{
	CSMI_SAS_GET_ADPT_CNTLR_CONFIG_BUFFER csmi_sas_adpt_cntlr_conf_buffer;
	PCSMI_SAS_GET_ADPT_CNTLR_CONFIG pcsmi_sas_adpt_cntlr_config;
	PIOCTL_HEADER pioctl_header;
	struct asd_softc *asd;
	u_long flags;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	csmi_user_buf_len = sizeof(csmi_sas_adpt_cntlr_conf_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_adpt_cntlr_conf_buffer, 
				  (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_adpt_cntlr_conf_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_adpt_cntlr_conf_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}

	pcsmi_sas_adpt_cntlr_config = 
			&csmi_sas_adpt_cntlr_conf_buffer.Configuration;

	asd_lock(asd, &flags);
	pcsmi_sas_adpt_cntlr_config->usPCIVendorID = asd_pci_dev(asd)->vendor;
	pcsmi_sas_adpt_cntlr_config->usPCIDeviceID = asd_pci_dev(asd)->device;
	pcsmi_sas_adpt_cntlr_config->usPCISubsystemVendorID =  
					asd_pci_dev(asd)->subsystem_vendor;
	pcsmi_sas_adpt_cntlr_config->usPCISubsystemID = 
					asd_pci_dev(asd)->subsystem_device;
		
	if (asd->hw_profile.flash_present) {
		pcsmi_sas_adpt_cntlr_config->usFlashManufacturerID =
					asd->hw_profile.flash_manuf_id;
		pcsmi_sas_adpt_cntlr_config->usFlashDeviceID = 
					asd->hw_profile.flash_dev_id;
		if (asd->hw_profile.flash_wr_prot) {
			pcsmi_sas_adpt_cntlr_config->uControllerFlags 
				= CSMI_ADPT_CNTLR_FLASH_READONLY;
		}
		retcode = CSMI_SAS_STATUS_SUCCESS;
	} else {
		pcsmi_sas_adpt_cntlr_config->usFlashManufacturerID 
				= CSMI_SAS_APDT_INVALID_FLASH_MANUFACTURER_ID; 
		pcsmi_sas_adpt_cntlr_config->usFlashDeviceID 
				= CSMI_SAS_ADPT_INVALID_FLASH_DEVICE_ID; 
		pcsmi_sas_adpt_cntlr_config->uControllerFlags 
				= CSMI_ADPT_CNTLR_FLASH_NOT_PRESENT; 
		retcode = CSMI_SAS_STATUS_FAILED;
	}
	asd_unlock(asd, &flags);
	
	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_adpt_cntlr_conf_buffer, 
			csmi_user_buf_len, 
			retcode);
}

static int asd_csmi_sas_get_nv_seg_prop(unsigned long arg)
{
	CSMI_SAS_NV_SEGMENT_PROPERTIES_BUFFER csmi_sas_nv_seg_props_buffer;
	PCSMI_SAS_NV_SEGMENT_PROPERTIES pcsmi_sas_nv_seg_properties;
	PIOCTL_HEADER pioctl_header;
	u_long flags;
	struct asd_softc *asd;
	uint32_t seg_offset, nv_offset;
	uint32_t pad_size, image_size, attr, bytes_read;
	int csmi_user_buf_len;
	int err;
	int retcode; 
	
	csmi_user_buf_len = sizeof(csmi_sas_nv_seg_props_buffer);
	if ((err = asd_ctl_check_buf_len(arg, csmi_user_buf_len))) {
		return -EINVAL;
	}
	
	if ((err = copy_from_user(&csmi_sas_nv_seg_props_buffer, 
				  (void *)arg, 
				  csmi_user_buf_len))) {
		return -EFAULT;
	}
	
	pioctl_header = &csmi_sas_nv_seg_props_buffer.IoctlHeader;
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_nv_seg_props_buffer, 
			csmi_user_buf_len, 
			CSMI_SAS_STATUS_INVALID_PARAMETER);
		return -ENODEV;
	}
	
	pcsmi_sas_nv_seg_properties = &csmi_sas_nv_seg_props_buffer.Information;
	
	asd_lock(asd, &flags);

	err = 0;
	retcode = CSMI_SAS_STATUS_SUCCESS;
	switch (pcsmi_sas_nv_seg_properties->uSegmentID) {
	case CSMI_SAS_SEGMENT_ID_MANUFACTURING_SECTOR0:
	{
		struct asd_manuf_base_seg_layout manuf_layout;
		
		if ((err = asd_hwi_search_nv_segment(asd, NVRAM_MANUF_TYPE,
				&nv_offset, &pad_size, &image_size, 
				&attr)) != 0) {
			err = -1;
			break;
		}
		asd_ctl_map_seg_attr_to_csmi(attr, 
			&pcsmi_sas_nv_seg_properties->usSegmentAttributes); 
		
		seg_offset = 0;
		memset(&manuf_layout, 0x0, sizeof(manuf_layout));
		
		/* Read Manufacturing base segment */
		if (asd_hwi_read_nv_segment(asd, NVRAM_MANUF_TYPE, 
				&manuf_layout, seg_offset, 
				sizeof(manuf_layout), &bytes_read) != 0) {
			err = -1;
			break;
		}
		image_size = manuf_layout.sector_size;
		pad_size = image_size;
		break;
	}
	case CSMI_SAS_SEGMENT_ID_CTRL_A_USER_SETTINGS0:
	{
		struct asd_pci_layout pci_layout;

		if ((err = asd_hwi_search_nv_segment(asd, NVRAM_CTRL_A_SETTING,
				&nv_offset, &pad_size, &image_size, 
				&attr)) != 0) {
			err = -1;
			break;
		}
		asd_ctl_map_seg_attr_to_csmi(attr, 
			&pcsmi_sas_nv_seg_properties->usSegmentAttributes); 
		
		seg_offset = 0;
		memset(&pci_layout, 0x0, sizeof(pci_layout));
		
		/* Read Ctrl-A first segment */
		if (asd_hwi_read_nv_segment(asd, NVRAM_CTRL_A_SETTING, 
				&pci_layout, seg_offset, sizeof(pci_layout),
				&bytes_read) != 0) {
			err = -1;
			break;
		}
		image_size = pci_layout.image_size;
		pad_size = image_size;
		break;
	}
	case CSMI_SAS_SEGMENT_ID_FLASH_DIRECTORY0:
	{
		struct asd_flash_dir_layout flash_dir;
		uint32_t nv_addr, active_entries;
		int i;
		if (asd_hwi_search_nv_cookie(asd, &nv_addr, &flash_dir) != 0) {
			err = -1;
			break;
		}
		seg_offset = 0;
		active_entries = 0;
		for (i = 0; i < NVRAM_MAX_ENTRIES; i++) {
			if (flash_dir.ae_mask & (1 << i)) {
				active_entries++;
			}
		}
		image_size = sizeof(flash_dir) +
			     (active_entries * 
			     sizeof(struct asd_fd_entry_layout));
		pad_size = sizeof(flash_dir) + 
			   (NVRAM_MAX_ENTRIES * 
			   sizeof(struct asd_fd_entry_layout));
		break;
	}
	case CSMI_SAS_SEGMENT_ID_FLASH0:
	case CSMI_SAS_SEGMENT_ID_SEEPROM0:
	case CSMI_SAS_SEGMENT_ID_COMPATIBILITY_SECTOR0:
	{
		pad_size = CSMI_SAS_SEGMENT_SIZE_INVALID;
		image_size = CSMI_SAS_SEGMENT_IMAGE_SIZE_INVALID;
		pcsmi_sas_nv_seg_properties->usSegmentAttributes = 0;
		break;
	}
	default:
		err = -1;
		break;
	}
	asd_unlock(asd, &flags);

	if (err) {
		pcsmi_sas_nv_seg_properties->uStatus = CSMI_SAS_NV_FAILURE;
		retcode = CSMI_SAS_STATUS_FAILED;
	} else {
		pcsmi_sas_nv_seg_properties->uSegmentSize = pad_size;
		pcsmi_sas_nv_seg_properties->uSegmentImageSize = image_size;
		pcsmi_sas_nv_seg_properties->uStatus = CSMI_SAS_NV_SUCCESS;
		retcode = CSMI_SAS_STATUS_SUCCESS;
	}

	/* return status */	
	return asd_ctl_copy_to_user((void *)arg, 
			&csmi_sas_nv_seg_props_buffer, 
			csmi_user_buf_len, 
			retcode);
}

static int asd_csmi_sas_write_nv_seg(unsigned long arg)
{
	PCSMI_SAS_WRITE_NV_SEGMENT_BUFFER pcsmi_sas_write_nv_seg_buffer;
	PCSMI_SAS_WRITE_NV_SEGMENT pcsmi_sas_write_nv_segment;
	PIOCTL_HEADER pioctl_header;
	//u_long flags;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	struct asd_softc *asd;
	uint8_t *psource_buffer;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	/* 
	 * Leave the data buffer at the end of the struct so as not to 
	 * overwrite the contents filled in by asd_ctl_issue_xxx() routine.
	 */
	csmi_user_buf_len = offsetof(CSMI_SAS_WRITE_NV_SEGMENT_BUFFER, 
				     bSourceBuffer) ;
	/* 
	 * The xxx_buffer data structure needs to be shared between
	 * the request and done routines, hence we allocate it dynamically.
	 */
	if ((pcsmi_sas_write_nv_seg_buffer = asd_alloc_mem(csmi_user_buf_len,
				GFP_ATOMIC)) == NULL) {
		return -ENOMEM;
	}

	if ((err = copy_from_user(pcsmi_sas_write_nv_seg_buffer, (void *)arg, 
				  csmi_user_buf_len))) {
		asd_free_mem(pcsmi_sas_write_nv_seg_buffer);
		return -EFAULT;
	}
	
	pioctl_header = &pcsmi_sas_write_nv_seg_buffer->IoctlHeader;
	pcsmi_sas_write_nv_segment = 
		&pcsmi_sas_write_nv_seg_buffer->Information;

	if ((err = asd_ctl_check_buf_len(arg, 
			  csmi_user_buf_len
			  + pcsmi_sas_write_nv_segment->uBufferLength))) {
		asd_free_mem(pcsmi_sas_write_nv_seg_buffer);
		return -EINVAL;
	}
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
				pcsmi_sas_write_nv_seg_buffer, 
				csmi_user_buf_len,
				CSMI_SAS_STATUS_INVALID_PARAMETER);
		asd_free_mem(pcsmi_sas_write_nv_seg_buffer);
		return -ENODEV;
	}

	if (!asd->hw_profile.flash_present) {
		asd_ctl_copy_to_user((void *)arg, 
				pcsmi_sas_write_nv_seg_buffer, 
				csmi_user_buf_len,
				CSMI_SAS_STATUS_FAILED);
		asd_free_mem(pcsmi_sas_write_nv_seg_buffer);
		return -EINVAL;
	}
	
	psource_buffer = NULL;	
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	down_interruptible(&pcsmi_mgmt->sem);

	err = asd_ctl_write_to_nvram(asd,
				pcsmi_sas_write_nv_seg_buffer, 
				arg, &psource_buffer);

	if (err) {
		retcode = CSMI_SAS_STATUS_FAILED; 
	} else {
		retcode = CSMI_SAS_STATUS_SUCCESS; 
	}

	/* return status */	
	asd_ctl_copy_to_user((void *)arg, 
			pcsmi_sas_write_nv_seg_buffer, 
			csmi_user_buf_len, 
			retcode);
	
	if (psource_buffer != NULL) {
		vfree(psource_buffer);
	}
	asd_free_mem(pcsmi_sas_write_nv_seg_buffer);
	up(&pcsmi_mgmt->sem);

	return err;
}

static int asd_csmi_sas_read_nv_seg(unsigned long arg)
{
	PCSMI_SAS_READ_NV_SEGMENT_BUFFER pcsmi_sas_read_nv_seg_buffer;
	PCSMI_SAS_READ_NV_SEGMENT pcsmi_sas_read_nv_segment;
	PIOCTL_HEADER pioctl_header;
	//u_long flags;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	struct asd_softc *asd;
	uint8_t *pdest_buffer;
	int csmi_user_buf_len;
	int retcode;
	int err;
	
	/* 
	 * Leave the data buffer at the end of the struct so as not to 
	 * overwrite the contents filled in by asd_ctl_issue_xxx() routine.
	 */
	csmi_user_buf_len = offsetof(CSMI_SAS_READ_NV_SEGMENT_BUFFER, 
				     bDestinationBuffer) ;
	/* 
	 * The xxx_buffer data structure needs to be shared between
	 * the request and done routines, hence we allocate it dynamically.
	 */
	if ((pcsmi_sas_read_nv_seg_buffer = asd_alloc_mem(
				csmi_user_buf_len,
				GFP_ATOMIC)) == NULL) {
		return -ENOMEM;
	}

	if ((err = copy_from_user(pcsmi_sas_read_nv_seg_buffer, (void *)arg, 
				  csmi_user_buf_len)) != 0) {
		asd_free_mem(pcsmi_sas_read_nv_seg_buffer);
		return -EFAULT;
	}
	
	pioctl_header = &pcsmi_sas_read_nv_seg_buffer->IoctlHeader;
	pcsmi_sas_read_nv_segment = 
		&pcsmi_sas_read_nv_seg_buffer->Information;

	if ((err = asd_ctl_check_buf_len(arg, 
			  csmi_user_buf_len
			  + pcsmi_sas_read_nv_segment->uBytesToRead))) {
		asd_free_mem(pcsmi_sas_read_nv_seg_buffer);
		return -EINVAL;
	}
	
	asd = asd_get_softc_by_hba_index(pioctl_header->IOControllerNumber);
	
	if (asd == NULL) {
		/* Requested controller not found */
		asd_ctl_copy_to_user((void *)arg, 
				pcsmi_sas_read_nv_seg_buffer, 
				csmi_user_buf_len,
				CSMI_SAS_STATUS_INVALID_PARAMETER);
		asd_free_mem(pcsmi_sas_read_nv_seg_buffer);
		return -ENODEV;
	}
	
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	pdest_buffer = NULL;
	err = 0;
	down_interruptible(&pcsmi_mgmt->sem);
	err = asd_ctl_read_from_nvram(asd,
				pcsmi_sas_read_nv_seg_buffer, 
				arg, &pdest_buffer);

	if (err) {
		retcode = CSMI_SAS_STATUS_FAILED; 
	} else {
		retcode = CSMI_SAS_STATUS_SUCCESS; 
		if (pdest_buffer != NULL) {
			if ((err = copy_to_user((void *)arg + csmi_user_buf_len,
				pdest_buffer,
				pcsmi_sas_read_nv_segment->uBytesRead))) {
				retcode = CSMI_SAS_STATUS_FAILED; 
			}
		}
	}

	/* return status */	
	asd_ctl_copy_to_user((void *)arg, 
			pcsmi_sas_read_nv_seg_buffer, 
			csmi_user_buf_len, 
			retcode);
	
	if (pdest_buffer != NULL) {
		vfree(pdest_buffer);
	}
	asd_free_mem(pcsmi_sas_read_nv_seg_buffer);
	up(&pcsmi_mgmt->sem);
	
	return err;
}
#endif /* #ifdef ASD_ADPT_CSMI_EXTENSION */

/* helper routines */
static void
asd_ctl_map_open_reject_to_csmi(uint8_t reason, int abandon,
				uint8_t *csmi_conn_stat)
{
	if (abandon) {
		/* Abandon specific reason. */ 
		switch (reason) {
		case BAD_DESTINATION:
			*csmi_conn_stat = CSMI_SAS_OPEN_REJECT_BAD_DESTINATION;
			break;

		case CONN_RATE_NOT_SUPPORTED:
			*csmi_conn_stat =
				CSMI_SAS_OPEN_REJECT_RATE_NOT_SUPPORTED;
			break;

		case PROTOCOL_NOT_SUPPORTED:
			*csmi_conn_stat =
				CSMI_SAS_OPEN_REJECT_PROTOCOL_NOT_SUPPORTED;
			break;

		case RSVD_ABANDON_0:
			*csmi_conn_stat = CSMI_SAS_OPEN_REJECT_RESERVE_ABANDON;
			break;

		case RSVD_ABANDON_1:
			*csmi_conn_stat = CSMI_SAS_OPEN_REJECT_RESERVE_CONTINUE;
			break;

		case RSVD_ABANDON_2:
			*csmi_conn_stat =
				CSMI_SAS_OPEN_REJECT_RESERVE_INITIALIZE;
			break;

		case RSVD_ABANDON_3:
			*csmi_conn_stat = CSMI_SAS_OPEN_REJECT_RESERVE_STOP;
			break;

		case WRONG_DESTINATION:
			*csmi_conn_stat =
				CSMI_SAS_OPEN_REJECT_WRONG_DESTINATION;
			break;

		case STP_RESOURCE_BUSY:
			*csmi_conn_stat =
				CSMI_SAS_OPEN_REJECT_STP_RESOURCES_BUSY;
			break;

		default:
			/* Unknown reason. */
			break;
		}
	} else {
		/* Open specific reason. */ 
		switch (reason) {
		case NO_DESTINATION:
			*csmi_conn_stat = CSMI_SAS_OPEN_REJECT_NO_DESTINATION;
			break;

		case PATHWAY_BLOCKED:
			*csmi_conn_stat = CSMI_SAS_OPEN_REJECT_PATHWAY_BLOCKED;
			break;
		
		default:
			/* Unknown reason. */
			break;
		}
	}
}

static int asd_ctl_find_smp_txr_len(uint8_t smpfunc, uint32_t *preq_len,
			uint32_t *presp_len)
{
	int reqlen, resplen;
	
	/* from SAS spec */
	switch (smpfunc) {
	case REPORT_GENERAL:
		reqlen = 8;
		resplen = 32;
		break;
	case REPORT_MANUFACTURER_INFORMATION:
		reqlen = 8;
		resplen = 64;
		break;
	case DISCOVER:
		reqlen = 16;
		resplen = 56;
		break;
	case REPORT_PHY_ERROR_LOG:
		reqlen = 16;
		resplen = 32;
		break;
	case REPORT_PHY_SATA:
		reqlen = 16;
		resplen = 60;
		break;
	case REPORT_ROUTE_INFORMATION:
		reqlen = 16;
		resplen = 44;
		break;
	case CONFIGURE_ROUTE_INFORMATION:
		reqlen = 44;
		resplen = 8;
		break;
	case PHY_CONTROL:
		reqlen = 44;
		resplen = 8;
		break;
	default:
		return -EINVAL;
	}

	*preq_len = reqlen;
	*presp_len = resplen;
	return 0;
}

static int asd_ctl_map_conn_type_to_csmi(uint8_t asd_conn_type, 
			uint8_t lane, uint32_t *csmi_conn_type)
{
	int err;
	
	err = 0;
	switch (asd_conn_type) {
	case NV_CONNT_SFF8470:
		switch (lane) {
		case 0:
			*csmi_conn_type = CSMI_SAS_CON_SFF_8470_LANE_1;
			break;
		case 1:
			*csmi_conn_type = CSMI_SAS_CON_SFF_8470_LANE_2;
			break;
		case 2:
			*csmi_conn_type = CSMI_SAS_CON_SFF_8470_LANE_3;
			break;
		case 3:
			*csmi_conn_type = CSMI_SAS_CON_SFF_8470_LANE_4;
			break;
		default:
			*csmi_conn_type = CSMI_SAS_CON_UNKNOWN;
			break;
		}
		break;
	case NV_CONNT_SFF8484:
		switch (lane) {
		case 0:
			*csmi_conn_type = CSMI_SAS_CON_SFF_8484_LANE_1;
			break;
		case 1:
			*csmi_conn_type = CSMI_SAS_CON_SFF_8484_LANE_2;
			break;
		case 2:
			*csmi_conn_type = CSMI_SAS_CON_SFF_8484_LANE_3;
			break;
		case 3:
			*csmi_conn_type = CSMI_SAS_CON_SFF_8484_LANE_4;
			break;
		default:
			*csmi_conn_type = CSMI_SAS_CON_UNKNOWN;
			break;
		}
		break;
	case NV_CONNT_SFF8482:
		*csmi_conn_type = CSMI_SAS_CON_SFF_8482;
		break;
	case NV_CONNT_PCIX_D0:
	case NV_CONNT_SAS_D0:
	case NV_CONNT_VEN_UNQ02:
	case NV_CONNT_VEN_UNQ7F:
	case NV_CONNT_UNKNOWN:
	default:
		*csmi_conn_type = CSMI_SAS_CON_UNKNOWN;
		break;
	}
	
	return err;
}

/* helper work-horse routines for ioctl handlers */
static int asd_ctl_issue_smp_pt(struct asd_softc	*asd,
	struct asd_port	*port, 
	PCSMI_SAS_SMP_PASSTHRU_BUFFER pcsmi_sas_smp_passthru_buffer)
{
/* 
 * As the csmi smp pass thru ioctl requires the response data 
 * after issuing an SMP request, this routine posts the scb and 
 * waits for it to complete. On completion it copies the return
 * status/data to user. This routine would return busy to any newly 
 * received requests if there are outstanding requests still pending.
 */
	PCSMI_SAS_SMP_PASSTHRU pcsmi_sas_smp_passthru;
	PCSMI_SAS_SMP_REQUEST pcsmi_sas_smp_request;
	PCSMI_SAS_SMP_RESPONSE pcsmi_sas_smp_response;
	PIOCTL_HEADER pioctl_header;
	dma_addr_t smprequestbusaddr;
	dma_addr_t smpresponsebusaddr;
	u_long flags;
	struct SMPRequest  *smprequestframe;
	struct SMPResponse *smpresponseframe;
	struct scb *scb;
	struct asd_smp_task_hscb *smp_hscb;
	struct asd_target *targ;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	uint32_t req_txr_len, resp_txr_len;
	uint32_t req_pci_data_dir, resp_pci_data_dir;
	int req_buf_len, resp_buf_len;
	int active_timer;
	int err;
		
	pcsmi_sas_smp_passthru = &pcsmi_sas_smp_passthru_buffer->Parameters;
	pcsmi_sas_smp_response = &pcsmi_sas_smp_passthru->Response;
	pcsmi_sas_smp_request = &pcsmi_sas_smp_passthru->Request;
	pioctl_header = &pcsmi_sas_smp_passthru_buffer->IoctlHeader;

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
		
	if ((targ = asd_get_sas_target_from_sasaddr(asd, port, 
		pcsmi_sas_smp_passthru->bDestinationSASAddress)) == NULL) {
		return -ENODEV;
	}
	
	
	down_interruptible(&pcsmi_mgmt->sem);
	if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
			     &pcsmi_mgmt->busy)) { 
		up(&pcsmi_mgmt->sem);
		return -EBUSY;
	}
	
	asd_lock(asd, &flags);
	if ((scb = asd_hwi_get_scb(asd, 1)) == NULL) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		up(&pcsmi_mgmt->sem);
		asd_unlock(asd, &flags);
		return -ENOMEM;
	}
	asd_unlock(asd, &flags);
	
	req_buf_len = sizeof(struct SMPRequest);
	if ((smprequestframe = (struct SMPRequest *)asd_alloc_mem(
				req_buf_len,	
				GFP_ATOMIC)) == NULL) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		up(&pcsmi_mgmt->sem);
		return -ENOMEM;
	}

	memset(smprequestframe, 0, req_buf_len);
	
	/*
	 * populate request frame with user data 
	 */
	smprequestframe->SMPFrameType = pcsmi_sas_smp_request->bFrameType;
	smprequestframe->Function = pcsmi_sas_smp_request->bFunction;
	req_txr_len = 0;
	resp_txr_len = 0;

	asd_ctl_find_smp_txr_len(pcsmi_sas_smp_request->bFunction, 
				 &req_txr_len, &resp_txr_len);

	/* Subtract length of CRC (4 bytes) from transfer length as sequencer
	 * adds-up CRC while posting the request to device
	 */
	req_txr_len -= sizeof(uint32_t);
	resp_txr_len -= sizeof(uint32_t);

	if (pcsmi_sas_smp_passthru->uRequestLength < req_txr_len) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		up(&pcsmi_mgmt->sem);
		return -EINVAL;
	}

	memcpy(&smprequestframe->Request, 
		pcsmi_sas_smp_request->bAdditionalRequestBytes,
		pcsmi_sas_smp_passthru->uRequestLength);
	
	req_pci_data_dir = scsi_to_pci_dma_dir(SCSI_DATA_READ);
	smprequestbusaddr = 0;
	smprequestbusaddr = asd_map_single(asd,
					   smprequestframe,
					   req_buf_len, 
					   req_pci_data_dir);
	if (!smprequestbusaddr) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		asd_free_mem(smprequestframe);
		up(&pcsmi_mgmt->sem);
		return -ENOMEM;
	}

	resp_buf_len = sizeof(struct SMPResponse);
	if ((smpresponseframe = (struct SMPResponse *)asd_alloc_mem(
				resp_buf_len,	
				GFP_ATOMIC)) == NULL) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		asd_free_mem(smprequestframe);
		up(&pcsmi_mgmt->sem);
		return -ENOMEM;
	}

	memset(smpresponseframe, 0, resp_buf_len);

	resp_pci_data_dir = scsi_to_pci_dma_dir(SCSI_DATA_WRITE);
	smpresponsebusaddr = 0;
	smpresponsebusaddr = asd_map_single(asd,
				            smpresponseframe,
				            resp_buf_len, 
				            resp_pci_data_dir);
	if (!smpresponsebusaddr) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		asd_free_mem(smprequestframe);
		asd_free_mem(smpresponseframe);
		up(&pcsmi_mgmt->sem);
		return -ENOMEM;
	}
	
	scb->flags |= (SCB_INTERNAL | SCB_CSMI);
	asd_push_post_stack(asd, scb, (void *)port, asd_ctl_issue_smp_pt_done);

	asd_lock(asd, &flags);
	
	list_add_tail(&scb->owner_links, &asd->platform_data->pending_os_scbs);
	asd_hwi_build_smp_task(scb, targ, smprequestbusaddr, req_txr_len,
			       smpresponsebusaddr, resp_txr_len);
	smp_hscb = &scb->hscb->smp_task;
	scb->flags |= SCB_ACTIVE;
	
	asd_hwi_post_scb(asd, scb);

	asd_unlock(asd, &flags);

	/* start timer */	
	asd_ctl_start_csmi_scb_timer(&pcsmi_mgmt->timer, 
				     ASD_CSMI_TIMEOUT(pioctl_header),
				     scb, asd_ctl_csmi_timeout);
	
	wait_event(pcsmi_mgmt->waitq, 
		  !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy));

	/* delete timer */	
	active_timer = del_timer_sync(&pcsmi_mgmt->timer);
	
	asd_unmap_single(asd, smprequestbusaddr, 
			req_buf_len, req_pci_data_dir);
	asd_unmap_single(asd, smpresponsebusaddr, 
			resp_buf_len, resp_pci_data_dir);
	
	if (active_timer) {
		/* copy response data to user */
		pcsmi_sas_smp_response->bFrameType = 
					smpresponseframe->SMPFrameType;
		pcsmi_sas_smp_response->bFunction = 
					smpresponseframe->Function;
		pcsmi_sas_smp_response->bFunctionResult = 
					smpresponseframe->FunctionResult;
		pcsmi_sas_smp_passthru->uResponseBytes = 
					smp_hscb->smp_resp_size;
		
		memcpy(pcsmi_sas_smp_response->bAdditionalResponseBytes, 
		      &smpresponseframe->Response, 
		      pcsmi_sas_smp_passthru->uResponseBytes);
		
		err = 0;
	} else {
		ASDDBG1(("adp94xx:Request Timed out:" 
			 "CSMI SMP pass thru command\n"));
		err = -ETIME;
#ifdef REMOVE_WHEN_ERROR_RECOVERY_FIXED
		list_del(&scb->hwi_links);
		list_del(&scb->owner_links);
#endif
	}

	/* free scb */
	asd_hwi_free_scb(asd, scb);

	/* free mem */
	asd_free_mem(smprequestframe);
	asd_free_mem(smpresponseframe);
	up(&pcsmi_mgmt->sem);
	return err;
}

static void asd_ctl_issue_smp_pt_done(struct asd_softc *asd, 
	struct scb *scb,
	struct asd_done_list *done_listp)
{
	struct asd_port *port;
	struct asd_csmi_mgmt *pcsmi_mgmt;

	if ((scb->flags & SCB_ACTIVE) == 0) {
		asd_print("SCB %d done'd twice\n", SCB_GET_INDEX(scb));
		panic("Stopping for safety");
	}

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	list_del(&scb->owner_links);
	//if ((scb->flags & SCB_TIMEDOUT) != 0)
	//	list_del(&scb->timedout_links);

	port = (struct asd_port *)scb->io_ctx;
	
	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
	wake_up(&pcsmi_mgmt->waitq);

	return;
}

static int asd_ctl_issue_ssp_pt(struct asd_softc	*asd,
	struct asd_port	*port, 
	PCSMI_SAS_SSP_PASSTHRU_BUFFER pcsmi_sas_ssp_passthru_buffer,
	unsigned long arg)
{
/* 
 * As the csmi ssp pass thru ioctl requires the response data 
 * after issuing an SSP request, this routine posts the scb and 
 * waits for it to complete. On completion it copies the return
 * status/data to user. If a new request is received when there
 * are outstanding requests still pending, the new request
 * would be put in Q waiting to acquire the semaphore.
 */
	PCSMI_SAS_SSP_PASSTHRU pcsmi_sas_ssp_passthru;
	PCSMI_SAS_SSP_PASSTHRU_STATUS pcsmi_sas_ssp_passthru_status;
	PIOCTL_HEADER pioctl_header;
	dma_addr_t addr;
	u_long flags;
	struct scb *scb;
	struct asd_target *targ;
	struct asd_device *dev;
	struct asd_domain *dm;
	struct asd_ssp_task_hscb *ssp_hscb;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	uint32_t asd_data_dir, scsi_data_dir, pci_data_dir;
	int user_buf_offset, active_timer;
	int err;
	size_t sg_copy_size;
	uint8_t *request_buffer;
		
	pcsmi_sas_ssp_passthru = &pcsmi_sas_ssp_passthru_buffer->Parameters;
	pcsmi_sas_ssp_passthru_status = &pcsmi_sas_ssp_passthru_buffer->Status;
	pioctl_header = &pcsmi_sas_ssp_passthru_buffer->IoctlHeader;

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;

	if ((dm = asd->platform_data->domains[port->id]) == NULL) {
		return -ENODEV;
	}

	if ((targ = asd_get_os_target_from_sasaddr(asd, dm, 
		pcsmi_sas_ssp_passthru->bDestinationSASAddress)) == NULL) {
		return -ENODEV;
	}
	
	if ((dev = asd_get_device_from_lun(asd, targ, 
			pcsmi_sas_ssp_passthru->bLun)) == NULL) {
		return -ENODEV;
	}
	
	down_interruptible(&pcsmi_mgmt->sem);
	if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
				&pcsmi_mgmt->busy)) { 
		/* unlikely */
		up(&pcsmi_mgmt->sem);
		return -EBUSY;
	}

	asd_lock(asd, &flags);
	/* Get an scb to use. */
	if ((scb = asd_hwi_get_scb(asd, 1)) == NULL) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		up(&pcsmi_mgmt->sem);
		asd_unlock(asd, &flags);
		return -ENOMEM;
	}
	asd_unlock(asd, &flags);

	scb->flags |= (SCB_INTERNAL | SCB_CSMI);
	asd_push_post_stack(asd, scb, (void *)pcsmi_sas_ssp_passthru_buffer,
			    asd_ctl_issue_ssp_pt_done);

	asd_lock(asd, &flags);
	list_add_tail(&scb->owner_links, &asd->platform_data->pending_os_scbs);

	asd_hwi_build_ssp_task(scb, targ,
			   pcsmi_sas_ssp_passthru->bLun, 
			   pcsmi_sas_ssp_passthru->bCDB, 
			   pcsmi_sas_ssp_passthru->bCDBLength,
			   pcsmi_sas_ssp_passthru->bAdditionalCDBLength,
			   pcsmi_sas_ssp_passthru->uDataLength);

	ssp_hscb = &scb->hscb->ssp_task;
	
	asd_unlock(asd, &flags);
	err = 0;
	sg_copy_size = 0;
	asd_data_dir = 0;
	scsi_data_dir = 0;
	pci_data_dir = 0;
	request_buffer = NULL;

	asd_ctl_map_ssp_dir_from_csmi(pcsmi_sas_ssp_passthru->uFlags, 
					   &asd_data_dir, &scsi_data_dir);

	pci_data_dir = scsi_to_pci_dma_dir(scsi_data_dir);

	/* 
	 * get offset of user Databuffer. DataBuffer is the final 
	 * element in the XXX_PASSTHRU_BUFFER.
	 */
	user_buf_offset = offsetof(CSMI_SAS_SSP_PASSTHRU_BUFFER, bDataBuffer);

	if (asd_ctl_copy_map_user_data_buf(asd, arg, scb, &request_buffer, 
				    &addr, &sg_copy_size,
				    pcsmi_sas_ssp_passthru->uDataLength, 
				    user_buf_offset, pci_data_dir) != 0) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		up(&pcsmi_mgmt->sem);
		return -ENOMEM;
	}
	
	memcpy(ssp_hscb->sg_elements, scb->sg_list, sg_copy_size);

	ssp_hscb->data_dir_flags = asd_data_dir;
	
	scb->platform_data->dev = dev;
	scb->platform_data->targ = dev->target;
	scb->flags |= SCB_ACTIVE;

	pcsmi_mgmt->err = SCB_EH_FAILED;

	/*queue scb */
	if ((err = asd_ctl_queue_cmd(asd, scb, targ, dev))) {
		asd_unmap_single(asd, addr, 
				pcsmi_sas_ssp_passthru->uDataLength, 
				pci_data_dir);
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		asd_free_mem(request_buffer);
		up(&pcsmi_mgmt->sem);
		return err;
	}
	
	/* start timer */	
	asd_ctl_start_csmi_scb_timer(&pcsmi_mgmt->timer,
		ASD_CSMI_TIMEOUT(pioctl_header),
		scb, asd_ctl_csmi_timeout); 

	wait_event(pcsmi_mgmt->waitq,
		  !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy));

	/* delete timer */	
	active_timer = del_timer_sync(&pcsmi_mgmt->timer);
	
	/* unmap pci mem before touching contents */
	asd_unmap_single(asd, addr, pcsmi_sas_ssp_passthru->uDataLength, 
			pci_data_dir);

	if (active_timer) {
		/* copy data buffer to user */
		if (pcsmi_mgmt->err == SCB_EH_SUCCEED) {
			if ((err = copy_to_user((void *)arg + user_buf_offset,
					request_buffer,
					pcsmi_sas_ssp_passthru->uDataLength))) {
				asd_hwi_free_scb(asd, scb);
				asd_free_mem(request_buffer);
				up(&pcsmi_mgmt->sem);
				return err;
			}

			pcsmi_sas_ssp_passthru_status->uDataBytes = 
				asd_htole32(ssp_hscb->xfer_len);
			err = 0;
		}
	} else {
		ASDDBG1(("adp94xx:Request Timed out:" 
			 "CSMI SSP pass thru command\n"));
		err = -ETIME;
#ifdef REMOVE_WHEN_ERROR_RECOVERY_FIXED
		list_del(&scb->hwi_links);
		list_del(&scb->owner_links);
#endif
	}

	/* free scb */	
	asd_hwi_free_scb(asd, scb);
	asd_free_mem(request_buffer);
	up(&pcsmi_mgmt->sem);

	return err;
}

static void asd_ctl_issue_ssp_pt_done(struct asd_softc *asd, 
	struct scb *scb,
	struct asd_done_list *done_listp)
{
	PCSMI_SAS_SSP_PASSTHRU_BUFFER pcsmi_sas_ssp_passthru_buffer;
	PCSMI_SAS_SSP_PASSTHRU pcsmi_sas_ssp_passthru;
	PCSMI_SAS_SSP_PASSTHRU_STATUS pcsmi_sas_ssp_passthru_status;
	struct asd_csmi_mgmt *pcsmi_mgmt;

	if ((scb->flags & SCB_ACTIVE) == 0) {
		asd_print("SCB %d done'd twice\n", SCB_GET_INDEX(scb));
		panic("Stopping for safety");
	}

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	list_del(&scb->owner_links);
	//if ((scb->flags & SCB_TIMEDOUT) != 0)
	//	list_del(&scb->timedout_links);

	pcsmi_sas_ssp_passthru_buffer = 
				(PCSMI_SAS_SSP_PASSTHRU_BUFFER)scb->io_ctx;
	pcsmi_sas_ssp_passthru = &pcsmi_sas_ssp_passthru_buffer->Parameters;
	pcsmi_sas_ssp_passthru_status = &pcsmi_sas_ssp_passthru_buffer->Status;
	
	pcsmi_mgmt->err = asd_ctl_process_ssp_resp(asd, scb, done_listp, 
				     pcsmi_sas_ssp_passthru_status);

	//scb->platform_data->flags &= ~ASD_SCB_UP_EH_SEM;
	scb->platform_data->dev->openings++;
	scb->platform_data->dev->active--;
	scb->platform_data->dev->commands_issued--;
	
	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
	wake_up(&pcsmi_mgmt->waitq);
	
	return;
}

static int asd_ctl_issue_stp_pt(struct asd_softc	*asd,
	struct asd_port	*port, 
	PCSMI_SAS_STP_PASSTHRU_BUFFER pcsmi_sas_stp_passthru_buffer,
	unsigned long arg)
{
/* 
 * As the csmi stp pass thru ioctl requires the response data 
 * after issuing an STP request, this routine posts the scb and 
 * waits for it to complete. On completion it copies the return
 * status/data to user. If a new request is received when there
 * are outstanding requests still pending, the new request
 * would be put in Q waiting to acquire the semaphore.
 */
	PCSMI_SAS_STP_PASSTHRU pcsmi_sas_stp_passthru;
	PCSMI_SAS_STP_PASSTHRU_STATUS pcsmi_sas_stp_passthru_status;
	PIOCTL_HEADER pioctl_header;
	dma_addr_t addr;
	u_long flags;
	struct asd_target *targ;
	struct asd_device *dev;
	struct asd_domain *dm;
	struct asd_ata_task_hscb *ata_hscb;
	struct scb *scb;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	int err;
	int user_buf_offset, active_timer;
	size_t sg_copy_size;
	uint8_t *request_buffer;
	uint32_t asd_data_dir, scsi_data_dir, pci_data_dir;
	unsigned features_state;
		
	pcsmi_sas_stp_passthru = &pcsmi_sas_stp_passthru_buffer->Parameters;
	pcsmi_sas_stp_passthru_status = &pcsmi_sas_stp_passthru_buffer->Status;
	pioctl_header = &pcsmi_sas_stp_passthru_buffer->IoctlHeader;

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	if ((dm = asd->platform_data->domains[port->id]) == NULL) {
		return -ENODEV;
	}

	if ((targ = asd_get_os_target_from_sasaddr(asd, dm, 
			pcsmi_sas_stp_passthru->bDestinationSASAddress)) 
	     == NULL) {
		return -ENODEV;
	}
	
	if ((dev = asd_get_device_from_lun(asd, targ, NULL)) == NULL) {
		return -ENODEV;
	}

	down_interruptible(&pcsmi_mgmt->sem);
	if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
				&pcsmi_mgmt->busy)) {
		/* unlikely */
		up(&pcsmi_mgmt->sem);
		return -EBUSY;
	}

	switch (targ->command_set_type)
	{
	case ASD_COMMAND_SET_ATA:
		features_state = targ->ata_cmdset.features_state;
		break;

	case ASD_COMMAND_SET_ATAPI:
		features_state = targ->atapi_cmdset.features_state;
		break;

	default:
		features_state = 0;
		/*
		 * Should never happen.
		 */
		break;
	}
	
	asd_lock(asd, &flags);
	// Get an scb to use.
	if ((scb = asd_hwi_get_scb(asd, 1)) == NULL) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		up(&pcsmi_mgmt->sem);
		asd_unlock(asd, &flags);
		return -ENOMEM;
	}
	asd_unlock(asd, &flags);

	scb->flags |= (SCB_INTERNAL | SCB_CSMI);
	asd_push_post_stack(asd, scb, (void *)pcsmi_sas_stp_passthru_buffer,
		asd_ctl_issue_stp_pt_done);
	asd_lock(asd, &flags);
	
	list_add_tail(&scb->owner_links, &asd->platform_data->pending_os_scbs);

	asd_hwi_build_stp_task(scb, targ, pcsmi_sas_stp_passthru->uDataLength);
	
	ata_hscb = &scb->hscb->ata_task;

	ata_hscb->ata_flags |= CSMI_TASK;
	if ((pcsmi_sas_stp_passthru->uFlags & CSMI_SAS_STP_DMA) 
	   && (features_state & SATA_USES_DMA)) {
		ata_hscb->ata_flags |= DMA_XFER_MODE;
	} else {
		if ((pcsmi_sas_stp_passthru->uFlags & CSMI_SAS_STP_DMA_QUEUED) 
		   && (features_state & SATA_USES_QUEUEING)) {
			ata_hscb->ata_flags |= DMA_XFER_MODE;
		}
	}

	memcpy(ata_hscb->host_to_dev_reg_fis, 
	      pcsmi_sas_stp_passthru->bCommandFIS, 
	      20);

	/* XXX BUG:translate queued commands into non-queued commands
	 */
	/*
	if (ASD_H2D_FIS(ata_hscb)->command == WIN_WRITEDMA_QUEUED) {
		ASD_H2D_FIS(ata_hscb)->command = WIN_WRITEDMA;
		ASD_H2D_FIS(ata_hscb)->sector_count = 
					ASD_H2D_FIS(ata_hscb)->features;
		ASD_H2D_FIS(ata_hscb)->features = 0;
	} else {
		if (ASD_H2D_FIS(ata_hscb)->command == WIN_READDMA_QUEUED) {
			ASD_H2D_FIS(ata_hscb)->command = WIN_READDMA;
			ASD_H2D_FIS(ata_hscb)->sector_count = 
					ASD_H2D_FIS(ata_hscb)->features;
			ASD_H2D_FIS(ata_hscb)->features = 0;
		}
	}*/
	if ((ASD_H2D_FIS(ata_hscb)->command == WIN_READDMA_EXT) 
	   || (ASD_H2D_FIS(ata_hscb)->command == WIN_WRITEDMA_EXT) 
	   || (ASD_H2D_FIS(ata_hscb)->command == WIN_READ_EXT) 
	   || (ASD_H2D_FIS(ata_hscb)->command == WIN_WRITE_EXT)) {
		if (!(features_state & SATA_USES_48BIT)) {
			asd_unlock(asd, &flags);
			clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
					&pcsmi_mgmt->busy);
			asd_hwi_free_scb(asd, scb);
			up(&pcsmi_mgmt->sem);
			return -EINVAL;
		}
	}
	asd_unlock(asd, &flags);
	
	err = 0;
	asd_data_dir = 0;
	sg_copy_size = 0;
	scsi_data_dir = 0;
	pci_data_dir = 0;
	request_buffer = NULL;

	asd_ctl_map_stp_dir_from_csmi(pcsmi_sas_stp_passthru->uFlags, 
					   &asd_data_dir, &scsi_data_dir);
	pci_data_dir = scsi_to_pci_dma_dir(scsi_data_dir);
	/* 
	 * get offset of user Databuffer. DataBuffer is the final 
	 * element in the XXX_PASSTHRU_BUFFER.
	 */
	user_buf_offset = offsetof(CSMI_SAS_STP_PASSTHRU_BUFFER, bDataBuffer);

	if (asd_ctl_copy_map_user_data_buf(asd, arg, scb, &request_buffer, 
				    &addr, &sg_copy_size,
				    pcsmi_sas_stp_passthru->uDataLength, 
				    user_buf_offset, pci_data_dir) != 0) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		up(&pcsmi_mgmt->sem);
		return -ENOMEM;
	}
	memcpy(ata_hscb->sg_elements, scb->sg_list, sg_copy_size);

	ata_hscb->ata_flags |= asd_data_dir;
	
	scb->platform_data->dev = dev;
	scb->platform_data->targ = dev->target;
	scb->flags |= SCB_ACTIVE;
	
	pcsmi_mgmt->err = SCB_EH_FAILED;

	/*queue scb */
	if ((err = asd_ctl_queue_cmd(asd, scb, targ, dev))) {
		asd_unmap_single(asd, 
				addr, 
				pcsmi_sas_stp_passthru->uDataLength, 
				pci_data_dir);
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_hwi_free_scb(asd, scb);
		asd_free_mem(request_buffer);
		up(&pcsmi_mgmt->sem);
		return err;
	}

	/* start timer */	
	asd_ctl_start_csmi_scb_timer(&pcsmi_mgmt->timer, 
		ASD_CSMI_TIMEOUT(pioctl_header),
		scb, asd_ctl_csmi_timeout);

	wait_event(pcsmi_mgmt->waitq,
		  !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy));

	/* delete timer */	
	active_timer = del_timer_sync(&pcsmi_mgmt->timer);

	/* unmap pci mem before touching contents */
	asd_unmap_single(asd, addr, 
			pcsmi_sas_stp_passthru->uDataLength, pci_data_dir);

	if (active_timer) {
		/* copy data buffer to user */
		if (pcsmi_mgmt->err == SCB_EH_SUCCEED) {
			if ((err = copy_to_user((void *)arg + user_buf_offset,
					request_buffer,
					pcsmi_sas_stp_passthru->uDataLength))) {
				asd_hwi_free_scb(asd, scb);
				return err;
			}

			pcsmi_sas_stp_passthru_status->uDataBytes = 
						asd_htole32(ata_hscb->xfer_len);
		}
	} else {
		ASDDBG1(("adp94xx:Request Timed out:" 
			 "CSMI STP pass thru command\n"));
		err = -ETIME;
#ifdef REMOVE_WHEN_ERROR_RECOVERY_FIXED
		list_del(&scb->hwi_links);
		list_del(&scb->owner_links);
#endif
	}

	/* free scb */	
	asd_free_mem(request_buffer);
	asd_hwi_free_scb(asd, scb);
	up(&pcsmi_mgmt->sem);
	
	return err;
}

static void asd_ctl_issue_stp_pt_done(struct asd_softc *asd, 
	struct scb *scb,
	struct asd_done_list *done_listp)
{
	PCSMI_SAS_STP_PASSTHRU_BUFFER pcsmi_sas_stp_passthru_buffer;
	PCSMI_SAS_STP_PASSTHRU pcsmi_sas_stp_passthru;
	PCSMI_SAS_STP_PASSTHRU_STATUS pcsmi_sas_stp_passthru_status;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	
	if ((scb->flags & SCB_ACTIVE) == 0) {
		asd_print("SCB %d done'd twice\n", SCB_GET_INDEX(scb));
		panic("Stopping for safety");
	}

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	list_del(&scb->owner_links);
	//if ((scb->flags & SCB_TIMEDOUT) != 0)
	//	list_del(&scb->timedout_links);
	pcsmi_sas_stp_passthru_buffer = 
				(PCSMI_SAS_STP_PASSTHRU_BUFFER)scb->io_ctx;
	pcsmi_sas_stp_passthru = &pcsmi_sas_stp_passthru_buffer->Parameters;
	pcsmi_sas_stp_passthru_status = &pcsmi_sas_stp_passthru_buffer->Status;
	
	switch (done_listp->opcode) {
	case CSMI_TASK_COMP_WO_ERR:
	case TASK_COMP_WO_ERR:
		scb->eh_state = SCB_EH_SUCCEED;
		pcsmi_mgmt->err = SCB_EH_SUCCEED;
		break;

	case ATA_TASK_COMP_W_RESP:
	{
		union edb *edb;
		struct ata_resp_edb *redb;
		struct scb *escb;
		uint32_t edb_index;

		pcsmi_mgmt->err = SCB_EH_FAILED;
		edb = asd_hwi_get_edb_from_dl(asd, scb, done_listp, 
					&escb, &edb_index);
		if (edb == NULL) {
			asd_log(ASD_DBG_ERROR, "Invalid EDB recv for SSP "
				"comp w/response.\n");
			break;
		}
		redb = &edb->ata_resp;
		memcpy(pcsmi_sas_stp_passthru_status->bStatusFIS,
			       redb->ending_fis, 20/*24*/);/*TBD*/
		
		asd_hwi_free_edb(asd, escb, edb_index);
		break;
	}

	default:
		pcsmi_mgmt->err = SCB_EH_FAILED;
		break;		
	}

	scb->platform_data->dev->openings++;
	scb->platform_data->dev->active--;
	scb->platform_data->dev->commands_issued--;
	//scb->platform_data->flags &= ~ASD_SCB_UP_EH_SEM;
	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
	wake_up(&pcsmi_mgmt->waitq);
	
	return;
}

static int
asd_ctl_handle_hard_reset(struct asd_softc *asd, struct scb *scb)
{
	PCSMI_SAS_SSP_PASSTHRU_STATUS 	 task_iu_stat;
	PCSMI_SAS_SSP_TASK_IU		 task_iu;
	struct asd_target		*targ;
	struct asd_device		*dev;
	struct asd_csmi_mgmt		*pcsmi_mgmt;
	int				 retval;
	u_long				 flags;

	pcsmi_mgmt =  &asd->asd_ctl_csmi_internal.mgmt;
	task_iu = &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Parameters;
	task_iu_stat = &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Status;
	
	if (task_iu->bHostIndex != 0xFF) {
		struct Scsi_Host	*host;

		host = asd->platform_data->scsi_host;
		if ((host == NULL) || (host->host_no != task_iu->bHostIndex)) {
			clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR,
				  &pcsmi_mgmt->busy);
			retval = CSMI_SAS_NO_SCSI_ADDRESS;
			goto exit;
		}
	}

	asd_lock(asd, &flags);

	dev = asd_get_device(asd, task_iu->bPathId, task_iu->bTargetId,
			     task_iu->bLun, /*alloc*/ 0);

	asd_unlock(asd, &flags);

	if (dev == NULL) {
		/* No device exists for the requested Ch. Id. Lun. */
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retval = CSMI_SAS_NO_SCSI_ADDRESS;
		goto exit;
	}
	
	targ = dev->target;
	if (targ == NULL) {
		/* No device exists for the requested Ch. Id. Lun. */
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retval = CSMI_SAS_NO_SCSI_ADDRESS;
		goto exit;
	}
	/* Sanity check if the target is an end-device. */
	if (targ->management_type != ASD_DEVICE_END) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retval = CSMI_SAS_NOT_AN_END_DEVICE;
		goto exit;
	}

	/*
	 * For performing HARD RESET, we can make use the error recovery
         * thread.
	 * In order to achieve this, we need to make the scb as if it has
	 * timedout. Also fill in information needed by the error recovery
	 * handler.
	 */
	scb->flags |= SCB_TIMEDOUT;
	scb->platform_data->targ = targ;
	scb->platform_data->dev = dev;
	scb->eh_state = SCB_EH_DEV_RESET_REQ;
	scb->eh_post = asd_ctl_task_mgmt_done;
	list_add_tail(&scb->timedout_links, &asd->timedout_scbs);

	/* Wakeup the error recovery handler thread. */
	asd_wakeup_sem(&asd->platform_data->ehandler_sem);

	/* Wait till TMF request is completed. */
	wait_event(pcsmi_mgmt->waitq,
		  !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy));

	if (scb->eh_status == SCB_EH_SUCCEED) {
		retval = CSMI_SAS_STATUS_SUCCESS;
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat,
					 CSMI_SAS_OPEN_REJECT_NO_DESTINATION);
	} else {
		retval = CSMI_SAS_STATUS_FAILED;
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat,
					 CSMI_SAS_OPEN_REJECT_NO_DESTINATION);
	}

exit:
	return (retval);
}

static int
asd_ctl_handle_task_mgmt(struct asd_softc *asd, struct scb *scb, 
			 int tmf_opcode)
{
	PCSMI_SAS_SSP_TASK_IU		 task_iu;
	struct asd_target		*targ;
	struct asd_device		*dev;
	struct asd_csmi_mgmt		*pcsmi_mgmt;
	int				 retval;
	u_long				 flags;

	task_iu = &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Parameters;
	pcsmi_mgmt =  &asd->asd_ctl_csmi_internal.mgmt;

	if (task_iu->bHostIndex != 0xFF) {
		struct Scsi_Host	*host;

		host = asd->platform_data->scsi_host;
		if ((host == NULL) || (host->host_no != task_iu->bHostIndex)) {
			clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
				  &pcsmi_mgmt->busy);
			retval = CSMI_SAS_NO_SCSI_ADDRESS;
			goto exit;
		}
	}

	asd_lock(asd, &flags);

	dev = asd_get_device(asd, task_iu->bPathId, task_iu->bTargetId,
			     task_iu->bLun, /*alloc*/ 0);

	asd_unlock(asd, &flags);

	if (dev == NULL) {
		/* No device exists for the requested Ch. Id. Lun. */
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retval = CSMI_SAS_NO_SCSI_ADDRESS;
		goto exit;
	}
	
	targ = dev->target;
	if (targ == NULL) {
		/* No device exists for the requested Ch. Id. Lun. */
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retval = CSMI_SAS_NO_SCSI_ADDRESS;
		goto exit;
	}
	/* Sanity check if the target is an end-device. */
	if (targ->management_type != ASD_DEVICE_END) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retval = CSMI_SAS_NOT_AN_END_DEVICE;
		goto exit;
	}
	
	/* Currently, we do not support TMF for SATA end-device. */
	if (targ->device_protocol_type == ASD_DEVICE_PROTOCOL_ATA) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retval = CSMI_SAS_STATUS_FAILED;
		goto exit;
	}
	
	scb->platform_data->targ = targ;
	scb->platform_data->dev = dev;

	/* Freeze the target's queue. */
	asd_freeze_targetq(asd, targ);

	switch (tmf_opcode) {
	case CSMI_SAS_SSP_ABORT_TASK:
	case CSMI_SAS_SSP_QUERY_TASK:
		retval = asd_ctl_abort_query_task(asd, scb, 
						  task_iu->uQueueTag,
						  tmf_opcode);
		break;
	case CSMI_SAS_SSP_ABORT_TASK_SET:
	case CSMI_SAS_SSP_LOGICAL_UNIT_RESET:
	case CSMI_SAS_SSP_CLEAR_TASK_SET:
	case CSMI_SAS_SSP_CLEAR_ACA:
		retval = asd_ctl_process_tmf(asd, scb);
		break;

	default:
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		retval = CSMI_SAS_STATUS_FAILED;
		break;
	}

	/* Unfreeze the target's queue. */
	asd_unfreeze_targetq(asd, targ);

exit:
	return (retval);
}

static int
asd_ctl_abort_query_task(struct asd_softc *asd, struct scb *scb, int tag,
			 int tmf_opcode)
{
	PCSMI_SAS_SSP_TASK_IU		 task_iu;
	PCSMI_SAS_SSP_PASSTHRU_STATUS 	 task_iu_stat;
	struct asd_csmi_mgmt		*pcsmi_mgmt;
	struct scb 			*err_scb;
	struct scsi_cmnd		*cmd;
	union asd_cmd			*acmd;
	union asd_cmd			*safe_acmd;
	int			 	 retval;
	int			 	 found;
	int				 suppress_res;
	u_long			 	 flags;

	pcsmi_mgmt =  &asd->asd_ctl_csmi_internal.mgmt;
	task_iu = &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Parameters;
	task_iu_stat = &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Status;
	found = 0;

	/* 
	 * Check if we need to suppress reporting task management event
	 * to the OS.
	 */
	suppress_res = (task_iu->uFlags & CSMI_SAS_SUPPRESS_RESULT);

	asd_list_lock(&flags);
	/* Check if the cmd is still in the device queue. */
	list_for_each_entry_safe(acmd, safe_acmd, 
				 &scb->platform_data->dev->busyq, acmd_links) {
		cmd = &acmd_scsi_cmd(acmd);
		if (cmd->tag == tag) {
			/* Found the cmd to be aborted or queried. */
			if (tmf_opcode == CSMI_SAS_SSP_ABORT_TASK) {
				list_del(&acmd->acmd_links);

				if (suppress_res)
					asd_cmd_set_host_status(
							cmd, DID_SOFT_ERROR);
				else
					asd_cmd_set_host_status(
							cmd, DID_ABORT);

				cmd->scsi_done(cmd);
			}
			retval = CSMI_SAS_STATUS_SUCCESS;
			asd_list_unlock(&flags);
			ASD_CSMI_FILL_SSP_STATUS(
					task_iu_stat,
					CSMI_SAS_OPEN_REJECT_NO_DESTINATION);
			clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
							&pcsmi_mgmt->busy);
			goto exit;
		}
	}

	/* Check if the cmd is on the pending queue. */
	list_for_each_entry(err_scb, 
			    &asd->platform_data->pending_os_scbs,
			    owner_links) {
		cmd = &acmd_scsi_cmd(err_scb->io_ctx);
		if (cmd->tag == tag) {
			/* Cmd is "on the fly". Need to issue abort. */
			found = 1;
			break;
		}
	}
	asd_list_unlock(&flags);

	if (found == 0) {
		/* 
		 * We can't find any command associated with the requested
		 * queue tag. The command could be completed already.
		 */
		retval = CSMI_SAS_STATUS_SUCCESS;
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat,
					 CSMI_SAS_OPEN_REJECT_NO_DESTINATION);
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		goto exit;
	}

	if (tmf_opcode == CSMI_SAS_SSP_ABORT_TASK)
		asd_hwi_build_abort_task(scb, err_scb);
	else
		asd_hwi_build_query_task(scb, err_scb);

	/* Set up the post routine. */
	asd_push_post_stack(asd, scb, scb->io_ctx,
			    asd_ctl_abort_query_done);
	asd_lock(asd, &flags);
	asd_hwi_post_scb(asd, scb);
	asd_unlock(asd, &flags);

	wait_event(pcsmi_mgmt->waitq,
		  !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy));

	if (scb->eh_state == SCB_EH_CLR_NXS_REQ) {
		/*
		 * If the TMF was successfull sent to the target, we need
		 * to request the firmware to clear the aborted SCB.
		 */
		/* We serialized SSP TMF to the target. */
		if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR,
				     &pcsmi_mgmt->busy)) { 
			retval = CSMI_SAS_STATUS_FAILED;
			goto exit;
		}
	
		/* Set up the post routine. */
		asd_push_post_stack(asd, scb, scb->io_ctx,
				    asd_ctl_abort_query_done);
		scb->flags |= (SCB_INTERNAL | SCB_CSMI | SCB_ACTIVE);
		asd_lock(asd, &flags);
		asd_hwi_post_scb(asd, scb);
		asd_unlock(asd, &flags);
		
		wait_event(pcsmi_mgmt->waitq,
			   !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
			   &pcsmi_mgmt->busy));
	} else if (scb->eh_state == SCB_EH_RESUME_SENDQ) {
		/* We serialized SSP TMF to the target. */
		if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR,
				     &pcsmi_mgmt->busy)) { 
			retval = CSMI_SAS_STATUS_FAILED;
			goto exit;
		}
		scb->flags |= (SCB_INTERNAL | SCB_CSMI | SCB_ACTIVE);
		asd_hwi_build_clear_nexus(scb, CLR_NXS_I_T_L_Q_TC,
			       		  SCB_GET_INDEX(err_scb),
				  	  SCB_EH_RESUME_SENDQ);
		asd_push_post_stack(asd, scb, scb->io_ctx,
				    asd_ctl_abort_query_done);
		asd_lock(asd, &flags);
		asd_hwi_post_scb(asd, scb);
		asd_unlock(asd, &flags);

		wait_event(pcsmi_mgmt->waitq,
			   !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
			   &pcsmi_mgmt->busy));
	}

	if (scb->eh_status == SCB_EH_SUCCEED)
		retval = CSMI_SAS_STATUS_SUCCESS;
	else
		retval = CSMI_SAS_STATUS_FAILED;

exit:
	return (retval);
}

static void 
asd_ctl_abort_query_done(struct asd_softc *asd, struct scb *scb,
			 struct asd_done_list *dl)
{
	PCSMI_SAS_SSP_PASSTHRU_STATUS 	 task_iu_stat;
	struct asd_csmi_mgmt		*csmi_mgmt;
	
	task_iu_stat = &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Status;
	csmi_mgmt =  &asd->asd_ctl_csmi_internal.mgmt;
	
	switch (dl->opcode) {
	case TASK_COMP_WO_ERR:
		scb->eh_state = SCB_EH_DONE;
		scb->eh_status = SCB_EH_SUCCEED;
		break;

	case TMF_F_W_TAG_ALREADY_DONE:
	case TMF_F_W_TAG_ALREADY_FREE:
	case TMF_F_W_TC_NOT_FOUND:
	case TMF_F_W_TAG_NOT_FOUND:
	case TMF_F_W_CONN_HNDL_NOT_FOUND:
	case TASK_F_W_NAK_RCVD:
		/* 
		 * Indicate that Abort failed.
		 * If this is a failure in issuing ABORT TMF, we need to 
		 * resume the data transmission of the task that was
		 * going to be aborted.
		 */
		if ((scb->eh_state & SCB_EH_SUSPEND_SENDQ) != 0)
			scb->eh_state = SCB_EH_RESUME_SENDQ;
		else
			scb->eh_state = SCB_EH_DONE;
			
		scb->eh_status = SCB_EH_FAILED;
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat,
					 CSMI_SAS_OPEN_REJECT_NO_DESTINATION);
		break;

	case SSP_TASK_COMP_W_RESP:
	{
		union edb 		*edb;
		struct ssp_resp_edb 	*redb;
		struct scb 		*escb;
		struct ssp_resp_iu 	*riu;
		u_int 			 edb_index;
		
		edb = asd_hwi_get_edb_from_dl(asd, scb, dl, &escb, &edb_index);
		if (edb == NULL) {
			asd_log(ASD_DBG_ERROR, "Invalid EDB recv for SSP "
				"comp w/response.\n");
			if ((scb->eh_state & SCB_EH_SUSPEND_SENDQ) != 0)
				scb->eh_state = SCB_EH_RESUME_SENDQ;
			else
				scb->eh_state = SCB_EH_DONE;
			
			scb->eh_status = SCB_EH_FAILED;
			ASD_CSMI_FILL_SSP_STATUS(task_iu_stat, 
						 CSMI_SAS_OPEN_ACCEPT);
			break;
		}

		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat, CSMI_SAS_OPEN_ACCEPT);
		redb = &edb->ssp_resp;
		riu = &redb->resp_frame.riu;
		if (SSP_RIU_DATAPRES(riu) == SSP_RIU_DATAPRES_RESP) {
			uint8_t	resp_code;

			resp_code = ((struct resp_data_iu *)
						&riu->data[0])->resp_code;
			asd_ctl_copy_ssp_resp_data(riu, task_iu_stat);
			
			if (!asd_hwi_check_cmd_pending(asd, scb, dl)) {
				asd_log(ASD_DBG_ERROR, 
					"Cmd has been completed.\n");
				scb->eh_status = SCB_EH_SUCCEED;
				asd_hwi_free_edb(asd, escb, edb_index);
				break;
			}

			asd_hwi_map_tmf_resp(scb, resp_code);
		
			if (SCB_GET_OPCODE(scb) == SCB_QUERY_SSP_TASK) {
				scb->eh_state = SCB_EH_DONE;
			} else { 
				if ((SCB_GET_OPCODE(scb) == SCB_ABORT_TASK) &&
				   (scb->eh_state == SCB_EH_CLR_NXS_REQ)) {
					asd_log(ASD_DBG_ERROR, 
						"Tag to Clear=0x%x.\n",	
						redb->tag_to_clear);
					/* 
					 * Upon receving TMF_COMPLETE, 
					 * we need to send CLEAR_NEXUS SCB 
					 * for tag_to_clear.
					 */
					asd_hwi_build_clear_nexus(
							scb,
					  		CLR_NXS_I_T_L_Q_TAG,
					  		redb->tag_to_clear,
					  		/*ctx*/0);
				}
			}
		} else {
			/* Response Data not available. Protocol error. */
			if ((scb->eh_state & SCB_EH_SUSPEND_SENDQ) != 0)
				scb->eh_state = SCB_EH_RESUME_SENDQ;
			else
				scb->eh_state = SCB_EH_DONE;

			scb->eh_status = SCB_EH_FAILED;
			ASD_CSMI_FILL_SSP_STATUS(task_iu_stat, 
						 CSMI_SAS_OPEN_ACCEPT);
		}

		asd_hwi_free_edb(asd, escb, edb_index);
		break;
	}

	case TASK_F_W_OPEN_REJECT:
	{
		struct open_reject_sb	*open_reject;
		int			 abandon;
		uint8_t			 conn_stat;

		open_reject = &dl->stat_blk.open_reject;
		abandon = ((open_reject->abandon_open & ABANDON_OPEN_MASK) 
				? 1 : 0);

		asd_ctl_map_open_reject_to_csmi(open_reject->reason, abandon,
						&conn_stat);

		if ((scb->eh_state & SCB_EH_SUSPEND_SENDQ) != 0)
			scb->eh_state = SCB_EH_RESUME_SENDQ;
		else
			scb->eh_state = SCB_EH_DONE;
		
		scb->eh_status = SCB_EH_FAILED;
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat, conn_stat);
		break;
	}

	case RESUME_COMPLETE:
		scb->eh_state = SCB_EH_DONE;
		scb->eh_status = SCB_EH_FAILED;
		break;

	default:
		if ((scb->eh_state & SCB_EH_SUSPEND_SENDQ) != 0)
			scb->eh_state = SCB_EH_RESUME_SENDQ;
		else
			scb->eh_state = SCB_EH_DONE;
		
		scb->eh_status = SCB_EH_FAILED;
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat,
					 CSMI_SAS_OPEN_REJECT_NO_DESTINATION);
		break;		
	}
	
	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &csmi_mgmt->busy);
	wake_up(&csmi_mgmt->waitq);
}

static int
asd_ctl_process_tmf(struct asd_softc *asd, struct scb *scb)
{
	PCSMI_SAS_SSP_TASK_IU		 task_iu;
	struct asd_csmi_mgmt		*csmi_mgmt;
	union asd_cmd			*acmd;
	struct scsi_cmnd		*cmd;
	int			 	 retval;
	int				 suppress_res;
	u_long			 	 flags;

	csmi_mgmt =  &asd->asd_ctl_csmi_internal.mgmt;
	task_iu = &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Parameters;

	scb->eh_state = SCB_EH_INITIATED;
	scb->eh_post = asd_ctl_task_mgmt_done;
	asd_lock(asd, &flags);
	asd_ctl_issue_tmf(asd, scb);
	asd_unlock(asd, &flags);

	/* Wait till TMF request is completed. */
	wait_event(csmi_mgmt->waitq,
		  !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &csmi_mgmt->busy));

	if (scb->eh_status == SCB_EH_SUCCEED)
		retval = CSMI_SAS_STATUS_SUCCESS;
	else
		retval = CSMI_SAS_STATUS_FAILED;

	if ((retval == CSMI_SAS_STATUS_FAILED) || 
	    (task_iu->bTaskManagementFunction == 
	     			CSMI_SAS_SSP_LOGICAL_UNIT_RESET)) {
		/* 
		 * If we failed in performing TMF request or finished issuing
		 * LU Reset, then nothing else need to be done. 
		 */
		goto exit;
	}

	/* 
	 * Check if we need to suppress reporting task management event
	 * to the OS.
	 */
	suppress_res = (task_iu->uFlags & CSMI_SAS_SUPPRESS_RESULT);
		
	asd_list_lock(&flags);
	/* 
	 * For ABORT TASK SET or CLEAR TASK SET, we need to abort any 
	 * commands that yet to be issued for that target.
	 */
	while (!list_empty(&scb->platform_data->dev->busyq)) {
		acmd = list_entry(scb->platform_data->dev->busyq.next, 
				  union asd_cmd, acmd_links);
		list_del(&acmd->acmd_links);
		cmd = &acmd_scsi_cmd(acmd);
		
		if (suppress_res)
			asd_cmd_set_host_status(cmd, DID_SOFT_ERROR);
		else
			asd_cmd_set_host_status(cmd, DID_ABORT);

		cmd->scsi_done(cmd);
	}
	asd_list_unlock(&flags);

exit:
	return (retval);
}

static void
asd_ctl_task_mgmt_done(struct asd_softc *asd, struct scb *scb)
{
	struct asd_csmi_mgmt	*csmi_mgmt;
	
	csmi_mgmt =  &asd->asd_ctl_csmi_internal.mgmt;
	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &csmi_mgmt->busy);
	wake_up(&csmi_mgmt->waitq);
}

static void
asd_ctl_issue_tmf(struct asd_softc *asd, struct scb *scb)
{
	switch (scb->eh_state) {
	case SCB_EH_INITIATED:
		/* Set up the post routine. */
		asd_push_post_stack(asd, scb, scb->io_ctx,
			    	    asd_ctl_issue_tmf_done);
		scb->eh_state = SCB_EH_LU_RESET_REQ;
		asd_ctl_issue_tmf(asd, scb);

		break;

	case SCB_EH_LU_RESET_REQ:
	{
		PCSMI_SAS_SSP_TASK_IU	 task_iu;

		task_iu = 
		     &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Parameters;

		/* Build LUR Task Management Function. */
		asd_hwi_build_ssp_tmf(scb, scb->platform_data->targ,
				      scb->platform_data->dev->saslun,	
				      task_iu->bTaskManagementFunction);

		scb->flags |= (SCB_INTERNAL | SCB_ACTIVE);
		/* Post the Logical Unit Reset TMF SCB. */
		asd_hwi_post_scb(asd, scb);

		break;
	}

	case SCB_EH_CLR_NXS_REQ:
		asd_hwi_build_clear_nexus(scb, CLR_NXS_I_T_L, 
					  (NOT_IN_Q | SEND_Q), /*ctx*/0);
		scb->flags |= (SCB_INTERNAL | SCB_CSMI | SCB_ACTIVE);
		asd_push_post_stack(asd, scb, scb->io_ctx,
			    	    asd_ctl_issue_tmf_done);
		asd_hwi_post_scb(asd, scb);
		
		break;

	case SCB_EH_RESUME_SENDQ:
		/*
		 * If we failed to perform LU Reset, we need to resume
		 * the firmware sned queue for the I_T.
		 */
		asd_hwi_build_clear_nexus(scb, CLR_NXS_IT_OR_TI, 
					  RESUME_TX, /*ctx*/0);
		scb->flags |= (SCB_INTERNAL | SCB_CSMI | SCB_ACTIVE);
		asd_push_post_stack(asd, scb, scb->io_ctx,
			    	    asd_ctl_issue_tmf_done);
		asd_hwi_post_scb(asd, scb);

		break;

	case SCB_EH_DONE:
		/* Finished issuing TMF request to the target. */
		scb->eh_post(asd, scb);
		break;

	default:
		asd_log(ASD_DBG_ERROR, "Invalid State.\n");
		scb->eh_status = SCB_EH_FAILED;
		scb->eh_post(asd, scb);
		break;
	}
}

static void
asd_ctl_issue_tmf_done(struct asd_softc *asd, struct scb *scb,
		       struct asd_done_list *dl)
{
	PCSMI_SAS_SSP_PASSTHRU_STATUS 	 task_iu_stat;
	
	task_iu_stat = &((PCSMI_SAS_SSP_TASK_IU_BUFFER) scb->io_ctx)->Status;

	switch (dl->opcode) {
	case TASK_COMP_WO_ERR:
		scb->eh_state = SCB_EH_DONE;
		scb->eh_status = SCB_EH_SUCCEED;
		break;

	case TMF_F_W_CONN_HNDL_NOT_FOUND:
	case TASK_F_W_NAK_RCVD:
		if ((scb->eh_state & SCB_EH_SUSPEND_SENDQ) != 0)
			scb->eh_state = SCB_EH_RESUME_SENDQ;
		else
			scb->eh_state = SCB_EH_DONE;

		scb->eh_status = SCB_EH_FAILED;
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat,
					 CSMI_SAS_OPEN_REJECT_NO_DESTINATION);
		break;

	case SSP_TASK_COMP_W_RESP:
	{	
		union edb 		*edb;
		struct scb 		*escb;
		struct ssp_resp_edb  	*redb;
		struct ssp_resp_iu	*riu;
		u_int			 edb_index;

		edb = asd_hwi_get_edb_from_dl(asd, scb, dl, &escb, &edb_index);
		if (edb == NULL) {
			asd_log(ASD_DBG_ERROR, "Invalid EDB recv for SSP "
				"comp w/response.\n");
			scb->eh_state = SCB_EH_RESUME_SENDQ;
			scb->eh_status = SCB_EH_FAILED;
			break;
		}
		
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat, CSMI_SAS_OPEN_ACCEPT);

		redb = &edb->ssp_resp;
		riu = &redb->resp_frame.riu;
		if (SSP_RIU_DATAPRES(riu) == SSP_RIU_DATAPRES_RESP) {
			uint8_t		resp_code;

			asd_ctl_copy_ssp_resp_data(riu, task_iu_stat);
			resp_code = ((struct resp_data_iu *)
						&riu->data[0])->resp_code;

			/* Handle the SSP TMF response code. */
			asd_hwi_map_tmf_resp(scb, resp_code);
		} else {
			/* 
			 * Response Data not available. Protocol error.
			 * Indicate that LUR failed.
			 */
			scb->eh_state = SCB_EH_RESUME_SENDQ;
			scb->eh_status = SCB_EH_FAILED;
		}

		asd_hwi_free_edb(asd, escb, edb_index);
		break;
	}

	case TASK_CLEARED:
		/* 
		 * Previous error recovery SCB that timed-out and
		 * was aborted.
		 * All we need to do here is just free the scb.
		 */ 
		asd_hwi_free_scb(asd, scb);
		return;

	case TASK_F_W_OPEN_REJECT:
	{
		struct open_reject_sb	*open_reject;
		int			 abandon;
		uint8_t			 conn_stat;

		open_reject = &dl->stat_blk.open_reject;
		abandon = ((open_reject->abandon_open & ABANDON_OPEN_MASK) 
				? 1 : 0);

		asd_ctl_map_open_reject_to_csmi(open_reject->reason, abandon,
						&conn_stat);

		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat, conn_stat);
		scb->eh_state = SCB_EH_RESUME_SENDQ;
		scb->eh_status = SCB_EH_FAILED;

		break;
	}

	case RESUME_COMPLETE:
		scb->eh_state = SCB_EH_DONE;
		scb->eh_status = SCB_EH_FAILED;
		break;

	default:
		asd_log(ASD_DBG_ERROR, "DL opcode not handled.\n");
		scb->eh_state = SCB_EH_RESUME_SENDQ;
		scb->eh_status = SCB_EH_FAILED;
		ASD_CSMI_FILL_SSP_STATUS(task_iu_stat,
					 CSMI_SAS_OPEN_REJECT_NO_DESTINATION);
		break;
	}

	asd_ctl_issue_tmf(asd, scb);
}

static int
asd_ctl_pc_get_phy_settings(struct asd_softc *asd, 
	struct asd_phy *phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer,
	int *num_cntls)
{
	//PCSMI_SAS_PHY_CONTROL pcsmi_sas_phy_control;
	//PCSMI_SAS_CHARACTER pcsmi_sas_character;
	int ctl_index;
	int err;

	ctl_index = 0;
	err = 0;
	if (phy->attr & ASD_SSP_INITIATOR) {
		/* XXX TBD need a var that explains capabilities */
		if ((err = asd_ctl_pc_map_phy_settings(phy, 
				pcsmi_sas_phy_control_buffer, 
				ASD_SAS_MODE, 
				phy->min_link_rate,
				ctl_index++))) {
			*num_cntls = ctl_index;
			return err;
		}
		if (phy->min_link_rate != phy->max_link_rate) {
			if((err = asd_ctl_pc_map_phy_settings(phy, 
					pcsmi_sas_phy_control_buffer, 
					ASD_SAS_MODE, 
					phy->max_link_rate,
					ctl_index++))) {
				*num_cntls = ctl_index;
				return err;
			}
		}
	}
	if (phy->attr & ASD_STP_INITIATOR) {
		if ((err = asd_ctl_pc_map_phy_settings(phy, 
				pcsmi_sas_phy_control_buffer, 
				ASD_SATA_MODE, 
				phy->min_link_rate,
				ctl_index)))
			*num_cntls = ctl_index;
			return err;
	}

	return err;
}

static int asd_ctl_process_phy_ctl(struct asd_softc *asd, 
	struct asd_phy	*phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer)
{
/*
TBD:
1.If I/Os are active and pattern generation is requested, fail the request
2.If length > ASD_MAX_DATA_PAT_GEN_SIZE_TX return error ?
3.patternbuffer.btypeflags ??
*/
	PCSMI_SAS_PHY_CONTROL pcsmi_sas_phy_control;
	//PCSMI_SAS_CHARACTER pcsmi_sas_character;
	int ctl_index;
	int pattern_ctl_index;
	int err;
	uint8_t pattern_set; 
	uint32_t phy_ctl_func;

	/* Check for multiple ACTIVATE_CONTROL bits */
	pattern_set = 0;
	pattern_ctl_index = -1;
	phy_ctl_func = 0;
	for (ctl_index = 0; 
	     ctl_index < pcsmi_sas_phy_control_buffer->bNumberOfControls;
	     ctl_index++) {
		pcsmi_sas_phy_control = 
			&pcsmi_sas_phy_control_buffer->Control[ctl_index];
		
		/* Check for multiple xxx_PATTERN type bits */
		if ((pcsmi_sas_phy_control->uPatternFlags 
		     & CSMI_SAS_PHY_FIXED_PATTERN)
		   && (pcsmi_sas_phy_control->uPatternFlags 
		       & CSMI_SAS_PHY_USER_PATTERN)) {
			return -EINVAL;
		}
		
		if (pattern_set == 1 
		   && (pcsmi_sas_phy_control->uPatternFlags &
			CSMI_SAS_PHY_ACTIVATE_CONTROL)) {
			return -EINVAL;
		}
		if (pcsmi_sas_phy_control->uPatternFlags 
		    & CSMI_SAS_PHY_ACTIVATE_CONTROL ) {
			pattern_set = 1;
			pattern_ctl_index = ctl_index;
		}
	}

	err = 0;
	/* build and issue scb for the required phy control function */
	asd_ctl_map_pc_func_from_csmi(
				pcsmi_sas_phy_control_buffer->uFunction,
				&phy_ctl_func);
	
	if (pcsmi_sas_phy_control_buffer->bNumberOfControls != 0) {
		asd_ctl_pc_get_user_pc_val(asd, phy,
			pcsmi_sas_phy_control_buffer);
	}
	asd_ctl_switch_pat_gen_mode(asd, phy);
	err = asd_ctl_issue_phy_ctl(asd, phy,
			&pcsmi_sas_phy_control_buffer->IoctlHeader,
			phy_ctl_func);

	if (pattern_set == 1) {
		/* pattern generation needed */
		pcsmi_sas_phy_control = 
		     &pcsmi_sas_phy_control_buffer->Control[pattern_ctl_index];

		/* 
		 * TBD: User values are already applied during phy control 
		 * execution, is this call needed here again ?? 
		 */
		if (pcsmi_sas_phy_control_buffer->bNumberOfControls != 0) {
			asd_ctl_pc_get_user_pc_val(asd, phy,
				pcsmi_sas_phy_control_buffer);
		}
		/* enter pattern generation mode */
		err = asd_ctl_enter_pattern_gen(asd, phy,
						pcsmi_sas_phy_control_buffer,
						pattern_ctl_index);
	}

	return err;
}

static int asd_ctl_issue_phy_ctl(struct asd_softc *asd, struct asd_phy *phy, 
			PIOCTL_HEADER pioctl_header, uint32_t phy_ctl_func) 
{
	u_long flags;
	struct scb *scb;
	struct asd_port	*port; 
	struct asd_csmi_mgmt *pcsmi_mgmt;
	struct asd_target *targ;
	int err;
	int active_timer;
	
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	port = phy->src_port;
	err = 0;

	if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
				&pcsmi_mgmt->busy)) { 
		return -EBUSY;
	}

	asd_lock(asd, &flags);
	// Get an scb to use.
	if ((scb = asd_hwi_get_scb(asd, 1)) == NULL) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		asd_unlock(asd, &flags);
		return -ENOMEM;
	}
	
	scb->flags |= (SCB_INTERNAL | SCB_CSMI);

	/* 
	 * Freeze all the targets's queue attached to the port to 
	 * be reset. 
	 */
	if ((port != NULL) && (!list_empty(&port->targets))) {
		list_for_each_entry(targ, &port->targets, 
			    all_domain_targets) {
			targ->qfrozen++;
		}
	}
	
	list_add_tail(&scb->owner_links, &phy->pending_scbs);
	asd_hwi_build_control_phy(scb, phy, phy_ctl_func);
	
	scb->flags |= SCB_ACTIVE;
	scb->io_ctx = (void *)phy;

	asd_push_post_stack(asd, scb, (void *)phy,
		asd_ctl_issue_phy_ctl_done);
	
	if (phy_ctl_func == ENABLE_PHY || phy_ctl_func == DISABLE_PHY) {
		asd_hwi_control_activity_leds(asd, phy->id, phy_ctl_func);
	}
	
	asd_hwi_post_scb(asd, scb);
	asd_unlock(asd, &flags);
	
	/* start timer */	
	asd_ctl_start_csmi_scb_timer(&pcsmi_mgmt->timer,
		ASD_CSMI_TIMEOUT(pioctl_header),
		scb, asd_ctl_csmi_timeout);
	
	wait_event(pcsmi_mgmt->waitq,
		  !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy));
	
	/* delete timer */	
	active_timer = del_timer_sync(&pcsmi_mgmt->timer);

	if (active_timer) {
		err = 0;
	} else {
		ASDDBG1(("adp94xx:Request Timed out:" 
			 "CSMI Phy Ctl command\n"));
		err = -ETIME;
#ifdef REMOVE_WHEN_ERROR_RECOVERY_FIXED
		list_del(&scb->hwi_links);
		list_del(&scb->owner_links);
#endif
	}

	/* free scb */
	asd_hwi_free_scb(asd, scb);

	return err;
}

static void
asd_ctl_issue_phy_ctl_done(struct asd_softc *asd, 
	struct scb *scb,
	struct asd_done_list *done_listp)
{
	struct asd_csmi_mgmt *pcsmi_mgmt;
	struct asd_phy	*phy; 
	struct asd_port	*port; 
	struct asd_target *targ;
	
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	pcsmi_mgmt->err  = SCB_EH_FAILED;
	phy = (struct asd_phy *)scb->io_ctx;
	port = phy->src_port;

	if (scb->eh_state == SCB_EH_PHY_NO_OP_REQ) {

		/*
		 * Upon completion of HARD RESET, we need to initialize
		 * OOB registers to enable hot-plug timer.
		 */
		asd_hwi_build_control_phy(scb, phy, PHY_NO_OP);

		asd_push_post_stack(asd, scb, scb->io_ctx,
			asd_ctl_issue_phy_ctl_done);
		scb->flags = (SCB_INTERNAL | SCB_ACTIVE | SCB_CSMI);
		asd_hwi_post_scb(asd, scb);
	} else {
		if ((scb->flags & SCB_ACTIVE) == 0) {
			asd_print("SCB %d done'd twice\n", SCB_GET_INDEX(scb));
			panic("Stopping for safety");
		}
		list_del(&scb->owner_links);
		if ((port != NULL) && (!list_empty(&port->targets))) {
			/* Unfreeze all the targets. */
			list_for_each_entry(targ, &port->targets, 
					all_domain_targets) {
				targ->qfrozen--;
			}
		}
		/* TBD: pause the LSeq if oprn is  DISABLE_PHY */
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		wake_up(&pcsmi_mgmt->waitq);
	}
		
	return;
}

static int asd_ctl_enter_pattern_gen(struct asd_softc *asd, 
	struct asd_phy	*phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer,
	int pattern_ctl_index)
{
	PCSMI_SAS_PHY_CONTROL pcsmi_sas_phy_control;
	//PCSMI_SAS_CHARACTER pcsmi_sas_character;
	uint8_t *ppat_buf;
	uint32_t pat_len;
	int err;

	ppat_buf = NULL;
	pcsmi_sas_phy_control = 
		&pcsmi_sas_phy_control_buffer->Control[pattern_ctl_index];

	/* TBD: check for I/O and return error if busy */

	if (pcsmi_sas_phy_control->uPatternFlags 
	     & CSMI_SAS_PHY_FIXED_PATTERN) {

		if (pcsmi_sas_phy_control->bFixedPattern 
		   == CSMI_SAS_PHY_CJPAT ) {
			/* generate fixed CJTPAT data pattern */
			ppat_buf = sascjtpat;
			pat_len = sizeof(sascjtpat);
			err = asd_ctl_data_pat_gen(asd, phy,
					pcsmi_sas_phy_control_buffer,
					pattern_ctl_index,
					ppat_buf, pat_len);
		} else {
			if (pcsmi_sas_phy_control->bFixedPattern 
			   == CSMI_SAS_PHY_ALIGN) {
				/* generate fixed primitive pattern */
				err = asd_ctl_primitive_pat_gen(asd, phy,
						pcsmi_sas_phy_control_buffer);
			}
		}
	} else {
#if 0 
	/* TBD: no macro for USER_PATTERN in adp94xx_csmi.h */
		if (pcsmi_sas_phy_control->uPatternFlags 
		     & CSMI_SAS_PHY_USER_PATTERN) {
			ppat_buf = pcsmi_sas_phy_control->UserPatternBuffer;
			pat_len = pcsmi_sas_phy_control->bUserPatternLength;
		
			/* check if aligned at 4 byte boundary */
			if ((pat_len % 4) != 0) {
				return -EINVAL;
			}
			if (pat_len > 32) {
				/* CSMI buffer restriction */
				pat_len = 32;
			}
			err = asd_ctl_data_pat_gen(asd, phy,
					pcsmi_sas_phy_control_buffer,
					pattern_ctl_index,
					ppat_buf, pat_len);
		}
#endif
	}
	return 0;
}

static int asd_ctl_primitive_pat_gen(struct asd_softc *asd, 
	struct asd_phy	*phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer)
{
	uint8_t reg_cont8;

	/* prepare to enter pattern generation mode */
	if (asd_ctl_prep_pat_gen_mode(asd, phy,
			pcsmi_sas_phy_control_buffer) != 0) {
		return -EINVAL;
	}

	/* Turn off LOW_POWER_TX and RESET bit (to prevent phy enabling) */
	reg_cont8 = asd_hwi_swb_read_byte(asd, LmSEQ_OOB_REG(phy->id, 
					  PHY_CONTROL_0));
	reg_cont8 &= ~(PHY_LOWPWREN_TX|PHY_RESET);
	asd_hwi_swb_write_byte(asd, LmSEQ_OOB_REG(phy->id, 
			  PHY_CONTROL_0), reg_cont8);

	/* CSMI spec(v0.82) uses only ALIGN[1] for primitive pattern */
	asd_hwi_swb_write_byte(asd, 
			LmSEQ_OOB_REG(phy->id, OOB_XMIT), 
			TX_ENABLE|XMIT_ALIGN_1);
	
	/* Indicate that the phy is in pattern generation mode */
	phy->pat_gen = 1;
	return 0;	
}

static int asd_ctl_data_pat_gen(struct asd_softc *asd, 
	struct asd_phy	*phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer,
	int pattern_ctl_index,
	uint8_t *ppat_buf,
	uint32_t pat_len)
{
	PCSMI_SAS_PHY_CONTROL pcsmi_sas_phy_control;
	int32_t timer_tick;
	uint32_t buf_offset;
	uint32_t reg_cont32;
	uint16_t reg_cont16;
	uint8_t reg_cont8;

	pcsmi_sas_phy_control = 
		&pcsmi_sas_phy_control_buffer->Control[pattern_ctl_index];
	
	/* prepare to enter pattern generation mode */
	if (asd_ctl_prep_pat_gen_mode(asd, phy,
			pcsmi_sas_phy_control_buffer) != 0) {
		return -EINVAL;
	}

	/* Disable OOB and DWS enable */
	asd_hwi_swb_write_byte(asd, LmSEQ_OOB_REG(phy->id, OOB_SIG_GEN), 
			       STOP_DWS);

	/* Disable OOB transmission */
	asd_hwi_swb_write_byte(asd, LmSEQ_OOB_REG(phy->id, OOB_XMIT), 0);
	
	/* Disable header insertion.
	 * Enable device only transfer and transmit test mode 
	 */
	reg_cont16 = asd_hwi_swb_read_word(asd, LmMnDDMAMODE(phy->id, 
					   LSEQ_MODE_TX));
	reg_cont16 |= (LmXTEST | LmMnDISHDR | LmMnDMATYPE_DEVICE_ONLY_TX);
	asd_hwi_swb_write_word(asd, LmMnDDMAMODE(phy->id, LSEQ_MODE_TX), 
			       reg_cont16);

	/* 
	 * Load transmit data buffer with the specified data pattern.
	 * Initialize DATABUFADR to 0, DATABUFADR is auto-incremented 
	 * when DATABUF is written.
	 */
	asd_hwi_swb_write_word(asd, LmMnDATABUFADR(phy->id, LSEQ_MODE_TX), 0);

	for(buf_offset = 0; buf_offset < pat_len; buf_offset++) {
		asd_hwi_swb_write_byte(asd, LmMnDATABUF(phy->id, LSEQ_MODE_TX), 
				       *ppat_buf++);
	}

	/* Set transfer length to pattern length */	
	asd_hwi_swb_write_dword(asd, LmMnXFRCNT(phy->id, LSEQ_MODE_TX),	 
				pat_len);
	
	/* Set transmit length to pattern length */	
	asd_hwi_swb_write_dword(asd, LmM1XMTCNT(phy->id), pat_len);
	
	/* Set device transmit buffer length to pattern length */	
	reg_cont8 = asd_hwi_swb_read_byte(asd, 
			LmMnDPSEL(phy->id, LSEQ_MODE_TX));
	asd_hwi_swb_write_byte(asd, 
			LmMnDPSEL(phy->id, LSEQ_MODE_TX),
			LmMnDBYTECNT);

	/* DPACC register is 24 bits wide */
	asd_hwi_swb_write_word(asd, 
			LmMnDPACC(phy->id, LSEQ_MODE_TX),
			(uint16_t)(pat_len & 0xFFFF));
	asd_hwi_swb_write_byte(asd, 
			LmMnDPACC(phy->id, LSEQ_MODE_TX) + 2,
			(uint8_t)((pat_len >> 16) & 0xFF));

	/* restore contents */
	asd_hwi_swb_write_byte(asd, 
			LmMnDPSEL(phy->id, LSEQ_MODE_TX),
			reg_cont8);

	/* Start data DMA */
	asd_hwi_swb_write_byte(asd, 
			LmMnDDMACTL(phy->id, LSEQ_MODE_TX),
			LmMnSTARTDAT);
	
	/* Set Phy transmit params as specified by user */
	asd_hwi_swb_write_byte(asd, LmSEQ_OOB_REG(phy->id, PHY_CONTROL_3),
			       phy->phy_ctl3);
	
	/* Start terminator calibration process */
	asd_hwi_swb_write_byte(asd, LmSEQ_OOB_REG(phy->id, PHY_CONTROL_4),
			       PHY_START_CAL);
	
	/* Poll for completion of terminator calibration */
	timer_tick = ASD_REG_TIMEOUT_TICK;
	do { 
		reg_cont8 = asd_hwi_swb_read_byte(asd, 
				LmSEQ_OOB_REG(phy->id, PHY_CONTROL_4));
		if (reg_cont8 & (PHY_DONE_CAL_TX | PHY_DONE_CAL_RX)) {
			timer_tick = 0;
			break;
		} else {
			timer_tick--;
			asd_delay(ASD_DELAY_COUNT);	
		}
	} while (timer_tick > 0);					
	
	if (timer_tick < 0) {
		/* Timer expired */
		return -EINVAL;
	}

	/* Set the Phy data rate as specified by user */
	reg_cont8 = asd_hwi_swb_read_byte(asd, LmSEQ_OOB_REG(phy->id, 
					  OOB_MODE));
	if (phy->max_link_rate == SAS_30GBPS_RATE
	   || phy->min_link_rate == SAS_30GBPS_RATE) {
		reg_cont8 |= PHY_SPEED_30;
	}
	if (phy->max_link_rate == SAS_15GBPS_RATE
	   || phy->min_link_rate == SAS_15GBPS_RATE) {
		reg_cont8 |= PHY_SPEED_15;
	}
	asd_hwi_swb_write_byte(asd, LmSEQ_OOB_REG(phy->id, 
				  OOB_MODE), reg_cont8);

	/* If B0 chip then disable Align output if requested by user */
	if ((pcsmi_sas_phy_control->uPatternFlags 
	     & CSMI_SAS_PHY_DISABLE_ALIGN) 
	   && (asd->hw_profile.rev_id == AIC9410_DEV_REV_B0)) {
		reg_cont8 = asd_hwi_swb_read_byte(asd, 
					LmM1ALIGNMODE(phy->id));
		reg_cont8 |= LmDISALIGN;
		asd_hwi_swb_write_byte(asd, LmM1ALIGNMODE(phy->id),
					reg_cont8);
	}
	
	/* Disable random scrambled data output if requested by user */
	if (pcsmi_sas_phy_control->uPatternFlags 
	     & CSMI_SAS_PHY_DISABLE_SCRAMBLING) {
		reg_cont32 = asd_hwi_swb_read_dword(asd, 
					LmDBGMODE(phy->id));
		reg_cont32 |= LmDSBLSCR;
		asd_hwi_swb_write_dword(asd, LmDBGMODE(phy->id),
					reg_cont32);
	}
	
	/* Turn off LOW_POWER_TX and RESET bit (to prevent phy enabling) */
	reg_cont8 = asd_hwi_swb_read_byte(asd, LmSEQ_OOB_REG(phy->id, 
					  PHY_CONTROL_0));
	reg_cont8 &= ~(PHY_LOWPWREN_TX|PHY_RESET);
	asd_hwi_swb_write_byte(asd, LmSEQ_OOB_REG(phy->id, 
			  PHY_CONTROL_0), reg_cont8);
	
	/* Turn on transmitter */
	asd_hwi_swb_write_byte(asd, LmSEQ_OOB_REG(phy->id, OOB_XMIT), 
				TX_ENABLE);

	/* Indicate that the phy is in pattern generation mode */
	phy->pat_gen = 1;

	return 0;
}

static int asd_ctl_prep_pat_gen_mode(struct asd_softc *asd,
				     struct asd_phy *phy,
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer)
{
	if (phy->state != ASD_PHY_OFFLINE) {
		/* If Phy is enabled, disable it before pausing */
		if (asd_ctl_issue_phy_ctl(asd, phy,
				&pcsmi_sas_phy_control_buffer->IoctlHeader,
				DISABLE_PHY) != 0) {
			return -EINVAL;
		}
	}

	/* Check if the lseq is already in pattern generation mode */	
	asd_ctl_switch_pat_gen_mode(asd, phy);
	
	if (asd_hwi_pause_lseq(asd, (uint8_t)(1 << phy->id)) != 0) {
		return -EINVAL;
	}

	return 0;
}

static int asd_ctl_switch_pat_gen_mode(struct asd_softc *asd,
				       struct asd_phy *phy)
{
	uint32_t reg_cont32;
	
	/* 
	 * TBD: For now use phy->pat_gen flag to ascertain if we are already 
	 * in pattern generation mode.
	 */
	if (phy->pat_gen == 1) {
		/* 
		 * If already in pattern generation state switch 
		 * to normal mode 
		 */
		reg_cont32 = LmBLKRST_COMBIST(phy->id);
		asd_write_dword(asd, COMBIST, reg_cont32);
		
		/* Intialize Mode n Link m Interrupt Enable */
		asd_hwi_swb_write_dword(asd, CMnINTEN(phy->id), EN_CMnRSPMBXF);

		/* Initialize Mode n Request Mailbox */
		asd_hwi_swb_write_dword(asd, CMnREQMBX(phy->id), 0);

		/* Indicate that the phy is out of pattern generation mode */
		phy->pat_gen = 0;

		return 0;
	}

	return -1;
}


static int 
asd_ctl_process_ssp_resp(struct asd_softc *asd, struct scb *scb, 
			     struct asd_done_list *done_listp,
			     PCSMI_SAS_SSP_PASSTHRU_STATUS 
			     pcsmi_sas_ssp_passthru_status)
{
	int err;

	err = SCB_EH_FAILED;
	switch (done_listp->opcode) {
	case TASK_COMP_WO_ERR:
	case TASK_COMP_W_UNDERRUN:
		pcsmi_sas_ssp_passthru_status->bConnectionStatus =
							CSMI_SAS_OPEN_ACCEPT;
		scb->eh_state = SCB_EH_SUCCEED;
		err = SCB_EH_SUCCEED;
		break;

	case TASK_F_W_NAK_RCVD:
		scb->eh_state = SCB_EH_FAILED;
		break;

	case SSP_TASK_COMP_W_RESP:
	{
		union edb 		*edb;
		struct ssp_resp_edb 	*redb;
		struct scb 		*escb;
		struct ssp_resp_iu 	*riu;
		u_int 			 edb_index;
		
		scb->eh_state = SCB_EH_FAILED; /* set it by default */
		edb = asd_hwi_get_edb_from_dl(asd, scb, done_listp, 
					&escb, &edb_index);
		if (edb == NULL) {
			asd_log(ASD_DBG_ERROR, "Invalid EDB recv for SSP "
				"comp w/response.\n");
			break;
		}

		redb = &edb->ssp_resp;
		riu = &redb->resp_frame.riu;
		if (SSP_RIU_DATAPRES(riu)) {
			asd_ctl_copy_ssp_resp_data(riu, pcsmi_sas_ssp_passthru_status);
		} else {
			/* Response Data not available. Protocol error. */
		}

		asd_hwi_free_edb(asd, escb, edb_index);
		break;
	}

	case TASK_F_W_OPEN_REJECT:
	{
		struct open_reject_sb	*open_reject;
		int			 abandon;

		open_reject = &done_listp->stat_blk.open_reject;
		abandon = ((open_reject->abandon_open & ABANDON_OPEN_MASK) 
				? 1 : 0);

		asd_ctl_map_open_reject_to_csmi(
			open_reject->reason, abandon,
			&pcsmi_sas_ssp_passthru_status->bConnectionStatus);

		break;
	}

	default:
		break;		
	}
	
	return err;
}

static int
asd_ctl_issue_link_adm(struct asd_softc *asd,
	struct asd_port	*port, 
	PCSMI_SAS_LINK_ERRORS_BUFFER pcsmi_sas_link_errors_buffer,
	uint8_t sub_func)
{
/* 
 * As the csmi link error ioctl requires the response data 
 * after issuing an link error request, this routine posts the scb and 
 * waits for it to complete. On completion it copies the return
 * status/data to user. If a new request is received when there
 * are outstanding requests still pending, the new request
 * would be put in Q waiting to acquire the semaphore.
 */
	PCSMI_SAS_LINK_ERRORS pcsmi_sas_link_errors;
	PIOCTL_HEADER pioctl_header;
	struct asd_link_admin_hscb *link_adm_hscb;
	struct scb *scb;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	int err;
	int active_timer;
	u_long flags;
	
	pcsmi_sas_link_errors = &pcsmi_sas_link_errors_buffer->Information;
	pioctl_header = &pcsmi_sas_link_errors_buffer->IoctlHeader;

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;

	down_interruptible(&pcsmi_mgmt->sem);
	if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
				&pcsmi_mgmt->busy)) { 
		/* unlikely */
		up(&pcsmi_mgmt->sem);
		return -EBUSY;
	}
	
	asd_lock(asd, &flags);
	// Get an scb to use.
	if ((scb = asd_hwi_get_scb(asd, 1)) == NULL) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		up(&pcsmi_mgmt->sem);
		asd_unlock(asd, &flags);
		return -ENOMEM;
	}

	scb->flags |= (SCB_INTERNAL | SCB_CSMI);
	list_add_tail(&scb->owner_links, &asd->platform_data->pending_os_scbs);
	
	link_adm_hscb = &scb->hscb->link_admin;
	link_adm_hscb->header.opcode = SCB_LINK_ADMIN_TASK;
	link_adm_hscb->phy_id = pcsmi_sas_link_errors->bPhyIdentifier;
	link_adm_hscb->sub_func = sub_func;
	link_adm_hscb->conn_handle = 0xffff;
	
	asd_push_post_stack(asd, scb, (void *)pcsmi_sas_link_errors_buffer,
		asd_ctl_issue_link_adm_done);

	scb->flags |= SCB_ACTIVE;
	asd_hwi_post_scb(asd, scb);
	asd_unlock(asd, &flags);
	
	/* start timer */	
	asd_ctl_start_csmi_scb_timer(&pcsmi_mgmt->timer,
		ASD_CSMI_TIMEOUT(pioctl_header),
		scb, asd_ctl_csmi_timeout);

	wait_event(pcsmi_mgmt->waitq,
		  !test_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy));

	/* delete timer */	
	active_timer = del_timer_sync(&pcsmi_mgmt->timer);

	if (active_timer) {
		err = 0;
	} else {
		ASDDBG1(("adp94xx:Request Timed out:" 
			 "CSMI Link Admin command\n"));
		err = -ETIME;
#ifdef REMOVE_WHEN_ERROR_RECOVERY_FIXED
		list_del(&scb->hwi_links);
		list_del(&scb->owner_links);
#endif
	}

	up(&pcsmi_mgmt->sem);
	return err;
}

static void
asd_ctl_issue_link_adm_done(struct asd_softc *asd, 
	struct scb *scb,
	struct asd_done_list *done_listp)
{
	PCSMI_SAS_LINK_ERRORS_BUFFER pcsmi_sas_link_errors_buffer;
	PCSMI_SAS_LINK_ERRORS pcsmi_sas_link_errors;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	
	if ((scb->flags & SCB_ACTIVE) == 0) {
		asd_print("SCB %d done'd twice\n", SCB_GET_INDEX(scb));
		panic("Stopping for safety");
	}

	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	list_del(&scb->owner_links);
	//if ((scb->flags & SCB_TIMEDOUT) != 0)
	//	list_del(&scb->timedout_links);

	pcsmi_sas_link_errors_buffer = 
			(PCSMI_SAS_LINK_ERRORS_BUFFER)scb->io_ctx;

	pcsmi_sas_link_errors = &pcsmi_sas_link_errors_buffer->Information;

	switch (done_listp->opcode) {
	case TASK_COMP_WO_ERR:
		break;

	case LINK_ADMIN_TASK_COMP_W_RESP:
	{
		union edb *edb;
		struct link_err_cnt_edb *redb;
		struct scb *escb;
		uint32_t edb_index;

		edb = asd_hwi_get_edb_from_dl(asd, scb, done_listp, 
					&escb, &edb_index);
		if (edb == NULL) {
			asd_log(ASD_DBG_ERROR, "Invalid EDB recv for SSP "
				"comp w/response.\n");
			break;
		}
		redb = &edb->link_err_cnt;

		pcsmi_sas_link_errors->uInvalidDwordCount = 
						redb->inv_dword_cnt;
		pcsmi_sas_link_errors->uRunningDisparityErrorCount = 
						redb->disparity_err_cnt;
		pcsmi_sas_link_errors->uLossOfDwordSyncCount = 
						redb->loss_of_sync_cnt;
		pcsmi_sas_link_errors->uPhyResetProblemCount = 
						redb->phy_reset_cnt;
		
		asd_hwi_free_edb(asd, escb, edb_index);
		break;
	}

	default:
		break;		
	}
	
	/* free scb */	
	asd_hwi_free_scb(asd, scb);
	
	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
	wake_up(&pcsmi_mgmt->waitq);
	
	return;
}

/* 
 * This routine wraps the SCB built for the SSP/STP pass thru CSMI command 
 * on to a scsi_cmnd struct and queues it to the device's queue.
 * The commands in the queue are fired by the xxx_flush_device_queue.
 */
static int
asd_ctl_queue_cmd(struct asd_softc *asd, 
	struct scb *scb, struct asd_target *targ, struct asd_device *dev)
{
	struct scsi_cmnd *cmd;
	u_long flags;

	if ((cmd = asd_alloc_mem(sizeof(struct scsi_cmnd), 
				 GFP_ATOMIC)) == NULL) {
		return -ENOMEM;
	}

	/* indicate its an CSMI generated scsi_cmnd */
	cmd->sc_magic = ASD_CSMI_COMMAND;

	/* stick SCB into scsi_cmnd */
	cmd->host_scribble = (unsigned char *)scb;

	asd_lock(asd, &flags);
	list_add_tail(&((union asd_cmd *)cmd)->acmd_links, &dev->busyq);
	if ((dev->flags & ASD_DEV_ON_RUN_LIST) == 0) {
		list_add_tail(&dev->links, &asd->platform_data->device_runq);
		dev->flags |= ASD_DEV_ON_RUN_LIST;
	
		asd_run_device_queues(asd);
	}
	asd_unlock(asd, &flags);

	return 0;
}

static int
asd_ctl_check_phy_port(struct asd_softc *asd, 
		uint8_t phy_id, uint8_t port_id, struct asd_phy **phy, 
		struct asd_port **port)
{
	struct asd_phy *temp_phy;
	struct asd_port *temp_port;
	int retcode;
	
	temp_phy = NULL;
	temp_port = NULL;	
	if ((phy_id == CSMI_SAS_USE_PORT_IDENTIFIER)
	  && (port_id == CSMI_SAS_IGNORE_PORT)) {
		return CSMI_SAS_SELECT_PHY_OR_PORT;
	}
	
	if (phy_id != CSMI_SAS_USE_PORT_IDENTIFIER) {
		/*TBD: CSMI_SAS_PHY_CANNOT_BE_SELECTED */
		temp_phy = asd_ctl_get_phy(asd, phy_id);
		if (temp_phy == NULL) {
			return CSMI_SAS_PHY_DOES_NOT_EXIST;
		}
	}
	
	if (port_id != CSMI_SAS_IGNORE_PORT) {
		/*TBD: CSMI_SAS_PORT_CANNOT_BE_SELECTED */
		temp_port = asd_ctl_get_port(asd, port_id);
		if (temp_port == NULL) {
			return CSMI_SAS_PORT_DOES_NOT_EXIST;
		}
	}

	retcode = 0;
	if (((phy_id != CSMI_SAS_USE_PORT_IDENTIFIER)
	    && (port_id != CSMI_SAS_IGNORE_PORT)) 
	  || (port_id == CSMI_SAS_IGNORE_PORT)) {
	
		if (temp_phy == NULL || temp_phy->src_port == NULL) {
			retcode = CSMI_SAS_STATUS_INVALID_PARAMETER;
		} else {
			
			if (port_id == CSMI_SAS_IGNORE_PORT) {
				temp_port = temp_phy->src_port;
			}
			
			if (temp_port == NULL) {
				retcode = CSMI_SAS_STATUS_INVALID_PARAMETER;
			} else {
				if (temp_phy->src_port->id != temp_port->id) {
					retcode = 
					      CSMI_SAS_PHY_DOES_NOT_MATCH_PORT;
				}
			}
		}
		
		if (retcode) {
			return retcode;
		}	
	}

	if (temp_port != NULL) {
		*port = temp_port;
		return 0;
	}

	return CSMI_SAS_STATUS_INVALID_PARAMETER;
}

static int
asd_ctl_pc_map_phy_settings(struct asd_phy *phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer,
	asd_attrib attr, 
	int link_rate,
	int ctl_index)
{
	PCSMI_SAS_PHY_CONTROL pcsmi_sas_phy_control;
	uint8_t phy_ctl0, phy_ctl1, phy_ctl2, phy_ctl3;
	uint8_t pc_sas_sata;
	uint8_t det_lvl_mask, shift;

	if (ctl_index >= pcsmi_sas_phy_control_buffer->bNumberOfControls) {
		return -EINVAL;
	}
	pcsmi_sas_phy_control = 
		&pcsmi_sas_phy_control_buffer->Control[ctl_index];

	asd_ctl_map_dev_type_to_csmi(attr, &pcsmi_sas_phy_control->bType);
	asd_ctl_map_link_rate_to_csmi(link_rate, &pcsmi_sas_phy_control->bRate);
	
	phy_ctl0 = phy->phy_ctl0;
	phy_ctl1 = phy->phy_ctl1;
	phy_ctl2 = phy->phy_ctl2;
	phy_ctl3 = phy->phy_ctl3;

	/* default: ASD_SAS_MODE */
	pc_sas_sata = phy_ctl3;
	det_lvl_mask = SAS_PHY_DETLEVEL_MASK;
	shift = PC_SAS_DETLVL_SHIFT;

	if (attr & ASD_SATA_MODE) {
		/* change to ASD_SATA_MODE */
		pc_sas_sata = phy_ctl2;
		det_lvl_mask = SATA_PHY_DETLEVEL_MASK;
		shift = PC_SATA_DETLVL_SHIFT;
	}
	//pcsmi_sas_phy_control->uVendorUnique;
	//pcsmi_sas_phy_control->uTransmitterFlags;/* TBD: OOB spec says TBD */
	asd_ctl_get_pc_tx_drvamp(pc_sas_sata,
			      &pcsmi_sas_phy_control->bTransmitAmplitude);
	asd_ctl_get_pc_preemp(pc_sas_sata,
			      &pcsmi_sas_phy_control->bTransmitterPreemphasis);
	asd_ctl_get_pc_slewrate(pc_sas_sata,
				&pcsmi_sas_phy_control->bTransmitterSlewRate);

	//pcsmi_sas_phy_control->bTransmitterReserved[13];
	//pcsmi_sas_phy_control->bTransmitterVendorUnique[64];
	//pcsmi_sas_phy_control->uReceiverFlags;
	asd_ctl_get_pc_detlvl(phy_ctl1, det_lvl_mask, shift,
			      &pcsmi_sas_phy_control->bReceiverThreshold);
	//pcsmi_sas_phy_control->bReceiverEqualizationGain;
	//pcsmi_sas_phy_control->bReceiverReserved[14];
	//pcsmi_sas_phy_control->bReceiverVendorUnique[64];
	/*
	pcsmi_sas_phy_control->uPatternFlags;
	pcsmi_sas_phy_control->bFixedPattern;
	pcsmi_sas_phy_control->bUserPatternLength;
	pcsmi_sas_phy_control->UserPatternBuffer[16];
	*/
	return 0;
}

static int
asd_ctl_pc_get_user_pc_val(struct asd_softc *asd, 
	struct asd_phy *phy, 
	PCSMI_SAS_PHY_CONTROL_BUFFER pcsmi_sas_phy_control_buffer)
{
	PCSMI_SAS_PHY_CONTROL pcsmi_sas_phy_control;
	uint8_t attr; 
	uint32_t phy_ctl_reg;
	u_int link_rate;
	int ctl_index;
	uint8_t *pc_sas3_sata2;
	uint8_t det_lvl_mask, det_lvl_shift;

	ctl_index = 0;
	for (ctl_index = 0; 
	     ctl_index < pcsmi_sas_phy_control_buffer->bNumberOfControls;
	     ctl_index++) {
		pcsmi_sas_phy_control = 
			&pcsmi_sas_phy_control_buffer->Control[ctl_index];

		asd_ctl_map_dev_type_from_csmi(pcsmi_sas_phy_control->bType, 
						&attr);

		asd_ctl_map_link_rate_from_csmi(pcsmi_sas_phy_control->bRate, 
						&link_rate);
		if (phy->max_link_rate > link_rate)
			phy->max_link_rate = link_rate;

		if (phy->max_link_rate < link_rate)
			phy->max_link_rate = link_rate;

		/* (default: ASD_SAS_MODE)  */
		pc_sas3_sata2 = &phy->phy_ctl3;
		det_lvl_mask = SAS_PHY_DETLEVEL_MASK;
		det_lvl_shift = PC_SAS_DETLVL_SHIFT;
		phy_ctl_reg = PHY_CONTROL_3;

		if (attr & ASD_SATA_MODE) {
			pc_sas3_sata2 = &phy->phy_ctl2;
			det_lvl_mask = SATA_PHY_DETLEVEL_MASK;
			det_lvl_shift = PC_SATA_DETLVL_SHIFT;
			phy_ctl_reg = PHY_CONTROL_2;
		}
		//pcsmi_sas_phy_control->uVendorUnique;
		//pcsmi_sas_phy_control->uTransmitterFlags;
		asd_ctl_set_pc_reg(pc_sas3_sata2, 
				   pcsmi_sas_phy_control->bTransmitAmplitude,
				   SAS_PHY_DRV_MASK,
				   PC_PHY_DRV_SHIFT,
				   0, PC_PHY_DRV_MAX_VAL);
		asd_ctl_set_pc_reg(pc_sas3_sata2, 
				   pcsmi_sas_phy_control->bTransmitterPreemphasis,
				   SAS_PREEMP_MASK,
				   PC_PREEMP_SHIFT,
				   0, PC_PREEMP_MAX_VAL);
		asd_ctl_set_pc_reg(pc_sas3_sata2, 
				   pcsmi_sas_phy_control->bTransmitterSlewRate,
				   SAS_SLEW_MASK,
				   PC_SLEW_SHIFT, 0, PC_SLEW_MAX_VAL);

		//pcsmi_sas_phy_control->bTransmitterReserved[13];
		//pcsmi_sas_phy_control->bTransmitterVendorUnique[64];
		//pcsmi_sas_phy_control->uReceiverFlags;
		asd_ctl_set_pc_reg(&phy->phy_ctl1, 
				   pcsmi_sas_phy_control->bReceiverThreshold,
				   det_lvl_mask,
				   det_lvl_shift, 0, PC_SLEW_MAX_VAL);
		//pcsmi_sas_phy_control->bReceiverEqualizationGain;
		//pcsmi_sas_phy_control->bReceiverReserved[14];
		//pcsmi_sas_phy_control->bReceiverVendorUnique[64];
		/*
		pcsmi_sas_phy_control->uPatternFlags;
		pcsmi_sas_phy_control->bFixedPattern;
		pcsmi_sas_phy_control->bUserPatternLength;
		pcsmi_sas_phy_control->bPatternReserved[6];
		pcsmi_sas_phy_control->UserPatternBuffer[16];
		*/

		asd_hwi_swb_write_byte(asd, 
			LmSEQ_OOB_REG(phy->id, phy_ctl_reg),
			*pc_sas3_sata2);

		asd_hwi_swb_write_byte(asd, 
			LmSEQ_OOB_REG(phy->id, PHY_CONTROL_1),
			phy->phy_ctl1);
	}
	return 0;
}

static int asd_ctl_copy_map_user_data_buf(struct asd_softc *asd, 
		unsigned long arg, struct scb *scb, uint8_t **preq_buf, 
		dma_addr_t *paddr, size_t *psg_copy_size, uint32_t data_len, 
		uint32_t user_buf_offset, uint32_t pci_data_dir)
{	
	dma_addr_t addr;
	struct sg_element *sg;
	uint8_t *request_buffer;
	size_t sg_copy_size;
	int err;

	scb->sg_count = 0;
	sg = scb->sg_list;
	if ((request_buffer = (uint8_t *)asd_alloc_mem( data_len, 
				GFP_ATOMIC)) == NULL) {
		return -ENOMEM;
	}
	memset(request_buffer, 0, data_len);

	if (pci_data_dir == PCI_DMA_TODEVICE 
	   || pci_data_dir == PCI_DMA_BIDIRECTIONAL) {
		if ((err = copy_from_user(request_buffer, 
				 (void *)arg + user_buf_offset,
				 data_len))) {
			asd_free_mem(request_buffer);
			return err;
		}
	}

	addr = 0;
	addr = asd_map_single(asd, request_buffer, data_len, pci_data_dir);

	if (!addr) {
		asd_free_mem(request_buffer);
		return -ENOMEM;
	}
	scb->platform_data->buf_busaddr = addr;
	err = asd_sg_setup(sg, addr, data_len, /*last*/1);

	if (err != 0) {
		asd_unmap_single(asd, addr, data_len, pci_data_dir);
		asd_free_mem(request_buffer);
		return -ENOMEM;
	}
	scb->sg_count = 1;
	sg_copy_size = scb->sg_count * sizeof(*sg);

	*paddr = addr;
	*preq_buf = request_buffer;
	*psg_copy_size = sg_copy_size;

	return 0;
}

static struct asd_nd_phydesc_format * 
asd_ctl_get_node_phyd(uint8_t *pnode_buf, 
		      uint32_t node_buf_size,
		      int phy_id) 
{
/* 
 * This routine parses thro' all node_descriptors and their associated 
 * phy_descriptors until it finds out a phy_descriptor that matches the 
 * given phy_id (corresponding phy_cnt)
 */
	struct asd_node_desc_format *pnode_desc;
	struct asd_nd_phydesc_format *pnd_phy_desc;
	uint32_t node_offset;
	int phy_cnt, poff_diff;
	
	node_offset = 0;
	phy_cnt = 0;

	while (node_offset <= node_buf_size) {
		pnode_desc = (struct asd_node_desc_format *)(pnode_buf 
				+ node_offset);
		phy_cnt += pnode_desc->num_phy_desc; 
		pnd_phy_desc = (struct asd_nd_phydesc_format *)
				((uint8_t *)pnode_desc + sizeof(*pnode_desc));
		if (pnode_desc->num_phy_desc > phy_id) {
			return &pnd_phy_desc[phy_id];
		}
		if (phy_cnt > phy_id) {
			poff_diff = phy_cnt - phy_id;
			return &pnd_phy_desc[pnode_desc->num_phy_desc 
					    - poff_diff];
		}
		node_offset += sizeof(*pnode_desc)
			      + (pnode_desc->num_phy_desc
				 * pnode_desc->phy_desc_size);
	}
	return NULL;
}

#ifdef ASD_ADPT_CSMI_EXTENSION
static int asd_ctl_write_to_nvram(struct asd_softc *asd,
	PCSMI_SAS_WRITE_NV_SEGMENT_BUFFER pcsmi_sas_write_nv_seg_buffer,
	unsigned long arg,
	uint8_t **psource_buffer)
{
	PCSMI_SAS_WRITE_NV_SEGMENT pcsmi_sas_write_nv_segment;
	PIOCTL_HEADER pioctl_header;
	u_long flags;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	uint32_t nv_segment_id, nv_offset;
	uint32_t pad_size, image_size, attr;
	uint32_t user_buf_offset/*, active_timer*/;
	int retcode;
	int err;
	
	pioctl_header = &pcsmi_sas_write_nv_seg_buffer->IoctlHeader;
	pcsmi_sas_write_nv_segment = 
				&pcsmi_sas_write_nv_seg_buffer->Information;
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	
	if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
			     &pcsmi_mgmt->busy)) { 
		return -EBUSY;
	}
	
	user_buf_offset = offsetof(CSMI_SAS_WRITE_NV_SEGMENT_BUFFER, 
				   bSourceBuffer);
	
	if ((err = asd_ctl_valloc_copy_buf(asd, psource_buffer, 
			pcsmi_sas_write_nv_segment->uBufferLength, 
			user_buf_offset, arg)) != 0) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		return err;
	}
	
	asd_lock(asd, &flags);
	err = 0;
	retcode = CSMI_SAS_STATUS_SUCCESS;
	switch (pcsmi_sas_write_nv_segment->uSegmentID) {
	case CSMI_SAS_SEGMENT_ID_MANUFACTURING_SECTOR0:
	{
		asd_ctl_map_nv_segid_from_csmi(
				pcsmi_sas_write_nv_segment->uSegmentID,
				&nv_segment_id);
		if ((err = asd_hwi_search_nv_segment(asd, nv_segment_id, 
				&nv_offset, &pad_size, &image_size, 
				&attr)) != 0) {
			err = -1;
			pcsmi_sas_write_nv_segment->uStatus = 
						CSMI_SAS_NV_FAILURE;
			break;
		}
		err = asd_ctl_write_to_flash(asd, psource_buffer, 
			nv_segment_id,
			pcsmi_sas_write_nv_segment->uDestinationOffset, 
			pcsmi_sas_write_nv_segment->uBufferLength);
		break;
	}
	case CSMI_SAS_SEGMENT_ID_FLASH0:
	{
		err = asd_ctl_write_to_flash(asd, psource_buffer, 
			NVRAM_NO_SEGMENT_ID,
			pcsmi_sas_write_nv_segment->uDestinationOffset, 
			pcsmi_sas_write_nv_segment->uBufferLength);
		break;
	}
	case CSMI_SAS_SEGMENT_ID_SEEPROM0:
	case CSMI_SAS_SEGMENT_ID_FLASH_DIRECTORY0:
	case CSMI_SAS_SEGMENT_ID_CTRL_A_USER_SETTINGS0:
	case CSMI_SAS_SEGMENT_ID_COMPATIBILITY_SECTOR0:
	default:
		pcsmi_sas_write_nv_segment->uStatus = 
					CSMI_SAS_NV_FAILURE;
		err = -EINVAL;
		break;
	}

	asd_unlock(asd, &flags);

	/* return */
	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
	return err;
}

static int asd_ctl_read_from_nvram(struct asd_softc *asd,
	PCSMI_SAS_READ_NV_SEGMENT_BUFFER pcsmi_sas_read_nv_seg_buffer,
	unsigned long arg,
	uint8_t **pdest_buffer)
{
	PCSMI_SAS_READ_NV_SEGMENT pcsmi_sas_read_nv_segment;
	PIOCTL_HEADER pioctl_header;
	u_long flags;
	struct asd_csmi_mgmt *pcsmi_mgmt;
	uint32_t nv_segment_id;
	int user_buf_offset/*, active_timer*/;
	int retcode;
	int err;
	
	pioctl_header = &pcsmi_sas_read_nv_seg_buffer->IoctlHeader;
	pcsmi_sas_read_nv_segment = 
				&pcsmi_sas_read_nv_seg_buffer->Information;
	pcsmi_mgmt = &asd->asd_ctl_csmi_internal.mgmt;
	
	if (test_and_set_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, 
			     &pcsmi_mgmt->busy)) { 
		return -EBUSY;
	}
	user_buf_offset = offsetof(CSMI_SAS_READ_NV_SEGMENT_BUFFER, 
				   bDestinationBuffer);

	
	if ((err = asd_ctl_valloc_copy_buf(asd, pdest_buffer, 
			pcsmi_sas_read_nv_segment->uBytesToRead, 
			user_buf_offset, arg)) != 0) {
		clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
		return err;
	}

	err = 0;
	retcode = CSMI_SAS_STATUS_SUCCESS;
	
	asd_lock(asd, &flags);
	switch (pcsmi_sas_read_nv_segment->uSegmentID) {
	case CSMI_SAS_SEGMENT_ID_CTRL_A_USER_SETTINGS0:
	case CSMI_SAS_SEGMENT_ID_MANUFACTURING_SECTOR0:
	{
		asd_ctl_map_nv_segid_from_csmi(
			pcsmi_sas_read_nv_segment->uSegmentID,
			&nv_segment_id);

		if (asd_hwi_read_nv_segment(asd, 
				nv_segment_id,
				(void *)*pdest_buffer,
				pcsmi_sas_read_nv_segment->uSourceOffset, 
				pcsmi_sas_read_nv_segment->uBytesToRead, 
				&pcsmi_sas_read_nv_segment->uBytesRead
				) != 0) {
			pcsmi_sas_read_nv_segment->uStatus = 
						CSMI_SAS_NV_FAILURE;
			err = -EINVAL;
			break;
		}
		pcsmi_sas_read_nv_segment->uStatus = CSMI_SAS_NV_SUCCESS;
		break;
	}
	case CSMI_SAS_SEGMENT_ID_FLASH_DIRECTORY0:
	{
		struct asd_flash_dir_layout flash_dir;
		uint32_t nv_addr;

		if (asd_hwi_search_nv_cookie(asd, &nv_addr, &flash_dir) != 0) {
			err = -1;
			break;
		}
		nv_addr += /*NVRAM_FIRST_DIR_ENTRY +*/ pcsmi_sas_read_nv_segment->uSourceOffset;

		if (asd_hwi_read_nv_segment(asd, NVRAM_NO_SEGMENT_ID,
				(void *)*pdest_buffer, nv_addr,
				pcsmi_sas_read_nv_segment->uBytesToRead, 
				&pcsmi_sas_read_nv_segment->uBytesRead
				) != 0) {
			err = -1;
			break;
		}
		break;
	}	
	case CSMI_SAS_SEGMENT_ID_FLASH0:
	{
		if (asd_hwi_read_nv_segment(asd, 
				NVRAM_NO_SEGMENT_ID,
				(void *)*pdest_buffer,
				pcsmi_sas_read_nv_segment->uSourceOffset, 
				pcsmi_sas_read_nv_segment->uBytesToRead, 
				&pcsmi_sas_read_nv_segment->uBytesRead
				) != 0) {
			pcsmi_sas_read_nv_segment->uStatus = 
						CSMI_SAS_NV_FAILURE;
			err = -EINVAL;
			break;
		}
		pcsmi_sas_read_nv_segment->uStatus = CSMI_SAS_NV_SUCCESS;
		break;
	}
	case CSMI_SAS_SEGMENT_ID_SEEPROM0:
	case CSMI_SAS_SEGMENT_ID_COMPATIBILITY_SECTOR0:/*TBD*/
	default:
		pcsmi_sas_read_nv_segment->uStatus = 
					CSMI_SAS_NV_FAILURE;
		err = -EINVAL;
		break;
	}
	asd_unlock(asd, &flags);

	/* return */
	clear_bit(ASD_CSMI_INTERNAL_BUSY_BIT_NR, &pcsmi_mgmt->busy);
	return err;
}

static uint32_t asd_ctl_write_to_flash(struct asd_softc *asd, 
			uint8_t **src_img_addr, uint32_t segment_id, 
			uint32_t dest_nv_offset, uint32_t src_img_size)
{
	uint32_t flash_addr;

	flash_addr = 0;

	/* Need to unlock MBAR key register */
	flash_addr = asd_pcic_read_dword(asd, PCIC_MBAR_KEY);
	if (flash_addr != 0) {
		/* currently locked, need to unlock */
		asd_pcic_write_dword(asd, PCIC_MBAR_KEY, flash_addr);
	}
	if (asd_hwi_write_nv_segment(asd, 
			(void *)*src_img_addr, segment_id, 
			dest_nv_offset, src_img_size) != 0) {
		return -EINVAL;
	}
	return 0;
}
#endif /* #ifdef ASD_ADPT_CSMI_EXTENSION */

